/*   
 * $Id: calc.c,v 2.53 2000/09/28 17:02:52 ondra Exp $
 * 
 * Scientific Calculator for Palms and PilmPilots.
 *   Copyright (C) 1999,2000 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *  You can contact me at 'ondrap@penguin.cz'.
 * 
 *  Antonio Fiol Bonnn" <Antonio.FIOL@enst-bretagne.fr> submitted patches 
 *   for the financial part.
*/

#include <PalmOS.h>
#include <StringMgr.h>
#include <stdio.h>

#include "defuns.h"
#include "MathLib.h"
#include "calc.h"
#include "calcDB.h"
#include "konvert.h"
#include "mathem.h"
#include "stack.h"
#include "funcs.h"
#include "result.h"
#include "ansops.h"
#include "varmgr.h"
#include "history.h"
#include "defmgr.h"
#include "calcrsc.h"    
#include "graph.h"
#include "finance.h"
#include "prefs.h"

#ifdef GRAPHS_ENABLED
 
#include "grprefs.h"
#include "grsetup.h"
#include "graph.h"

#endif /* GRAPH_ENABLED */

static void default_form_update(void);

void *GetObjectPtr(UInt16 objectID)
{
	FormPtr frm;
	
	frm=FrmGetActiveForm();
	return (FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,objectID)));
}

static char *historylist[MAX_HISTORY];

static void
PushHistory(char *text)
{
	int i;
	
	for (i=MAX_HISTORY-1;i>0;i--)
	  StrCopy(historylist[i],historylist[i-1]);
	StrCopy(historylist[0],text);
}

static void 
calc_init_application()
{
	int i;

	prefs_read_preferences();
	
	SetDisplayMode(calcPrefs.sciMode,calcPrefs.defFrac,
		       !calcPrefs.stripZeros,calcPrefs.reducePrecision);
/* Initialize the field for the list resource */
	for (i=0;i<MAX_HISTORY;i++)
	  historylist[i]=calcPrefs.history[i];
}

/* Change to the next screen */
void
GotHardKey(void)
{
	int formId;
	
	formId=(FrmGetActiveForm()->formId)+1;
#ifdef GRAPHS_ENABLED
	if (formId>tdFormGraph)	
	  formId=tdFormBasic;
#else
	if (formId>tdFormInt)
	  formId=tdFormBasic;
#endif
	ChangeForm(formId);
}

/* Generic button action - actions for buttons  on all screens */
Int16
chooseForm(Int16 button)
{
	Boolean handled = true;

	switch (button) {
	 case tdHelp2:
		FrmHelp(tdHelp2);
		break;
	 case tdAbout:
		FrmAlert(tdAlertAbout);
		break;
	 case tdBInt:
		ChangeForm(tdFormInt);
		break;
	 case tdBBasic:
		ChangeForm(tdFormBasic);
		break;
#ifdef GRAPHS_ENABLED		
	 case tdBGra:
		ChangeForm(tdFormGraph);
		break;
#endif		
	 case tdBSci:
		ChangeForm(tdFormScientific);
		break;
	 default:
		handled = false;
	}
	return handled;
}

static Err 
StartApplication(void)
{
	
	Err error;

	/* Initialize all modules */
	mlib_init_mathlib();
	
	error = db_open();
	ErrFatalDisplayIf(error, "Can't open CalcDB");
	
	error = history_open();
	ErrFatalDisplayIf(error, "Can't open History DB");
	
	calc_init_application();

#ifdef GRAPHS_ENABLED	
	grpref_init();
#endif	
	
	FrmGotoForm(calcPrefs.form);	

/* Set the floating 'point' to '.' or ',' */	
	switch (PrefGetPreference(prefNumberFormat)) {
	 case nfCommaPeriod:
	 case nfApostrophePeriod:
		SetFlPoint('.');
		break;
	 case nfPeriodComma:
	 case nfSpaceComma:
	 case nfApostropheComma:
		SetFlPoint(',');
		break;
	}
	return 0;
}

static void 
StopApplication(void)
{	
	FrmSaveAllForms();
	FrmCloseAllForms();

	/* close all modules */
	result_destroy();
	
	mlib_close_mathlib();
	
	prefs_save_preferences();
	
	db_close();
	
	history_close();
#ifdef GRAPHS_ENABLED	
	grpref_close();
#endif	
}

static void
ReplaceEnters(char *text)
{
	for (;*text;text++)
	  if (*text=='\n')
	    *text=';';
}
	  


static int 
CalculateIt()
{
	char *inp;
	CError err;
	FieldPtr pole;
	int inpsize;
	Trpn vysledek;
	CodeStack *stack;
	
	inp = FldGetTextPtr(GetObjectPtr(tdInput));
	if (!inp)
	  return 0;
	inpsize=StrLen(inp);
	
	ReplaceEnters(inp); /* PalmIII resets when '\n' in list, PalmPilot is
			     * ok. I wonder why... */
	stack = text_to_stack(inp,&err);
	if (!err) {
		err = stack_compute(stack);
		if (!err) {
			vysledek = stack_pop(stack);
			set_ans_var(vysledek);
			result_set(vysledek);
			rpn_delete(vysledek);
		}
		stack_delete(stack);
	}
	if (err) 
	  result_error(err);

	result_draw();

	pole=GetObjectPtr(tdInput);
	if (!err) {
		FldSetSelection(pole,0,inpsize);
		FrmSetFocus(FrmGetActiveForm(),FrmGetObjectIndex(FrmGetActiveForm(),tdInput));
	}
	pole=GetObjectPtr(tdInput);
	if (FldGetTextPtr(pole))
	  PushHistory(FldGetTextPtr(pole));
	return 0;
}

struct {
	UInt16 id;
	char *string;
	Boolean operator; /* Prepend Ans when in the beggining of line */
	Boolean parenth;  /* Add ')' and backspace one position */
}tdButStr[]={	
	{tdBut0,"0",false},
	{tdBut1,"1",false},
	{tdBut2,"2",false},
	{tdBut3,"3",false},
	{tdBut4,"4",false},
	{tdBut5,"5",false},
	{tdBut6,"6",false},
	{tdBut7,"7",false},
	{tdBut8,"8",false},
	{tdBut9,"9",false},
	{tdButA,"A",false},
	{tdButB,"B",false},
	{tdButC,"C",false},
	{tdButD,"D",false},
	{tdButE,"E",false},
	{tdButF,"F",false},
	{tdButAnd,"&",true},
	{tdButOr,"|",true},
	{tdButShl,"<",true},
	{tdButShr,">",true},
	{tdButDot,".",false},
	{tdButPow,"^",true},
	{tdButSqrt,"sqrt(",false,true},
	{tdButLn,"ln(",false,true},
	{tdButLog,"log(",false,true},
	{tdButPlus,"+",true},
	{tdButMinus,"-",true},
	{tdButMult,"*",true},
	{tdButDivide,"/",true},
	{tdButSin,"sin(",false,true},
	{tdButCos,"cos(",false,true},
	{tdButTan,"tan(",false,true},
	{tdButAcos,"acos(",false,true},
	{tdButAsin,"asin(",false,true},
	{tdButAtan,"atan(",false,true},
	{tdButSinh,"sinh(",false,true},
	{tdButCosh,"cosh(",false,true},
	{tdButTanh,"tanh(",false,true},
	{tdButAcosh,"acosh(",false,true},
	{tdButAsinh,"asinh(",false,true},
	{tdButAtanh,"atanh(",false,true},
	{tdButExp,"exp(",false,true},
	{tdButAbs,"abs(",false,true},
	{tdButFact,"fact(",false,true},
	{tdButLog2,"logb(",false,true},
	{tdButRound,"round(",false,true},
	{tdButTrunc,"trunc(",false,true},
	{tdButClBr,")",false},
	{tdButOpBr,"(",false},
	{tdButSqr,"^2",true},
	{tdButInv,"^(-1)",true},
	{tdButEE,"E",false},
	{tdButAns,"ans",false},
	{tdBute,"e",false},
	{tdButPi,"pi",false},
	{tdButSq1,"^(1/",true,true},
	{tdButX,"x",false},
	{tdButY,"y",false},
	{tdButQt,"\"",false},
	{tdButPths,"()",false},
	{tdButf,"f",false},
	{tdButEqual,"=",false},
	{0,NULL,false}
};

void ChangeForm(Int16 formID)
{
	if (FrmGetActiveForm()->formId==formID)
	  return;
	FrmGotoForm(formID);       
}

static void 
default_form_init(int formId)
{
	FieldPtr pole;
	FormPtr frm;
	
	frm = FrmGetActiveForm();

	result_init(resultGadget);
	result_draw();
	
	pole=GetObjectPtr(tdInput);
	FldInsert(pole,calcPrefs.input,StrLen(calcPrefs.input));
	FldSetInsPtPosition(pole,calcPrefs.insertPos);
	FrmSetFocus(frm,FrmGetObjectIndex(frm,tdInput));
	FldSetSelection(pole,calcPrefs.selPosStart,
					calcPrefs.selPosEnd);	
	
	LstSetListChoices(GetObjectPtr(tdHistList),historylist,
					  MAX_HISTORY);			
}

static void default_form_update(void)
{
	FormPtr frm;
	Tint_mode mode;
	
	frm=FrmGetActiveForm();
	switch (frm->formId) {
	 case tdFormInt:
		mode=calcPrefs.int_mode;
		CtlSetValue(GetObjectPtr(tdIntDec),mode==decimal);
		CtlSetValue(GetObjectPtr(tdIntOct),mode==octal);
		CtlSetValue(GetObjectPtr(tdIntBin),mode==binary);
		CtlSetValue(GetObjectPtr(tdIntHex),mode==hexa);
		CtlSetValue(GetObjectPtr(tdIntFlo),mode==floating);
		break;
	}
	result_draw();
}

void
wait_draw(void)
{
	RectangleType rec,frame;

	frame.topLeft.x = 50;
	frame.topLeft.y = 50;
	frame.extent.x = 60;
	frame.extent.y = 20;
	
	rec.topLeft.x = 48;
	rec.topLeft.y = 48;
	rec.extent.x = 64;
	rec.extent.y = 24;
	
	WinEraseRectangle(&rec,2);
	WinDrawRectangleFrame(boldRoundFrame,&frame);
	WinDrawChars("Please wait...",14,55,55);
						  
}

void
wait_erase(void)
{
	RectangleType rec;
		
	rec.topLeft.x = 48;
	rec.topLeft.y = 48;
	rec.extent.x = 64;
	rec.extent.y = 24;
	WinEraseRectangle(&rec,2);
}

static Boolean 
MainFormHandleEvent(EventPtr event)
{
	Boolean    handled = false;
	FormPtr     frm;
	FieldPtr pole;
	ListPtr  history;
	ControlPtr knoflik;
	char *text;
	int formId;
	int controlID;
	int i;
	
	  switch (event->eType) {
	   case popSelectEvent:
		  if (event->data.popSelect.controlID==tdHistory) {
			  history=event->data.popSelect.listP;
			  pole=GetObjectPtr(tdInput);
			  text=LstGetSelectionText(history,
						       event->data.popSelect.selection);
			  if (text==NULL) /* I didn't know this might happen */
				break;
			  if (FldGetTextPtr(pole))
				FldDelete(pole,0,StrLen(FldGetTextPtr(pole)));
			  FldInsert(pole,text,StrLen(text));
			  FrmSetFocus(FrmGetActiveForm(),FrmGetObjectIndex(FrmGetActiveForm(),tdInput));
			  handled=true;
		  }		
		  break;
	   case penDownEvent:
		  handled=result_track(event->screenX,event->screenY);
		  break;
	   case keyDownEvent:
		  if (TxtCharIsHardKey(event->data.keyDown.modifiers,event->data.keyDown.chr)) {
			  GotHardKey();
			  handled=true;
		  }
		  else if (event->data.keyDown.chr=='\n') {
			  CalculateIt();
			  handled=true;
		  }
		  break;
	   case frmCloseEvent:		 
		  formId=event->data.frmClose.formID;
		  
		  pole=GetObjectPtr(tdInput);
		  calcPrefs.insertPos=FldGetInsPtPosition(pole);
		  FldGetSelection(pole,&calcPrefs.selPosStart,
				  &calcPrefs.selPosEnd);
		  if (FldGetTextPtr(pole))
			StrCopy(calcPrefs.input,FldGetTextPtr(pole));
		  
		  handled=false; /* This is important - let the
				  * FrmHandleEvent handle this
				  * itself */
		  break;
	   case frmUpdateEvent:
		  if (event->data.frmUpdate.updateCode == frmUpdateVars) {
			  default_form_update();
			  handled = true;
		  }
		  if (event->data.frmUpdate.updateCode == frmRedrawUpdateCode)
			result_draw();
		  break;
	   case frmOpenEvent:		  
		  frm=FrmGetActiveForm();
		  formId=frm->formId;
		  
		  calcPrefs.form=formId;
		  
		  FrmDrawForm(frm);
		  
		  default_form_init(formId);
		  default_form_update();
		  
		  switch (formId) {
		   case tdFormBasic:
			  knoflik=GetObjectPtr(tdBBasic);
			  break;
		   case tdFormScientific:
			  knoflik=GetObjectPtr(tdBSci);
			  break;
		   case tdFormInt:
			  switch (calcPrefs.int_mode) {
			   case decimal:
				  i=tdIntDec;
				  break;
			   case octal:
				  i=tdIntOct;
				  break;
			   case binary:
				  i=tdIntBin;
				  break;
			   case hexa:
				  i=tdIntHex;
				  break;
			   case floating:
				  i=tdIntFlo;
				  break;
			   default:
				  i=0;
				  break;
			  }
			  if (i)
			    CtlSetValue(GetObjectPtr(i),1);
			  
			  knoflik=GetObjectPtr(tdBInt);
			  break;
		   default:
			  knoflik=NULL;
			  break;
		  }
		  CtlSetValue(knoflik,1);
								
		  handled = true;
		  break;
	   case ctlSelectEvent:
		  controlID=event->data.ctlSelect.controlID;

		  if (chooseForm(controlID)) {
			  handled=true;
			  break;
		  }
		  
		  handled=true;
		  switch (controlID) {
		   case calcResPopupBut: 
			  result_popup();
			  break;
		   case varFuncListButton:
			  text = varmgr_popup(function);
			  if (text) {
				  pole = GetObjectPtr(tdInput);
				  FldInsert(pole,text,StrLen(text));
				  FldInsert(pole,"(",1);
				  if (calcPrefs.matchParenth) {
				      if (FldInsert(pole,")",1))
						FldSetInsPtPosition(pole,FldGetInsPtPosition(pole)-1);
				  }
				  MemPtrFree(text);
			  }
			  break;
		   case varListButton:
		   case histListButton:
			  if (controlID==varListButton)
				text = varmgr_popup(variable);
			  else 
				text = history_popup();
			  if (text) {
				  pole = GetObjectPtr(tdInput);
				  FldInsert(pole,text,StrLen(text));
				  MemPtrFree(text);
			   }
			  break;
		   case tdIntBin:
		   case tdIntHex:
		   case tdIntDec:
		   case tdIntOct:
		   case tdIntFlo:
			  switch (controlID) {
			   case tdIntBin:
				  i=binary;
				  break;
			   case tdIntHex:
				  i=hexa;
				  break;
			   case tdIntOct:
				  i=octal;
				  break;
			   case tdIntDec:
				  i=decimal;
				  break;
			   case tdIntFlo:
				  i=floating;
				  break;
			   default:
				  i=0;
				  break;
			  }
			  calcPrefs.int_mode=i;
			  if (i==floating)
			    ans_redisplay("ans=real(ans)");
			  else
			    ans_redisplay("ans=int(round(ans))");
			  break;
		   case tdDoit:
			  CalculateIt();
			  break;
		   default:
			  handled=false;
		  }
		  if (handled)
		    break;
		  
		  if (controlID == tdClear) {
			  FieldPtr pole;
			  
			  pole=GetObjectPtr(tdInput);
			  FldDelete(pole,0,FldGetTextLength(pole));
			  handled=true;
			  break;
		  }		  
		  for (i=0;tdButStr[i].id;i++) 
		    if (tdButStr[i].id==event->data.ctlSelect.controlID) {
			    FieldPtr pole;
			    UInt16 startp, endp;

			    pole=GetObjectPtr(tdInput);
			    /* If we are on the beggining and 
			     * the user pressed operator, add Ans to 
			     * the beggining */
			    FldGetSelection(pole,&startp,&endp);
			    if (tdButStr[i].operator && 
				(FldGetInsPtPosition(pole)==0 || 
				(startp!=endp && startp==0)))
			      FldInsert(pole,"ans",StrLen("ans"));
			    /* If it is kind of function and
			     * '()' matching is enabled, then
			     * add the ')' and 
			     * backspace one position
			     */
			    if (FldInsert(pole,tdButStr[i].string,
					  StrLen(tdButStr[i].string))) 
			      if (calcPrefs.matchParenth && tdButStr[i].parenth) {
				      if (FldInsert(pole,")",1))
					FldSetInsPtPosition(pole,FldGetInsPtPosition(pole)-1);
			      }
			    
			    FrmSetFocus(FrmGetActiveForm(),
					FrmGetObjectIndex(FrmGetActiveForm(),
							  tdInput));
			    
			    handled=true;
			    break;
		    }		  
		  break;
	   case menuEvent:		  
		  if (chooseForm(event->data.menu.itemID)) {
			  handled=true;
			  break;
		  }
		  switch (event->data.menu.itemID) {
		   case tdDefMgr:
			  FrmPopupForm(defForm);
			  handled = true;
			  break;
		   case tdFinCalc:
			  FrmPopupForm(finForm);
			  handled = true;
			  break;
	 	   case tdBPref:
			  FrmPopupForm(prefForm);
			  handled=true;
			  break;
		  }
		  break;		    
	  }
	
	  return handled;
}

static Boolean 
ApplicationHandleEvent(EventPtr event)
{
	FormPtr  frm;
	Int16    formId;
	Boolean  handled = false;
	
	if (event->eType == frmLoadEvent) {
		//Load the form resource specified in the event then activate it
		formId = event->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);
		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmDispatchEvent each time it is called
		switch (formId) {
#ifdef GRAPHS_ENABLED			
		 case grSetupForm:
			FrmSetEventHandler(frm,GraphSetupHandleEvent);
			break;
		 case grPrefForm:
			FrmSetEventHandler(frm,GraphPrefsHandleEvent);
			break;			
		 case tdFormGraph:
			FrmSetEventHandler(frm,GraphFormHandleEvent);
			break;
#endif /* GRAPHS_ENABLED */
		 case prefForm:
			FrmSetEventHandler(frm,PreferencesHandleEvent);
			break;
		 case tdFormBasic:
		 case tdFormScientific:
		 case tdFormInt:
			FrmSetEventHandler(frm, MainFormHandleEvent);
			break;
		 case defForm:
			FrmSetEventHandler(frm, DefmgrHandleEvent);
			break;
		 case finForm:
			FrmSetEventHandler(frm, FinHandleEvent);
			break;
		 case varEditForm:
			FrmSetEventHandler(frm,VarmgrEditHandleEvent);
			break;
		 case varEntryForm:
			FrmSetEventHandler(frm,VarmgrEntryHandleEvent);
			break;
		}	
		handled = true;	       
	}
	
	return handled;
}

Int32 evtTimeout = evtWaitForever;

void
calc_nil_timeout(Int32 timeout)
{
	evtTimeout = timeout;
}

static void 
EventLoop(void)
{
	EventType  event;
	UInt16      error;
	
	do {
		EvtGetEvent(&event, evtTimeout);
		if (! SysHandleEvent(&event))
		  if (! MenuHandleEvent(0, &event, &error))
		    if (! ApplicationHandleEvent(&event))
		      FrmDispatchEvent(&event);
	} while (event.eType != appStopEvent);
}

UInt32
PilotMain(UInt16 launchCode, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err err=0;
	
	if (launchCode == sysAppLaunchCmdNormalLaunch) { 
		if ((err = StartApplication()) == 0) {
			EventLoop();
			StopApplication();
		}
	}
	else
	  return sysErrParamErr;
	
	return err;
}
