/*
 *   $Id: finance.c,v 2.24 2000/09/27 00:01:54 ondra Exp $
 * 
 *   Scientific Calculator for Palm's and PilmPilot's.
 *   Copyright (C) 2000 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *  You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>

#include "defuns.h"
#include "prefs.h"
#include "calc.h"
#include "stack.h"
#include "finance.h"
#include "calcDB.h"
#include "funcs.h"
#include "varmgr.h"

#include "calcrsc.h"

struct {
	UInt16 buttonID;
	UInt16 outputID;
	UInt16 inputID;
	char *var;
	char *description;
	char *begStr;
	char *endStr;
}inpFinForm[]= {
	  {finFVButton,finFVLabel,finFVSet,"fv","Future Value",
			"fv=-(pv*(1+(it/pyr/100))^n+pmt*(1+(it/pyr/100))*((1+(it/pyr/100))^n-1)/(it/pyr/100))",
			"fv=-(pv*(1+(it/pyr/100))^n+pmt*((1+(it/pyr/100))^n-1)/(it/pyr/100))"},
	  {finPVButton,finPVLabel,finPVSet,"pv","Present Value",
			"pv=(-fv-pmt*(1+(it/pyr/100))*((1+(it/pyr/100))^n-1)/(it/pyr/100))/(1+(it/pyr/100))^n",
			"pv=(-fv-pmt*((1+(it/pyr/100))^n-1)/(it/pyr/100))/(1+(it/pyr/100))^n"},
	  {finPMTButton,finPMTLabel,finPMTSet,"pmt","Payment",
			"pmt=(-fv-pv*(1+(it/pyr/100))^n)*((it/pyr/100)/((1+(it/pyr/100))^n-1)/(1+(it/pyr/100)))",
			"pmt=(-fv-pv*(1+(it/pyr/100))^n)*(it/pyr/100)/((1+(it/pyr/100))^n-1)"},
	  {finNButton,finNLabel,finNSet,"n","Number of payments",
			"n=log((-fv*(it/pyr/100)+pmt*(1+(it/pyr/100)))/(pv*(it/pyr/100)+pmt*(1+(it/pyr/100))))/log(1+(it/pyr/100))",
			"n=log((-fv*(it/pyr/100)+pmt)/(pv*(it/pyr/100)+pmt))/log(1+(it/pyr/100))"},
	  {finIButton,finILabel,finISet,"it","Interest",
			"it=fzero(0.1:1000:\"fv+(pv*(1+(x/pyr/100))^n+pmt*(1+(x/pyr/100))*((1+(x/pyr/100))^n-1)/(x/pyr/100))\"",
			"it=fzero(0.1:1000:\"fv+(pv*(1+(x/pyr/100))^n+pmt*((1+(x/pyr/100))^n-1)/(x/pyr/100))\""},
	  {finPYRButton,finPYRLabel,finPYRSet,"pyr","Number of payments per year",
			"pyr=fzero(0.1:366:\"fv+(pv*(1+(it/x/100))^n+pmt*(1+(it/x/100))*((1+(it/x/100))^n-1)/(it/x/100))\"",
			"pyr=fzero(0.1:366:\"fv+(pv*(1+(it/x/100))^n+pmt*((1+(it/x/100))^n-1)/(it/x/100))\""},
  {0,0,NULL,NULL}
};

/***********************************************************************
 *
 * FUNCTION:     fin_update_fields
 * 
 * DESCRIPTION:  Updates the onscreen representation of
 *               financial variables
 *
 * PARAMETERS:   None
 *
 * RETURN:       None
 *      
 ***********************************************************************/
static void fin_update_fields() IFACE;
static void
fin_update_fields()
{
	CError err;
	char text[MAX_FP_NUMBER+1];
	Trpn item;
	Int16 i;
	FormPtr frm = FrmGetActiveForm();
	
	for (i=0;inpFinForm[i].var!=NULL;i++) {
		item = db_read_variable(inpFinForm[i].var,&err);
		 if (err || item.type!=real)
		  StrCopy(text,"Undefined");
		 else
		  DoubleToAscii(item.u.realval,text);
		if (!err)
		  rpn_delete(item);
		/* Show result on screen */
		FrmHideObject(frm,FrmGetObjectIndex(frm,inpFinForm[i].outputID));
		FrmCopyLabel(frm,inpFinForm[i].outputID,text);
		FrmShowObject(frm,FrmGetObjectIndex(frm,inpFinForm[i].outputID));
	}
}

/***********************************************************************
 *
 * FUNCTION:     fin_compute
 * 
 * DESCRIPTION:  Computes a new value for a financial variable
 *
 * PARAMETERS:   pos - index into inpFinForm for a selected variable
 *               begin - the status of the Begin/End switch
 *
 * RETURN:       
 *      
 ***********************************************************************/
static void fin_compute(int pos,int begin) IFACE;
static void
fin_compute(int pos,int begin)
{
	CError err;
	CodeStack *stack;
	
	stack = text_to_stack(begin?inpFinForm[pos].begStr:inpFinForm[pos].endStr,
						  &err);
	
	if (err)
	  return;
	
	wait_draw();
	err = stack_compute(stack);
	stack_delete(stack);	

	if (err)
	  FrmAlert(tdAlertCompute);
	
	wait_erase();
	FrmUpdateForm(finForm,frmUpdateVars);
	FrmUpdateForm(finForm,frmRedrawUpdateCode);
	return;
}

static void
fin_update_callback(void) IFACE;
static void
fin_update_callback(void)
{
	FrmUpdateForm(finForm,frmUpdateVars);
}


/***********************************************************************
 *
 * FUNCTION:     FinHandleEvent
 * 
 * DESCRIPTION:  Event handler for financial calculator
 *               FinCalc always operates in non-complex,
 *               floating mode
 *
 * PARAMETERS:   event
 *
 * RETURN:       handled
 *      
 ***********************************************************************/
Boolean
FinHandleEvent(EventPtr event)
{
	Boolean handled = false;
	FormPtr frm;
	Int16 controlID;
	Int16 i;
	static Tint_mode oldIntMode;
	static Boolean oldForceComplex;
	
	switch (event->eType) {
	 case ctlSelectEvent:
		controlID = event->data.ctlSelect.controlID;
		if (controlID == finDoneButton) {
			calcPrefs.finBegin = CtlGetValue(GetObjectPtr(finBeginButton));
			FrmReturnToForm(0);
			calcPrefs.int_mode = oldIntMode;
			calcPrefs.forceComplex = oldForceComplex;
			handled = true;
			break;
		}
		for (i=0;inpFinForm[i].var!=NULL;i++) {
			if (inpFinForm[i].buttonID==controlID) {
				fin_compute(i,CtlGetValue(GetObjectPtr(finBeginButton)));
				fin_update_fields();
				handled = true;
				break;
			}
			else if (inpFinForm[i].inputID==controlID) {
				varmgr_edit(inpFinForm[i].var,
							inpFinForm[i].description,
							variable,
							fin_update_callback,false);
				handled = true;
				break;
			}
		}
		break;
	 case frmUpdateEvent:
		if (event->data.frmUpdate.updateCode == frmUpdateVars) {
			fin_update_fields();
			handled = true;
		}
		break;
	 case frmOpenEvent:
		oldIntMode = calcPrefs.int_mode;
		oldForceComplex = calcPrefs.forceComplex;
		calcPrefs.int_mode = floating;
		calcPrefs.forceComplex = false;		
		
		frm = FrmGetActiveForm();
		FrmDrawForm(frm);
		CtlSetValue(GetObjectPtr(calcPrefs.finBegin?finBeginButton:
								 finEndButton),1);
		fin_update_fields();
		handled = true;
		break;
	 case frmCloseEvent:
		/* It seems to me that DefaultBtnId doesn't work
		 * on application exit, this shouldn't be called */
		calcPrefs.finBegin = CtlGetValue(GetObjectPtr(finBeginButton));
		calcPrefs.forceComplex = oldForceComplex;
		calcPrefs.int_mode = oldIntMode;
		/* FrmHandleEvent must handle this event itself */
		handled = false;
		break;
	}
	
	return handled;
}
