/*
 *   $Id: grsetup.c,v 2.7 2000/09/28 22:17:53 ondra Exp $
 * 
 *   Scientific Calculator for Palms and PilmPilots.
 *   Copyright (C) 1999,2000 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *  You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>

#include <string.h>

#include "calcDB.h"
#include "konvert.h"
#include "grsetup.h"
#include "grprefs.h"
#include "varmgr.h"
#include "calcrsc.h"
#include "calc.h"

static Int16 firstVisible = 0;
static Int16 selectedRow;

static char *tblFuncs[] = {"Y1","Y2","Y3","Y4","Y5","Y6"};
static char *tblPols[] = {"r1","r2","r3","r4","r5","r6"};
static char *tblPars[] = {"X1","Y1","X2","Y2","X3","Y3","X4","Y4","X5","Y5","X6","Y6"};

enum {
	COL_FLIST = 2,
	COL_FUNC = 3
};

/***********************************************************************
 *
 * FUNCTION:     grsetup_def_name
 * 
 * DESCRIPTION:  Return name of function, that should be created specially
 *               for graphing purposes (the 'User' selection)
 *
 * PARAMETERS:   row - the row number in a grsetup table
 *
 * RETURN:       Pointer to static function name 
 *      
 ***********************************************************************/
char * grsetup_def_name(Int16 row) IFACE;
char *
grsetup_def_name(Int16 row)
{
	static char result[12];
	char tmp[2];
	
	StrCopy(tmp,"a");
	switch (graphPrefs.functype) {
	 case graph_func:
		StrCopy(result,"z_grafun");
		tmp[0]+=(char)row;
		StrCat(result,tmp);
		break;
	 case graph_polar:
		StrCopy(result,"z_grapol");
		tmp[0]+=(char)row;
		StrCat(result,tmp);
		break;
	 case graph_param:
		StrCopy(result,"z_grapar");
		StrCat(result,row%2?"b":"a");
		tmp[0]+=(char)row/2;
		StrCat(result,tmp);
		break;
	}
	return result;
}

/***********************************************************************
 *
 * FUNCTION:     grsetup_get_fname
 * 
 * DESCRIPTION:  Get a name of a function for the given row, that is
 *               already saved in Preferences
 *
 * PARAMETERS:   row - number of row in the displayed table
 *
 * RETURN:       !!! Pointer to a graphPrefs structure, so
 *               functions can modify the name directly
 *      
 ***********************************************************************/
char * grsetup_get_fname(Int16 row) IFACE;
char *
grsetup_get_fname(Int16 row)
{
	switch (graphPrefs.functype) {
	 case graph_func:
		return graphPrefs.funcFunc[row];
	 case graph_polar:
		return graphPrefs.funcPol[row];
	 case graph_param:
	 default:
		return graphPrefs.funcPar[row/2][row%2];
	}
}

/***********************************************************************
 *
 * FUNCTION:     grsetup_tbl_contents
 * 
 * DESCRIPTION:  Callback function for drawing the function name or
 *               contents of the user defined function
 *
 * PARAMETERS:   see PalmOS Reference
 *
 * RETURN:       None
 *      
 ***********************************************************************/
static void
grsetup_tbl_contents(void *table,Int16 row,Int16 column, 
					 RectangleType *bounds) IFACE;
static void
grsetup_tbl_contents(void *table,Int16 row,Int16 column, 
					 RectangleType *bounds)
{
	char *name;
	char *usrname;
	char *text;
	CError err;
	RectangleType rp;
	
	row += firstVisible;
	name = grsetup_get_fname(row);
	usrname = grsetup_def_name(row);

	WinGetClip(&rp);
	WinSetClip(bounds);
	if (StrLen(name)==0)
	  ; /* Do not draw nothing */
	else if (StrCompare(usrname,name)==0) {
		/* Special graph function, show contents */
		err = db_func_description(usrname,&text);
		if (!err) {
			WinDrawChars(text,StrLen(text),bounds->topLeft.x,bounds->topLeft.y);
			MemPtrFree(text);			
		}
		else 
		  /* Somone probably deleted it, reset the value */
		  StrCopy(name,"");
	} else {
		/* Normal function, show ordinary name */
		char tmpname[MAX_FUNCNAME+5];
		StrCopy(tmpname,name);
		StrCat(tmpname,"()");
		WinDrawChars(tmpname,StrLen(tmpname),bounds->topLeft.x,bounds->topLeft.y);
	}
	WinSetClip(&rp);
}

/***********************************************************************
 *
 * FUNCTION:     grsetup_tbl_load
 * 
 * DESCRIPTION:  Loads an on-screen table with values
 *
 * PARAMETERS:   None
 *
 * RETURN:       None
 *      
 ***********************************************************************/
static void
grsetup_tbl_load(void) IFACE;
static void
grsetup_tbl_load(void) 
{
	UInt16 rows;
	UInt16 i;
	TablePtr table;
	
	table = GetObjectPtr(grSetupTable);
	rows = TblGetNumberOfRows(table) + firstVisible;
	for (i=firstVisible;i<rows;i++) {
		if (graphPrefs.functype == graph_func)
		  TblSetItemPtr(table,i-firstVisible,2,tblFuncs[i]);
		else if (graphPrefs.functype == graph_polar)
		  TblSetItemPtr(table,i-firstVisible,2,tblPols[i]);
		else 
		  TblSetItemPtr(table,i-firstVisible,2,tblPars[i]);
	}
	/* Setup the up/down pointers */
	if (graphPrefs.functype == graph_param) {
		FormPtr frm = FrmGetActiveForm();
		FrmShowObject(frm,FrmGetObjectIndex(frm,grSetupDown));
		FrmShowObject(frm,FrmGetObjectIndex(frm,grSetupUp));
	}
}

/***********************************************************************
 *
 * FUNCTION:     grsetup_tbl_init
 * 
 * DESCRIPTION:  Initialize the on-screen table 
 *
 * PARAMETERS:   None
 *
 * RETURN:       None
 *      
 ***********************************************************************/
static void grsetup_tbl_init(void) IFACE;
static void
grsetup_tbl_init(void)
{
	Int16 i;
	TablePtr table;
	UInt16 rows;
	
	table = GetObjectPtr(grSetupTable);
	rows = TblGetNumberOfRows(table);
	
	for (i=0;i<rows;i++) {
		TblSetItemStyle(table,i,0,customTableItem);
		TblSetItemStyle(table,i,1,customTableItem);
		TblSetItemStyle(table,i,2,labelTableItem);
		TblSetItemStyle(table,i,3,customTableItem);
	}
	TblSetColumnUsable(table,2,true);
	TblSetColumnUsable(table,3,true);
	
	TblSetCustomDrawProcedure(table,3,grsetup_tbl_contents);
	
	/* We will effectively remember the last state of graph_param */
	/* because others do not have scrolling table */
	if (graphPrefs.functype != graph_param)
	  firstVisible = 0;
	
	grsetup_tbl_load();
}

/***********************************************************************
 *
 * FUNCTION:     grsetup_handle_pendown
 * 
 * DESCRIPTION:  Simulates a button in all cells of table
 *
 * PARAMETERS:   bounds of a cell in table, where the user tapped
 *
 * RETURN:       true - user lifted the pen inside bounds of table 
 *                      (button pressed)
 *               false - user lifted pen outside bounds of a field
 *      
 ***********************************************************************/
Boolean
grsetup_handle_pendown(RectangleType *bounds)
{
	Boolean penDown;
	Boolean inrect = true;
	Coord x,y;
	
	WinInvertRectangle(bounds,0);
	do {
		EvtGetPen(&x,&y,&penDown);
		if (RctPtInRectangle(x,y,bounds) && !inrect) {
			inrect = true;
			WinInvertRectangle(bounds,0);
		} else if (!RctPtInRectangle(x,y,bounds) && inrect) {
			inrect = false;
			WinInvertRectangle(bounds,0);
		}		
	}while(penDown);
	if (inrect)
	  WinInvertRectangle(bounds,0);
	return inrect;
}

/***********************************************************************
 *
 * FUNCTION:     grsetup_row_changed
 * 
 * DESCRIPTION:  Callback function for varmgr_edit, reloads a changed
 *               row and triggers table redraw
 *
 * PARAMETERS:   None
 *
 * RETURN:       None
 *      
 ***********************************************************************/
static void grsetup_row_changed(void) IFACE;
static void
grsetup_row_changed(void)
{
	char *name,*fname;
	
	name = grsetup_def_name(selectedRow);
	fname = grsetup_get_fname(selectedRow);
	/* fname is actually pointer into graphPrefs structure */
	StrCopy(fname,name);
			
	FrmUpdateForm(grSetupForm,frmUpdateVars);
}

/***********************************************************************
 *
 * FUNCTION:     grsetup_func_tap
 * 
 * DESCRIPTION:  Triggered when user tapped on the function name, show
 *               a varmgr_edit window
 *
 * PARAMETERS:   row - where the function name was pressed
 *
 * RETURN:       None
 *      
 ***********************************************************************/
static void
grsetup_func_tap(Int16 row) IFACE;
static void
grsetup_func_tap(Int16 row)
{
	char *usrname;
	char *grname;
	
	grname = grsetup_get_fname(row);	
	usrname = grsetup_def_name(row);
	
	if (StrLen(grname)==0) {
		varmgr_edit(usrname,"Graph function",function,
					grsetup_row_changed,false);
		return;
	}		
	if (StrCompare(usrname,grname)==0)
	  varmgr_edit(usrname,"Graph function",function,grsetup_row_changed,false);
	else
	  varmgr_edit(grname,"Function modification",function,NULL,false);	
}

/***********************************************************************
 *
 * FUNCTION:     grsetup_scroll
 * 
 * DESCRIPTION:  Scrolls a table (parameter functions do not fit on 1 
 *               screen)
 *
 * PARAMETERS:   dir - which direction shall we scroll
 *
 * RETURN:       None
 *      
 ***********************************************************************/
static void
grsetup_scroll(WinDirectionType dir) IFACE;
static void
grsetup_scroll(WinDirectionType dir)
{
	TablePtr table;		
	Int16 tmp;
	
	if (graphPrefs.functype!=graph_param)
	  return;
	table = GetObjectPtr(grSetupTable);
	if (dir == winUp) {
		tmp=firstVisible - TblGetNumberOfRows(table);
		if (tmp<0)
		  tmp = 0;
	}
	else /* winDown */ {
		tmp = firstVisible + TblGetNumberOfRows(table);
		if (tmp > TblGetNumberOfRows(table))
		  tmp = TblGetNumberOfRows(table);
	}
	/* Don't redraw if we did not move */
	if (tmp==firstVisible)
	  return;
	firstVisible = tmp;
	grsetup_tbl_load();
	TblMarkTableInvalid(table);
	TblRedrawTable(table);
}

/***********************************************************************
 *
 * FUNCTION:     grsetup_flist_tap
 * 
 * DESCRIPTION:  User tapped the 'X:'/'Y:' etc. Show a popup list of 
 *               defined functions with 'Off' and 'User' at the beginning
 *               Make a table searchable, Off and User have space as 
 *               a 1st character to allow incremental search
 *
 * PARAMETERS:   row, where the 'button' was pressed
 *
 * RETURN:       None
 *      
 ***********************************************************************/
static void grsetup_flist_tap(Int16 row) IFACE;
static void
grsetup_flist_tap(Int16 row)
{
	dbList *dblist;
	char **displist;
	ListPtr list;
	RectangleType bounds;
	char *fname;
	Int16 i,pos=0;

	dblist = db_get_list(function);
	displist = MemPtrNew(sizeof(*displist)*(dblist->size+2));
	
	/* displist will contain dynamic and static items, but the dynamic
	 * will be released by db_delete_list with dblist */
	displist[0] = " None";
	displist[1] = " User";
	memcpy((void *)&displist[2],(void *)dblist->list,dblist->size*sizeof(*displist));

	/* Fname is actually pointer to graphPref structure */
	fname = grsetup_get_fname(row);
	if (StrLen(fname)==0)
	  pos = 0;
	else if (StrCompare(fname,grsetup_def_name(row))==0)
	  pos = 1;
	else
	  for (i=0;i<dblist->size;i++)
		if (StrCompare(fname,dblist->list[i])==0)
		  pos = i+2;
	
	TblGetBounds(GetObjectPtr(grSetupTable),&bounds);
	list = GetObjectPtr(grSetupList);
	list->attr.search = true;
	LstSetListChoices(list,displist,dblist->size+2);
	LstSetSelection(list,pos);
	LstMakeItemVisible(list,pos);
	LstSetPosition(list,bounds.topLeft.x,bounds.topLeft.y);
	LstSetHeight(list,dblist->size+2>6?6:dblist->size+2);
	
	i = LstPopupList(list);
	
	if (i!=noListSelection && i!=pos) {
		if (i==0) 
		  StrCopy(fname,"");
		else if (i==1) /* The 'User' function get always edit window */
		  varmgr_edit(grsetup_def_name(row),"Graph function",function,
					  grsetup_row_changed,false);
		else
		  StrCopy(fname,displist[i]);
		FrmUpdateForm(grSetupForm,frmUpdateVars);
	}
	
	db_delete_list(dblist);
	MemPtrFree(displist);
}

/***********************************************************************
 *
 * FUNCTION:     GraphSetupHandleEvent
 * 
 * DESCRIPTION:  Event handler for graphic Setup form, where
 *               the functions, that should be drawn are selected
 *
 * PARAMETERS:   event
 *
 * RETURN:       handled
 *      
 ***********************************************************************/
Boolean 
GraphSetupHandleEvent(EventPtr event)
{
	Boolean    handled = false;
	FormPtr    frm=FrmGetActiveForm();
	Int16 formId;
	Int16 controlId;
	RectangleType bounds;
	Int16 row,col;

	switch (event->eType) {
	 case tblEnterEvent:
		row = event->data.tblSelect.row;
		col = event->data.tblSelect.column;
		TblGetItemBounds(GetObjectPtr(grSetupTable),
						 row,col,&bounds);
		if (grsetup_handle_pendown(&bounds)) {
			selectedRow = row + firstVisible;
			switch (col) {
			 case COL_FLIST:
				grsetup_flist_tap(selectedRow);
				break;
			 case COL_FUNC:
				grsetup_func_tap(selectedRow);
				break;
			}
		}
		handled = true;
		break;
	 case keyDownEvent:
		switch (event->data.keyDown.chr) {
		 case pageDownChr:
			grsetup_scroll(winDown);
			handled = true;
			break;
		 case pageUpChr:
			grsetup_scroll(winUp);
			handled = true;
			break;
		}
		break;
	 case ctlSelectEvent:
		controlId=event->data.ctlSelect.controlID;
		switch (controlId) {
		 case grSetupDone:
			FrmReturnToForm(0);
			FrmUpdateForm(tdFormGraph,frmUpdateVars);
			handled = true;
			break;
		 case grSetupDown:
			grsetup_scroll(winDown);
			handled = true;
			break;
		 case grSetupUp:
			grsetup_scroll(winUp);
			handled = true;
			break;
		}
		break;
	 case frmUpdateEvent:
		if (event->data.frmUpdate.updateCode == frmUpdateVars) {
			TablePtr tbl;
			
			tbl = GetObjectPtr(grSetupTable);
			TblMarkRowInvalid(tbl,selectedRow-firstVisible);
			TblRedrawTable(tbl);
			handled = true;
		}		  
		break;
	 case frmCloseEvent:
		handled = false;
		break;
	 case frmOpenEvent:
		formId=frm->formId;
		
		grsetup_tbl_init();

		FrmDrawForm(frm);
		handled=true;
		break;
	}
	  
	return handled;
}
