/*
 *   $Id: grtaps.c,v 2.7 2000/10/01 22:30:39 ondra Exp $
 * 
 *   Scientific Calculator for Palms and PilmPilots.
 *   Copyright (C) 1999,2000 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *  You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>
#include "MathLib.h"
#include "grtaps.h"
#include "calcrsc.h"
#include "grprefs.h"
#include "graph.h"
#include "calc.h"
#include "mathem.h"

Int16 trackSelected=0;

static void
grtaps_rectangle(Int16 x1,Int16 y1,Int16 x2,Int16 y2)
{
	if (x1<0 || x2<0 || y1<0 || y2<0)
	  return;
	
	WinInvertLine(x1,y1,x2,y1);
	WinInvertLine(x1,y2,x2,y2);
	WinInvertLine(x1,y1,x1,y2);
	WinInvertLine(x2,y1,x2,y2);
	WinInvertLine(x1,y1,x2,y2);
	WinInvertLine(x2,y1,x1,y2);
}

static void
grtaps_clear_edges(RectangleType *bounds,Coord mx,Coord my)
{
	RectangleType tmp,tmp2;
	
	tmp = *bounds;
	tmp.topLeft.x+=mx;
	tmp.topLeft.y+=my;
	WinClipRectangle(&tmp);

	tmp2 = *bounds;
	tmp2.extent.x = tmp.topLeft.x - bounds->topLeft.x;
	WinEraseRectangle(&tmp2,0);

	tmp2 = *bounds;
	tmp2.extent.y = tmp.topLeft.y - bounds->topLeft.y;
	WinEraseRectangle(&tmp2,0);
	
	tmp2 = *bounds;
	tmp2.topLeft.x = tmp.topLeft.x+tmp.extent.x;
	WinEraseRectangle(&tmp2,0);
	
	tmp2 = *bounds;
	tmp2.topLeft.y = tmp.topLeft.y+tmp.extent.y;
	WinEraseRectangle(&tmp2,0); 
}

static Boolean
grtaps_move(Coord begx,Coord begy,RectangleType *bounds)
{
	Boolean penDown;
	Coord x,y;
	Coord oldx=-1,oldy=-1;
	WinHandle orgwindow;
	UInt16 err;
	RectangleType orgbounds,rs;
	
	orgwindow = WinCreateOffscreenWindow(bounds->extent.x,bounds->extent.y,
										 screenFormat,&err);
	ErrFatalDisplayIf(err,"Error while creating tmp window.");
	WinCopyRectangle(NULL,orgwindow,bounds,0,0,winPaint);
	WinGetClip(&rs);
	WinSetClip(bounds);
	orgbounds = *bounds;
	orgbounds.topLeft.x = orgbounds.topLeft.y = 0;
	do {
		EvtGetPen(&x,&y,&penDown);
		if ((oldx!=x || oldy!=y) && RctPtInRectangle(x,y,bounds)) {
			WinCopyRectangle(orgwindow,NULL,&orgbounds,
							 bounds->topLeft.x + (x-begx),
							 bounds->topLeft.y + (y-begy),winPaint);
			grtaps_clear_edges(bounds,x-begx,y-begy);
			oldx=x;oldy=y;
		}
	}while(penDown);

	if (!RctPtInRectangle(x,y,bounds)) 
	  WinCopyRectangle(orgwindow,NULL,&orgbounds,bounds->topLeft.x,
					   bounds->topLeft.y,winPaint);
	else {
		graphPrefs.xmax=graph_xscr2gr(bounds->topLeft.x+bounds->extent.x-
									  (x-begx));
		graphPrefs.xmin=graph_xscr2gr(bounds->topLeft.x-(x-begx));		
		graphPrefs.ymax=graph_yscr2gr(bounds->topLeft.y-(y-begy));
		graphPrefs.ymin=graph_yscr2gr(bounds->topLeft.y+bounds->extent.y-
									  (y-begy));
	}
	
	WinSetClip(&rs);
	WinDeleteWindow(orgwindow,false);
	
	if (!RctPtInRectangle(x,y,bounds))
	  return false;
	return true;
}

static Boolean
grtaps_zoom_in(Coord begx,Coord begy,RectangleType *bounds)
{
	Boolean penDown;
	Coord x,y;
	Coord oldx=-1,oldy=-1;
	double tmp;
	
	do {
		EvtGetPen(&x,&y,&penDown);
		if ((oldx!=x || oldy!=y) && RctPtInRectangle(x,y,bounds)) {
			grtaps_rectangle(begx,begy,x,y);
			grtaps_rectangle(begx,begy,oldx,oldy);
			
			oldx=x;oldy=y;
		}
	}while(penDown);
	grtaps_rectangle(begx,begy,oldx,oldy); /* Remove the rectangle from screen */
	if (!RctPtInRectangle(x,y,bounds)) /* User wanted to cancel the zoom */
	  return false;
	
	if (oldx==begx || oldy==begy)
	  return false;
	
	if (begx>oldx) {
		tmp=begx;begx=oldx;oldx=tmp;
	}
	/* The Y axis is inversed on the screen */
	if (begy<oldy) {
		tmp=begy;begy=oldy;oldy=tmp;
	}       	
	/* begx - Left corner of the box, oldx - right */
	/* begy - bottom corner, oldy - top corner */
	
	graphPrefs.xmax=graph_xscr2gr(oldx);
	graphPrefs.xmin=graph_xscr2gr(begx);

	graphPrefs.ymax=graph_yscr2gr(oldy);
	graphPrefs.ymin=graph_yscr2gr(begy);

	return true;
}

Boolean
grtaps_track_but(void)
{
	ListPtr list;
	char *descr[MAX_GRFUNCS];
	char f_descr[MAX_GRFUNCS][4];
	Int16 nums[MAX_GRFUNCS];
	Int16 count,i;
	char *format;
	char *text;
	
	if (graphPrefs.functype == graph_func) 
	  format = "Y%d";
	else if (graphPrefs.functype == graph_polar) 
	  format = "r%d";
	else
	  return false;
	
	for (i=0,count=0;i<MAX_GRFUNCS;i++) {
		if (graphPrefs.functype == graph_func)
		  text = graphPrefs.funcFunc[i];
		else  /* graph_polar */
		  text = graphPrefs.funcPol[i];
		if (StrLen(text)) {
			StrPrintF(f_descr[i],format,i+1);
			descr[count] = f_descr[i];
			nums[count] = i;
			count++;
		}
	}
	if (count==0)
	  return false;
	list = GetObjectPtr(graphTrackList);
	LstSetListChoices(list,descr,count);
	LstSetHeight(list,count);
	LstSetSelection(list,trackSelected);
	i = LstPopupList(list);
	if (i == noListSelection)
	  return false;
	trackSelected = nums[i];
	
	return true;
}

static void
grtaps_cross(Int16 x,Int16 y,RectangleType *bounds)
{
	if (x!=-1) 
	  WinInvertLine(x,bounds->topLeft.y,x,
					bounds->topLeft.y+bounds->extent.y-1);
	if (y!=-1) 
	  WinInvertLine(bounds->topLeft.x,y,
					bounds->topLeft.x+bounds->extent.x-1,
					y);	
}

static void
grtaps_print_pol(double r, double angle,RectangleType *bounds)
{
	char text[MAX_FP_NUMBER+4];
	static double oldr=0.0,oldangle=0.0;
	
	StrCopy(text,"r:");
	DoubleToAscii(oldr,text+2);
	WinEraseChars(text,StrLen(text),bounds->topLeft.x+1,
				  bounds->topLeft.y+bounds->extent.y-12);
	DoubleToAscii(r,text+2);
	WinDrawChars(text,StrLen(text),bounds->topLeft.x+1,
				 bounds->topLeft.y+bounds->extent.y-12);
	
	StrCopy(text,"fi:");
	DoubleToAscii(oldangle,text+3);
	WinEraseChars(text,StrLen(text),bounds->topLeft.x+bounds->extent.x/2+2,
				 bounds->topLeft.y+bounds->extent.y-12);
	DoubleToAscii(angle,text+3);
	WinDrawChars(text,StrLen(text),bounds->topLeft.x+bounds->extent.x/2+2,
				 bounds->topLeft.y+bounds->extent.y-12);
	oldr = r;
	oldangle = angle;
}


static void
grtaps_print_coords(double realx, double realy,RectangleType *bounds)
{
	char text[MAX_FP_NUMBER+4];
	static double oldx=0.0,oldy=0.0;
	
	StrCopy(text,"x:");
	DoubleToAscii(oldx,text+2);
	WinEraseChars(text,StrLen(text),bounds->topLeft.x+1,bounds->topLeft.y);
	DoubleToAscii(realx,text+2);
	WinDrawChars(text,StrLen(text),bounds->topLeft.x+1,bounds->topLeft.y);
	
	StrCopy(text,"y:");
	DoubleToAscii(oldy,text+2);
	WinEraseChars(text,StrLen(text),bounds->topLeft.x+bounds->extent.x/2+2,
				 bounds->topLeft.y);
	DoubleToAscii(realy,text+2);
	WinDrawChars(text,StrLen(text),bounds->topLeft.x+bounds->extent.x/2+2,
				 bounds->topLeft.y);
	oldx = realx;
	oldy = realy;
}

static Boolean
grtaps_track_pol(Coord x,Coord y,RectangleType *bounds)
{
	double realx,realy;
	double angle,r;	
	Coord oldx,oldy,dx,dy,odx,ody;
	CodeStack *stack;
	Boolean penDown;	

	stack = graphCurves[trackSelected].stack1;
	if (!stack)
	  return false;
	
	oldx = oldy = -1;
	odx = ody = -1;
	realx = realy = r = angle = NaN;
	do {
		EvtGetPen(&x,&y,&penDown);
		if ((oldx!=x || oldy!=y) && RctPtInRectangle(x,y,bounds)) {
			realx = graph_xscr2gr(x);
			realy = graph_yscr2gr(y);
			angle = NaN;
			r = hypot(realx,realy);
			if (r!=0.0) {
				if (realx>=0.0)
				  angle = asin(realy/r);
				else if (realy>=0.0)		  
				  angle = M_PIl - asin(realy/r);
				else /* This isn't needed, but it looks better */
				  angle = -M_PIl - asin(realy/r);

				r = graph_get_value(stack,angle);
				
				dx = graph_xgr2scr(r*cos(angle));
				dy = graph_ygr2scr(r*sin(angle));
				if (!finite(angle) || !RctPtInRectangle(x,y,bounds)) {
					dy = -1;				
					dx = -1;
				}
				  
			}
			else 
			  dx = dy = -1;
			grtaps_cross(dx,dy,bounds);
			grtaps_cross(odx,ody,bounds);
			grtaps_print_coords(realx,realy,bounds);
			grtaps_print_pol(r,angle,bounds);
			oldx = x;oldy = y;
			odx = dx;ody=dy;
		}		
	}while(penDown);	   	
	/* Pen up - remove the cross */
	grtaps_cross(odx,ody,bounds);
	grtaps_print_coords(realx,realy,bounds); 
	grtaps_print_pol(r,angle,bounds);
	
	if (RctPtInRectangle(odx,ody,bounds))
	  return true;
	else
	  return false;
}

static Boolean
grtaps_track_func(Coord x,Coord y,RectangleType *bounds)
{
	double realx,realy;
	Coord oldx,oldy;
	CodeStack *stack;
	Boolean penDown;	

	stack = graphCurves[trackSelected].stack1;
	if (!stack)
	  return false;

	realx = realy = NaN;
	oldx = oldy = -1;
	do {
		EvtGetPen(&x,&y,&penDown);
		if (oldx!=x && RctPtInRectangle(x,y,bounds)) {
			realx = graph_xscr2gr(x);
			realy = graph_get_value(stack,realx);
			y = graph_ygr2scr(realy);
			if (!finite(realy) || !RctPtInRectangle(x,y,bounds))
			  y = -1;
			grtaps_cross(x,y,bounds);
			grtaps_cross(oldx,oldy,bounds);
			grtaps_print_coords(realx,realy,bounds);
			oldx = x;oldy = y;
		}
	}while(penDown);	   	
	/* Pen up - remove the cross */
	grtaps_cross(oldx,oldy,bounds);
	grtaps_print_coords(realx,realy,bounds);
	
	if (RctPtInRectangle(x,y,bounds))
	  return true;
	else
	  return false;
}
	
Boolean
grtaps_tap(Int16 x,Int16 y)
{	
	FormPtr frm = FrmGetActiveForm();
	UInt16 gadgetindex=FrmGetObjectIndex(frm,graphGadget);
	RectangleType bounds;
	
	FrmGetObjectBounds(frm,gadgetindex,&bounds);
	
	if (RctPtInRectangle(x,y,&bounds)) {
		/* Is zoom-in checked? */
		if (CtlGetValue(GetObjectPtr(graphZoomIn))) {
			if (grtaps_zoom_in(x,y,&bounds)) {
				CtlSetValue(GetObjectPtr(graphZoomIn),0);
				FrmUpdateForm(tdFormGraph,frmUpdateVars);
			}
		}				    
		else if (CtlGetValue(GetObjectPtr(graphTrack))) {
			if (graphPrefs.functype==graph_func)
			  grtaps_track_func(x,y,&bounds);
			else if (graphPrefs.functype == graph_polar)
			  grtaps_track_pol(x,y,&bounds);
		}
		else
		  if (grtaps_move(x,y,&bounds)) 
			FrmUpdateForm(tdFormGraph,frmUpdateVars);
		return true;
	}	
	return false;
}
