/*   
 *   $Id: konvert.h,v 2.18 2000/09/25 15:56:39 ondra Exp $
 * 
 *   Scientific Calculator for Palms and PilmPilots.
 *   Copyright (C) 1999,2000 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *  You can contact me at 'ondrap@penguin.cz'.
*/


#ifndef _konvert_h_
#define _konvert_h_

#include "segment.h"

#define MAX_FUNCNAME 10
#define MAX_VARNAME 10

#define IS_FUNC_LETTER(x) (((x)>='a' && (x)<='z') || (x)=='_')

typedef enum {
	degree,
	radian,
	grad
}Ttrigo_mode;

typedef enum {
	notype=0,
	integer,
	real,
	function,
	functext,
	funcempty,
	variable,
	string,	
/* These shouldn't be in rpn, it's temporary */	
	oper,
	discard,
	complex
}rpntype;

typedef enum {
	c_noerror=0,
	c_syntax,
	c_badfunc,
	c_noarg,	
	c_badarg,
	c_internal,
	c_badmode,
	c_noresult,
	c_divbyzero,
	c_notvar,
	c_badresult,
	c_deeprec,
	c_compimp,
}CError;

typedef struct {
	double real;
	double imag;
}Complex;

typedef struct {
	double r;
	double angle;
}Complex_gon;

/* item on the RPN stack */
typedef struct {
	rpntype type;
	Int16 allocsize; /* If we should free the item or not */
	union {
		UInt32 intval;
		double realval;
		Int16 funcval;
		char funcname[MAX_FUNCNAME+1];
		char varname[MAX_FUNCNAME+1];
		char *stringval;
		Complex *cplxval;
		char *data;
	}u;
}Trpn;

typedef struct {
	Trpn rpn;
	Int16 priority;
	Int16 operpri;
}Tmeq;

/* stack */
typedef struct {
	Int16 size;
	Int16 allocated;
	Trpn *stack;
}CodeStack;

typedef enum {
	floating=0,
	decimal,
	octal,
	binary,
	hexa
}Tint_mode;

struct Tdefined_funcs {
	char name[MAX_FUNCNAME+1];
	Int16 funcnum;
	CError (*function)(Int16 funcnum,CodeStack *stack);
	Int16 param_count;
};

Trpn * allocate_rpn(Int16 elem);
void delete_rpn(Trpn *myrpn,Int16 rpnsize,Boolean freerpn);
CodeStack *text_to_stack(char *buf,CError *error);
char * print_error(CError err);
Err mlib_init_mathlib(void) MLIB;
void mlib_close_mathlib(void) MLIB;

extern Tint_mode int_mode;
extern struct Tdefined_funcs defined_funcs[];
extern Ttrigo_mode trigo_mode;
extern char parameter_name[MAX_FUNCNAME+1];

#endif
