/*   
 *   $Id: calcDB.c,v 2.15 2000/09/26 20:04:15 ondra Exp $
 * 
 *   Scientific Calculator for Palms and PilmPilots.
 *   Copyright (C) 1999,2000 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *  You can contact me at 'ondrap@penguin.cz'.
*/



#include <PalmOS.h>
#include <string.h>

#include "defuns.h"
#include "konvert.h"
#include "calcDB.h"
#include "stack.h"

static DmOpenRef gDB;

static Int16 CompareRecordFunc(dbPackedRecord *rec1,dbPackedRecord *rec2,
	    Int16 unesedInt16,SortRecordInfoPtr unused1,SortRecordInfoPtr unused2,
	    MemHandle appInfoH) MLIB;
static Int16 CompareRecordFunc(dbPackedRecord *rec1,dbPackedRecord *rec2,
	    Int16 unesedInt16,SortRecordInfoPtr unused1,SortRecordInfoPtr unused2,
	    MemHandle appInfoH)
{
	Int16 tmp;
	
	tmp=StrCompare(rec1->name,rec2->name);
	return tmp;
/*	
 *	if (tmp)
 *	  return tmp;
 *
 *	if (rec1->type > rec2->type)
 *	  return 1;
 *	else if (rec1->type < rec2->type)
 *	  return -1;
 *	else
 *	  return 0;
 */
}
/***********************************************************************
 *
 * FUNCTION:     db_recompile_all
 * 
 * DESCRIPTION:  Recompiles functions stored in database and removes
 *               functions where a recompile fails. It should be called
 *               whenever a function is inserted into konvert.c
 *
 * PARAMETERS:   None
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
void
db_recompile_all(void)
{
	dbList *list;
	Int16 i;
	char *origtext;
	CError err;
	CodeStack *stack;
	
	list = db_get_list(function);
	for (i=0;i<list->size;i++) {
		err = db_func_description(list->list[i],&origtext);
		if (err) {
			db_delete_record(list->list[i]);
			continue;
		}
		stack = text_to_stack(origtext,&err);
		if (err) {
			MemPtrFree(origtext);
			db_delete_record(list->list[i]);
			continue;
		}
		db_write_function(list->list[i],stack,origtext);
		stack_delete(stack);
		MemPtrFree(origtext);
	}	
	db_delete_list(list);
}

/***********************************************************************
 *
 * FUNCTION:     db_write_function
 * 
 * DESCRIPTION:  Write a defined piece of code into database
 *
 * PARAMETERS:   name - name of function
 *               stack - compiled code
 *               origtext - original uncompiled code
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
void db_write_function(char *name,CodeStack *stack,char *origtext) MLIB;
void
db_write_function(char *name,CodeStack *stack,char *origtext)
{
	dbPackedRecord *record;
	dbPackedStack *pstack;
	Int16 psize,rsize;
	void *ptr;
	
	pstack = stack_pack(stack,&psize);
	record = MemPtrNew(sizeof(*record)+psize+StrLen(origtext)+1);
	
	record->type = function;
	StrCopy(record->name,name);
	memcpy(&record->u.stack,pstack,psize);
	
	ptr = &record->u.stack.data;
	ptr += record->u.stack.header.orig_offset;
	StrCopy(ptr,origtext);
			
	rsize = sizeof(*record)+psize+StrLen(origtext)+1;
	rsize -= sizeof(dbStackItem)>sizeof(dbPackedStack)?sizeof(dbStackItem):sizeof(dbPackedStack);
	db_create_record(record,rsize);
	MemPtrFree(record);
	MemPtrFree(pstack);
}

/***********************************************************************
 *
 * FUNCTION:     db_read_function
 * 
 * DESCRIPTION:  Read a compiled function from database
 *
 * PARAMETERS:   name - name of the function
 *
 * RETURN:       CodeStack - compiled code
 *               err - error code or 0
 *      
 ***********************************************************************/
CodeStack * db_read_function(char *name,CError *err) MLIB;
CodeStack *
db_read_function(char *name,CError *err)
{
	dbPackedRecord *record;
	CodeStack *result;
	
	*err = c_noerror;
	
	record = db_read_record(name);
	if (!record || record->type!=function) {
		if (record)
		  MemPtrFree(record);
		*err = c_notvar;
		return NULL;
	}
    result = stack_unpack(&record->u.stack);
	MemPtrFree(record);
	
	return result;
}

/***********************************************************************
 *
 * FUNCTION:     db_write_variable
 * 
 * DESCRIPTION:  Write variable to database
 *
 * PARAMETERS:   name - name of the variable
 *               contents - the value of the variable
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
void db_write_variable(char *name,Trpn contents) MLIB;
void
db_write_variable(char *name,Trpn contents)
{
	dbPackedRecord *record;
	dbStackItem *packitem;
	Int16 adatalen; /* Additional data size */
	
	packitem = rpn_pack_record(contents);
	ErrFatalDisplayIf(!packitem,"Bad variable type.");
	
	adatalen = packitem->datasize;
	
	record = MemPtrNew(sizeof(*record) + adatalen);
	record->type = variable;
	StrCopy(record->name,name);
	memcpy(record->u.data,packitem,sizeof(*packitem)+adatalen);
	
	MemPtrFree(packitem);
	db_create_record(record,sizeof(*record)+adatalen);
	
	MemPtrFree(record);
}
/***********************************************************************
 *
 * FUNCTION:     db_read_variable
 * 
 * DESCRIPTION:  Read a variable value from database
 *
 * PARAMETERS:   name - name of the requested variable
 *
 * RETURN:       Trpn - value of the variable
 *               err - error code or zero
 *      
 ***********************************************************************/
Trpn db_read_variable(char *name,CError *err) MLIB;
Trpn
db_read_variable(char *name,CError *err)
{
	Trpn result;
	dbPackedRecord *record;
	
	*err = c_noerror;
	
	record = db_read_record(name);
	if (!record || record->type!=variable) {
		if (record)
		  MemPtrFree(record);
		*err = c_notvar;
		return result;
	}
	result = rpn_unpack_record(record->u.data);
	MemPtrFree(record);
	
	return result;
}

/***********************************************************************
 *
 * FUNCTION:     db_delete_list
 * 
 * DESCRIPTION:  Free memory allocated by list
 *
 * PARAMETERS:   list - list to free
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
void
db_delete_list(dbList *list)
{
	Int16 i;
	
	for (i=0;i<list->size;i++)
	  MemPtrFree(list->list[i]);
	if (list->type)
	  MemPtrFree(list->type);
	MemPtrFree(list);
}

/***********************************************************************
 *
 * FUNCTION:     db_get_list
 * 
 * DESCRIPTION:  Creates a dynamic list of all requested records
 *
 * PARAMETERS:   type - type of records to build a list 
 *                      or 0 for all types
 *
 * RETURN:       dbList - array of all records of given type
 *      
 ***********************************************************************/
dbList *
db_get_list(rpntype type)
{
	UInt16 totalItems = DmNumRecords(gDB);
	UInt16 count,i;
	MemHandle recordHandle;
	dbPackedRecord *record;
	dbList *result;

	result = MemPtrNew(sizeof(dbList)+totalItems*sizeof(result->list[0]));
	result->type = MemPtrNew(sizeof(result->type[0])*totalItems);
	
	for (i=0,count=0;i<totalItems;i++) {
		recordHandle = DmQueryRecord(gDB,i);
		record = MemHandleLock(recordHandle);
		if (!type || record->type == type) {
			result->type[count] = record->type;
			result->list[count] = MemPtrNew(StrLen(record->name)+1);
			StrCopy(result->list[count],record->name);
			count++;
		}
		MemHandleUnlock(recordHandle);
	}
	result->size = count;
	return result;
}

/* Return original (before-compilation) text of a function */
CError db_func_description(char *funcname,char **dest) MLIB;
CError
db_func_description(char *funcname,char **dest)
{
	dbPackedRecord *tmp;

	tmp = db_read_record(funcname);
	if (!tmp)
	  return c_notvar;
	if (tmp->type != function) {
		MemPtrFree(tmp);
		return c_badarg;
	}
	*dest = MemPtrNew(StrLen(&tmp->u.stack.data[tmp->u.stack.header.orig_offset])+1);
	StrCopy(*dest,&tmp->u.stack.data[tmp->u.stack.header.orig_offset]);
	
	MemPtrFree(tmp);
	return c_noerror;
}

dbPackedRecord * db_read_record(char *name) MLIB;
dbPackedRecord *
db_read_record(char *name)
{
	UInt16 recordNumber;
	Int16 foundIt=0;
	dbPackedRecord tmp;
	
	StrCopy(tmp.name,name);
	recordNumber=DmFindSortPosition(gDB,&tmp,0,
					(DmComparF *)CompareRecordFunc,0);
	if (recordNumber >0) {
		MemHandle theRecordMemHandle;
		dbPackedRecord *record;
		
		theRecordMemHandle = DmQueryRecord(gDB,recordNumber - 1);

		record = MemHandleLock(theRecordMemHandle);
		foundIt = StrCompare(name,record->name)==0;
		if (foundIt) {
			dbPackedRecord *result;
			
			result = MemPtrNew(MemHandleSize(theRecordMemHandle));
			memcpy(result,record,MemHandleSize(theRecordMemHandle));
			MemHandleUnlock(theRecordMemHandle);
			
			return result;
		}
		MemHandleUnlock(theRecordMemHandle);
	}      
	return NULL;
}

Boolean db_record_exists(char *name) MLIB;
Boolean
db_record_exists(char *name)
{
	UInt16 recordNumber;
	Int16 foundIt=0;
	dbPackedRecord tmprec;
	
	StrCopy(tmprec.name,name);
	
	recordNumber=DmFindSortPosition(gDB,&tmprec,0,
					(DmComparF *)CompareRecordFunc,0);
	
	if (recordNumber >0) {
		MemHandle theRecordMemHandle;
		dbPackedRecord *record;
		
		theRecordMemHandle = DmQueryRecord(gDB,recordNumber - 1);
		
		record = MemHandleLock(theRecordMemHandle);
		foundIt = StrCompare(tmprec.name,record->name)==0;
		MemHandleUnlock(theRecordMemHandle);
	}      
	return foundIt;
}

Int16 db_delete_record(char *name) MLIB;
Int16 
db_delete_record(char *name)
{
	UInt16 recordNumber;
	Int16 foundIt=0;
	dbPackedRecord tmprec;
	
	StrCopy(tmprec.name,name);
	
	recordNumber=DmFindSortPosition(gDB,&tmprec,0,
					(DmComparF *)CompareRecordFunc,0);
	
	if (recordNumber >0) {
		MemHandle theRecordMemHandle;
		dbPackedRecord *record;
		
		theRecordMemHandle = DmQueryRecord(gDB,recordNumber - 1);
		
		record = MemHandleLock(theRecordMemHandle);
		foundIt = StrCompare(tmprec.name,record->name)==0;
		MemHandleUnlock(theRecordMemHandle);
		if (foundIt) 
		  DmRemoveRecord(gDB,recordNumber-1);
	}      
	return foundIt;
}

Int16 db_create_record(dbPackedRecord *newRecord,Int16 size) MLIB;
Int16
db_create_record(dbPackedRecord *newRecord,Int16 size)
{
	UInt16 recordNumber;
	MemHandle myRecordMemHandle;
	dbPackedRecord *newRecordPtr;
	
	recordNumber = DmFindSortPosition(gDB,newRecord,0,
					 (DmComparF *)CompareRecordFunc,0);
	if (recordNumber > 0) {  /* We might modify the record */
		dbPackedRecord *record;	       
		MemHandle recordMemHandle;
		Int16 foundIt;
		
		recordMemHandle = DmQueryRecord(gDB,recordNumber - 1);
		record = MemHandleLock(recordMemHandle);
		foundIt = StrCompare(newRecord->name,record->name) == 0;
		MemHandleUnlock(recordMemHandle);
		if (foundIt) {		      
			/* OK, we have to modify existing record */
			MemHandle modRecord = DmGetRecord(gDB,recordNumber-1);

			/* resize the handle */
			MemHandleResize(modRecord,size);
			
			record = MemHandleLock(modRecord);
			DmWrite(record,0,newRecord,size);
			MemHandleUnlock(modRecord);
			DmReleaseRecord(gDB,recordNumber-1,true);
			return 0;
		}
	}
	myRecordMemHandle = DmNewRecord(gDB,&recordNumber,size);
	newRecordPtr = MemHandleLock(myRecordMemHandle);
	DmWrite(newRecordPtr,0,newRecord,size);
	MemHandleUnlock(myRecordMemHandle);	
	DmReleaseRecord(gDB,recordNumber,true); /* Now the recordNumber contains real index */
	
	return 0;
}

static Err db_open_byid(LocalID dbid) MLIB;
static Err
db_open_byid(LocalID dbid)
{
        gDB=DmOpenDatabase(CARDNO,dbid,dmModeReadWrite);
        if (gDB)
          return DmGetLastErr();                        
        return 0;
}

Int16 db_open() MLIB;
Int16 
db_open()
{
	Err err;
	LocalID dbid;
	UInt16 version;
	UInt32 creator;
	UInt32 type;

	dbid=DmFindDatabase(CARDNO,DBNAME);	
	if (dbid) {  /* Database exists */
		DmDatabaseInfo(CARDNO,dbid,
			       NULL, /* name */
                               NULL, /* attrib */
                               &version, /* version */
                               NULL, /* crDate */
                               NULL, /* modDate */
                               NULL, /* bckUpDate */
                               NULL, /* modNum */
                               NULL, /* appinfoID */
                               NULL, /* sortInfoID */
                               &type, /* Type */
                               &creator); /* Creator */

		if (version==DBVERSION && creator==LIB_ID && type==DBTYPE)
		  return db_open_byid(dbid);
		
		/* Database exists, but with uncorrect version */
		err=DmDeleteDatabase(CARDNO,dbid);
		if (err)
		  return err;
	}
	
	/* Database doesn't exist or old version */
	err=DmCreateDatabase(CARDNO,DBNAME,LIB_ID,DBTYPE,false);
	if (err)
	  return err;
	
	dbid=DmFindDatabase(CARDNO,DBNAME);
	
	version=DBVERSION;
	DmSetDatabaseInfo(CARDNO,dbid,
			  NULL, /* name */
			  NULL, /* attrib */
			  &version, /* version */
			  NULL, /* crDate */
			  NULL, /* modDate */
			  NULL, /* bckUpDate */
			  NULL, /* modNum */
			  NULL, /* appinfoID */
			  NULL, /* sortInfoID */
			  NULL, /* Type */
			  NULL); /* Creator */
	return db_open_byid(dbid);
}

Int16 db_close() MLIB;
Int16 
db_close()
{
	return DmCloseDatabase(gDB);
}
