/*
 *   $Id: fp.c,v 2.5 2000/09/28 16:09:37 ondra Exp $
 * 
 *   Scientific Calculator for Palms and PilmPilots.
 *   Copyright (C) 1999 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *  You can contact me at 'ondrap@penguin.cz'.
*/


#ifdef UNIX
#define StrChr(x,y)	strchr(x,y)
#define StrCopy(x,y)	strcpy(x,y)
#define StrLen(x)	strlen(x)

#include <stdlib.h>
#include <string.h>
#else
#include <PalmOS.h>
#include <StringMgr.h>
#include <MathLib.h>
#endif
#include <defuns.h>

static double SCI_LIMIT_MAX;   /* Above this number show it as xEy */
static double SCI_LIMIT_MIN;   /* Under this number show as xE-y */
static double DISP_MIN_LIMIT = 1E-15; /* Show numbers as sin(pi) as 1E-16,
				       it's 0 */
static double DISP_MAX_LIMIT = 1E300; /* Don't show bigger numbers, it
				       * makes problems */
static Int16 DISP_FRAC = 9;
static Int16 FLOAT_MODE = 0;
static Int16 SHOW_MODE = 0;

static char FLPOINT_CHAR = '.';
 
/* If the number is smaller, reduce the number of 
 * decimal points, so we do not get something like
 * 9.999999987E-10
 */
static double REDUCE_PREC = 1E-6;
#define REDUCE_TO 6

/* Internal prototypes */

static void format_number(double,short,char *,Int16) MLIB;

void GetDisplayMode(Int16 *mode,Int16 *frac,Int16 *floatmode,Boolean *reducePrecision) MLIB;
void GetDisplayMode(Int16 *mode,Int16 *frac,Int16 *floatmode,Boolean *reducePrecision)
{
	*mode=SHOW_MODE;
	*frac=DISP_FRAC;
	*floatmode=FLOAT_MODE;
	*reducePrecision=DISP_MIN_LIMIT==1E-15;
}

void SetFlPoint(char flpoint) MLIB;
void SetFlPoint(char flpoint)
{
	FLPOINT_CHAR = flpoint;
}

/* Mode-> Normal, Scientific, Engineer 
 * frac - precision (up to 9)
 * floatmode - stripzeros at the end
 */
void SetDisplayMode(Int16 mode,Int16 frac,Int16 floatmode,Boolean reducePrecision) MLIB;
void SetDisplayMode(Int16 mode,Int16 frac,Int16 floatmode,Boolean reducePrecision)
{	
	switch (mode) {
         case 0:
                SCI_LIMIT_MAX=1E10;
                SCI_LIMIT_MIN=1E-5;
                break;
         case 1:
	 case 2:
                SCI_LIMIT_MAX=1.00;
                SCI_LIMIT_MIN=1.01;
                break;	
        }                      
	SHOW_MODE=mode;
	DISP_FRAC=frac;
	FLOAT_MODE=floatmode;
	if (reducePrecision) {
		DISP_MIN_LIMIT = 1E-15;
		REDUCE_PREC = 1E-6;
	} else {
		DISP_MIN_LIMIT = 1E-100;
		REDUCE_PREC = 1E-90;
	}
}

static void cvt_fltoa(double value,char *strP) MLIB;
static void
cvt_fltoa(double value,char *strP)				/* convert ulong to string */
	/* value - value to convert
	 * strP - destination buffer */
{
Int16 i;
char *cP;
char tmpstr[32];	       

	if (value==0) {
		StrCopy(strP,"0");
		return;
	}
	/* We convert the number backwards */
	for( cP = tmpstr+19, i = 20; i-- > 0; )
	  {
		  *cP-- = (Int16)fmod(value,10) + '0';
		  value /= 10;     
		  value=trunc(value);
	  }
	
	tmpstr[20] = '\0';
	
	for( cP = tmpstr; *cP && (*cP == '0'); )
	  ++cP;
	
	StrCopy(strP,cP);
}

/*
 * General number formatting, 
 */

void  DoubleToAscii(double value,char *strP) MLIB;	
void 
DoubleToAscii(double value,char *strP)
{
	Int16 ex;
	double origval=value;
	char tmpstr[32];
	     
	
	if (!finite(value)) {
		/* Handle the NaN, +Inf & -Inf cases */
		format_number(value,0,strP,false);
	}
	else if (fabs(value)>DISP_MAX_LIMIT) {
		StrCopy(strP,"It's too big");
	}
	else if (fabs(value)<DISP_MIN_LIMIT) { 
		/* Don't show too small numbers */
		format_number(0,DISP_FRAC,strP,FLOAT_MODE);
		if (SHOW_MODE!=0) 
		  StrCat(strP,"E0");
	}
	else if (fabs(value)>=1 && SHOW_MODE==2) {
		/* Engineer mode selected */
		for (ex=0;fabs(value)>=1000.0 || (ex % 3);ex++,value/=10)
		  ;
		format_number(value,DISP_FRAC,strP,FLOAT_MODE);	      
		StrPrintF(tmpstr,"E%d",ex);
		StrCat(strP,tmpstr);
	}
	else if (fabs(value)<1.0 && SHOW_MODE==2) {
		/* Engineer mode selected */
		for (ex=0;fabs(value)<0.99999999 || (ex % 3);ex++,value*=10)
		  ;
/* Reduce the precision by so small numbers */
		if (fabs(origval)<REDUCE_PREC)	
		  format_number(value,DISP_FRAC>REDUCE_TO?REDUCE_TO:DISP_FRAC,strP,FLOAT_MODE);
		else
		  format_number(value,DISP_FRAC,strP,FLOAT_MODE);
		
		StrPrintF(tmpstr,"E-%d",ex);
		StrCat(strP,tmpstr);
	}
	else if (fabs(value)>SCI_LIMIT_MAX) {
		/* Normal/Scientific mode */
		for (ex=0;fabs(value)>=10.0;value/=10,ex++)
		  ;
		format_number(value,DISP_FRAC,strP,FLOAT_MODE);
		StrPrintF(tmpstr,"E%d",ex);
		StrCat(strP,tmpstr);
	}
	else if(fabs(value)<SCI_LIMIT_MIN && fabs(value)>0) {
		/* Normal/Scientific mode */
		for (ex=0;fabs(value)<0.99999999;value*=10,ex--)
		  ;
		/* Reduce the precision by so small numbers */
		if (fabs(origval)<REDUCE_PREC)	
		  format_number(value,DISP_FRAC>REDUCE_TO?REDUCE_TO:DISP_FRAC,strP,FLOAT_MODE);
		else
		  format_number(value,DISP_FRAC,strP,FLOAT_MODE);
		StrPrintF(tmpstr,"E%d",ex);
		StrCat(strP,tmpstr);
	}
	else {
		/* No special formatting required */
		format_number(value,DISP_FRAC,strP,FLOAT_MODE);
	}       
}

static void
format_number(double value,Int16 frac,char *strP,Int16 fixed) MLIB;
static void
format_number(double value,Int16 frac,
			  char *strP,Int16 fixed)			/* format a number into str */
/*
value;					 value to format 
frac;					 number of frac digits,
						  -1 = floating point
*strP;					 destination 
fixed;                   fixed point (0/1) */
{
	Int16 i;
	double ipart;
	double fpart;
	double limit;
	char str[MAX_FP_NUMBER*2];
	
	if (isinf(value)) {
		if (isinf(value)>0) 
		  StrCopy(strP,"+Inf");
		else
		  StrCopy(strP,"-Inf");
		return;
	}
	if (!finite(value)) {
		StrCopy(strP,"NaN");
		return;
	}
    
    if( value < 0.0 ) {
		*strP++ = '-';
		value = -value;
	}

    ipart = trunc(value);
    value -= ipart;
	/* recover frac part */
    for( limit = 1.0,i=frac; i-- > 0; ) {
		value *= 10.0;
		limit *= 10.0;
	}
	
    value += 0.5;
    if( value >= limit ) {
		fpart = 0.0;				/* overflowed */
		ipart++;
    }
    else
	  fpart = value;

    cvt_fltoa(ipart,strP);
   
    if (frac) {
		cvt_fltoa(fpart,str);
		strP += StrLen(strP);
		if (!fixed) {
			/* remove trailing 0's */
			for (i = StrLen(str)-1;i>=0 && str[i]=='0';i--) {
				str[i]='\0';
				frac--;
			}	      
		}      
		if (StrLen(str)>0) {
			*strP++ = FLPOINT_CHAR;
			for( i = StrLen(str); i++ < frac; )
			  *strP++ = '0';			/* need some padding */
		}
		StrCopy(strP,str);			/* and now the value */
    }
}
