/*
 *   $Id: konvert.c,v 2.39 2000/09/28 23:12:34 ondra Exp $
 *
 *   Scientific Calculator for Palms and PilmPilots.
 *   Copyright (C) 1999,2000 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *  You can contact me at 'ondrap@penguin.cz'.
*/

#include <stdio.h>
#include <PalmOS.h>
#include <StringMgr.h>

#include "defuns.h"
#include "MathLib.h"
#include "konvert.h"
#include "mathem.h"
#include "funcs.h"
#include "stack.h"
#include "prefs.h"
#include "history.h"


void DoubleToAscii(double,char *);

/* Global parameters */
char parameter_name[MAX_FUNCNAME+1]="x";


/* Structure, that is used both to call functions from compiled 
 * code (defined_funcs[funcval]) and to decode an expression.
 * The first few elements are directly dependent on the constants used
 * in 'def_ops', as the constant points directly to the position
 * in this table. This dependence is here because of efficiency, it might
 * be removed in the future, because compilation isn't so time-critical
 * task.
 * 
 * Items -
 *   name - name of the function, for operators this is only informational,
 *          must be lowercase and less the MAX_FUNCNAME
 *   funcnum - a 'helping' constant, allows us to use one computing
 *             function for multiple operators/user functions
 *   function - the function to be called
 *   param_count - number of parameters this function requiers,
 *                 for operators only informational
 */  

struct Tdefined_funcs defined_funcs[]={	
	  {"+",FUNC_PLUS,math_oper,2},
	  {"-",FUNC_MINUS,math_oper,2},
	  {"*",FUNC_MULTIPLY,math_oper,2},
	  {"/",FUNC_DIVIDE,math_oper,2},
	  {"^",FUNC_POWER,math_oper,2},
	  {"=",FUNC_EQUAL,define_func,2},
	  {"neg",FUNC_NEGATE,math_neg,1},
	  {";",FUNC_DISCARD,discard_arg,1},
	  {"&",FUNC_AND,math_bin,2},
	  {"|",FUNC_OR,math_bin,2},
	  {"<",FUNC_SHL,math_bin,2},
	  {">",FUNC_SHR,math_bin,2},
	  {":",FUNC_EMPTY,empty_arg,1},
	  {"!",FUNC_X,eval_x,0},
	  {"%",FUNC_MOD,math_oper,2},
/* The 'name' isn't used for previous operators, they are assigned other way */
/* Previous functions are in defined order, be aware of FUNC_ constants */
	  {"sin",MATH_SIN,math_trigo,1},
	  {"cos",MATH_COS,math_trigo,1},
	  {"tan",MATH_TAN,math_trigo,1},
	  {"sinh",MATH_SINH,math_trigo,1},
	  {"cosh",MATH_COSH,math_trigo,1},
	  {"tanh",MATH_TANH,math_trigo,1},
	  {"asin",MATH_ASIN,math_arctrigo,1},
	  {"acos",MATH_ACOS,math_arctrigo,1},
	  {"atan",MATH_ATAN,math_arctrigo,1},
	  {"asinh",MATH_ASINH,math_arctrigo,1},
	  {"acosh",MATH_ACOSH,math_arctrigo,1},
	  {"atanh",MATH_ATANH,math_arctrigo,1},
	  {"exp",MATH_EXP,math_math,1},
	  {"ln",MATH_LN,math_math,1},
	  {"log",MATH_LOG,math_math,1},
	  {"logb",MATH_LOG2,math_math,1},
	  {"sqrt",MATH_SQRT,math_math,1},
	  {"cbrt",MATH_CBRT,math_math,1},
	  {"abs",MATH_ABS,math_math,1},
	  {"fact",MATH_FACT,math_fact,1},
	  {"todeg",FUNC_DEGREE,convert_real_to_string,1},	  
	  {"int",0,math_int,1},
	  {"real",MATH_REAL,math_math,1},
	  {"round",MATH_ROUND,math_math,1},
	  {"trunc",MATH_TRUNC,math_math,1},
	  {"npr",MATH_PERM,math_comb,2},
	  {"ncr",MATH_COMB,math_comb,2},
	  {"defparamn",0,change_param_name,1},
	  {"fzero",MATH_FZERO,math_fsolve,3},
	  {"unset",0,unset_variable,1},
	  {"rtopr",MATH_RTOPR,math_convcoord,2},
	  {"rtopd",MATH_RTOPD,math_convcoord,2},
	  {"ptorx",MATH_PTORX,math_convcoord,2},
	  {"ptory",MATH_PTORY,math_convcoord,2},
	  {"torad",FUNC_RADIAN,convert_real_to_string,1},
	  {"togonio",FUNC_GONIO,convert_cplx_to_string,1},
	  {"ipart",MATH_TRUNC,math_math,1},
	  {"fpart",MATH_FPART,math_math,1},
	  {"history",0,history_command,1},
	  {"tocis",FUNC_TOCIS,convert_cplx_to_string,1},
	  {"",0,NULL}
};

/* Fast lookup table for operators, will be probably changed in the
 * future.
 * 
 * chr - character to be recognized as an operator
 * operpri - priority of the oprator
 * func - position in defined_funcs array of the corresponding function
 */
struct {
	char chr;
	Int16 operpri;
	Int16 func;
}def_ops[]={
	{'+',1,FUNC_PLUS},
	{'-',1,FUNC_MINUS},
	{'*',2,FUNC_MULTIPLY},
	{'/',2,FUNC_DIVIDE},
	{'^',3,FUNC_POWER},
	{'=',0,FUNC_EQUAL},
	{'&',1,FUNC_AND},
	{'|',1,FUNC_OR},
	{'<',1,FUNC_SHL},
	{'>',1,FUNC_SHR},
	{'%',2,FUNC_MOD},
	{0,0,0}
};

static Trpn *rpn=NULL;
static Tmeq *meq=NULL;
static Tmeq *operstack=NULL;
static Int16 meqcount=0;
static Int16 opercount=0;
static Int16 rpncount=0;


static Int16 konvert_to_meq(char *buf,Int16 priority,Int16 param_count);
Int16 fl_num(char *input,double *cislo) MLIB;

static Int16
try_real_num(char *buf,Int16 priority)
{
	double t1;
	Int16 res;
	
	if (!(res=fl_num(buf,&t1)))
	  return 0;
	meq[meqcount].priority=priority;
	meq[meqcount].rpn.type=real;
	meq[meqcount].rpn.u.realval=t1;
	meq[meqcount].rpn.allocsize=0;
	meqcount++;
	return res;
}

static Int16
try_degree(char *buf,Int16 priority)
{
	Int16 length=0;
	Int16 res;
	double result=0,tmp;
	Int16 negate=0;       	
	
	if (calcPrefs.trigo_mode!=degree || calcPrefs.int_mode!=floating)
	  return 0;
	if (*buf!='\225')
	  return 0;	
	buf++;length++;
	
	meq[meqcount].rpn.type=real;	
	meq[meqcount].priority=priority;
	if (*buf=='-') {
		buf++;length++;
		negate=1;
	}
	while (*buf>='0' && *buf<='9') {
		result*=10.0;
		result+=(*buf-'0');
		buf++;length++;
	}
	if (*buf!='') {
		if (negate)
		  result=-result;
		meq[meqcount].rpn.u.realval=result;
		meqcount++;
		return length;
	}
	length++;buf++;
	tmp=0;
	if (*buf<'0' || *buf>'9')
	  return 0;
	if (!(res=fl_num(buf,&tmp)))
	  return 0;
	length+=res;	
	result+=tmp/60.0;
	if (negate)
	  result=-result;
	
	meq[meqcount].rpn.u.realval=result;
	meq[meqcount].rpn.allocsize=0;
	meqcount++;
	
	return length;
}

static Int16
try_int_num(char *buf,Int16 priority)
{
	Int16 length=0;
	
	meq[meqcount].rpn.type=integer;
	meq[meqcount].rpn.allocsize = 0;
	meq[meqcount].rpn.u.intval=0;
	switch (calcPrefs.int_mode) {
	 case decimal:
		while (*buf>='0' && *buf<='9') {
			length++;
			meq[meqcount].rpn.u.intval*=10;
			meq[meqcount].rpn.u.intval+=*buf-'0';
			buf++;
		}
		break;
	 case octal:
		while (*buf>='0' && *buf<='7') {
			length++;
			meq[meqcount].rpn.u.intval*=8;
			meq[meqcount].rpn.u.intval+=*buf-'0';
			buf++;
		}
		break;
	 case binary:
		while (*buf>='0' && *buf<='1') {
			length++;
			meq[meqcount].rpn.u.intval*=2;
			meq[meqcount].rpn.u.intval+=*buf-'0';
			buf++;
		}
		break;
	 case hexa:
		while ((*buf>='0' && *buf<='9') || (*buf>='A' && *buf<='F')) {
			length++;
			meq[meqcount].rpn.u.intval*=16;
			if (*buf>='0' && *buf<='9')
			  meq[meqcount].rpn.u.intval+=*buf-'0';
			else
			  meq[meqcount].rpn.u.intval+=*buf-'A'+10;
			buf++;
		}
		break;
	 default:
		return 0;
	}
	if (length) {
		meq[meqcount].priority=priority;
		meqcount++;
	}
	return length;
}

static Int16 
try_op(char *buf,Int16 priority)
{
	Int16 retval=0;
	Int16 i=0;
	
	meq[meqcount].priority=priority;	
	for (;def_ops[i].chr;i++) {
		if (def_ops[i].chr==*buf) {
			meq[meqcount].rpn.type=oper;
			meq[meqcount].rpn.u.funcval=def_ops[i].func;
			meq[meqcount].operpri=def_ops[i].operpri;
			meq[meqcount].rpn.allocsize = 0;
			meqcount++;
			retval=1;
			break;
		}
	}
	return retval;
}

static Int16 
try_func(char *buf,Int16 priority)
{	
	Int16 length=0;
	Int16 tmp,i;
	char fname[MAX_FUNCNAME+1];
	
	if (*buf=='(') { /* Only parentheses */
		length++;
		buf++;		
		if (!(tmp=konvert_to_meq(buf,priority,1)))
		  return 0;
		return length+tmp;
	}
	
	while (IS_FUNC_LETTER(*buf) && length<MAX_FUNCNAME) {
		fname[length]=*buf;
		length++;buf++;
	}             
	fname[length]='\0';
	if (*buf!='(') /* Not a function */
	  return 0;
	length++;buf++;
	
	for (i=0;defined_funcs[i].name[0];i++) 
	  if (StrCompare(defined_funcs[i].name,fname)==0) {
		  meq[meqcount].priority=priority;
		  meq[meqcount].rpn.type=function;
		  meq[meqcount].rpn.allocsize = 0;
		  meq[meqcount].rpn.u.funcval=i;
		  meqcount++;
		  if (!(tmp=konvert_to_meq(buf,priority+1,
					   defined_funcs[i].param_count)))
		    return 0;
		  return length+tmp;
	  }
	/* Not known function */
	if (*buf==')') /* Is an 'empty' function */ {
		meq[meqcount].priority=priority;
		meq[meqcount].rpn.type=funcempty;
		meq[meqcount].rpn.allocsize = 0;
		StrCopy(meq[meqcount].rpn.u.funcname,fname);
		meqcount++;
		return length+1;
	}
	/* Is a 'not known'(text) function with arguments */
	meq[meqcount].priority=priority;
	meq[meqcount].rpn.type=functext;
	meq[meqcount].rpn.allocsize = 0;
	StrCopy(meq[meqcount].rpn.u.funcname,fname);
	meqcount++;
	if (!(tmp=konvert_to_meq(buf,priority+1,1)))
	  return 0;
	return length+tmp;
}

static Int16
try_variable(char *buf,Int16 priority)
{
	Int16 length=0;

	if (!IS_FUNC_LETTER(*buf))
	  return 0;
	
	meq[meqcount].priority=priority;
	while (IS_FUNC_LETTER(*buf) && length<MAX_FUNCNAME) {
		meq[meqcount].rpn.u.varname[length]=*buf;
		length++;buf++;
	}
	if (*buf=='(' || IS_FUNC_LETTER(*buf))
	  return 0;
	/*  It is not a function, and it is smaller then MAX_FUNCNAME */
	meq[meqcount].rpn.u.varname[length]='\0';
	if (StrCompare(meq[meqcount].rpn.u.varname,parameter_name)==0) {
		/* Special - parameter of a function */
		meq[meqcount].rpn.type=function;
		meq[meqcount].rpn.u.funcval=FUNC_X;
		meq[meqcount].rpn.allocsize = 0;
		meqcount++;
		return length;
	}
	meq[meqcount].rpn.type=variable;
	meq[meqcount].rpn.allocsize = 0;
	meqcount++;
	return length;
}

/* Syntax of string: "stringkjdfj", 
 * dquot: "....\"...."
 * slash: "....\\...."
 */
static Int16
try_string(char *buf,Int16 priority)
{
	char *text;
	Int16 i;
	Int16 slashcount=0;
	
	if (*buf!='"')
	  return 0;
	buf++;
	text=MemPtrNew(MAX_INPUT_LENGTH);
      
	
	for(i=0;*buf && *buf!='"';i++,buf++) {
		/* Ignore special meaning of character following slash */
		if (*buf=='\\' && *(buf+1)) {
			buf++;
			slashcount++;
		}
		text[i]=*buf;
	}
	if (*buf!='"') {
		MemPtrFree(text);
		return 0;
	}
	text[i]='\0';
	meq[meqcount].priority=priority;
	meq[meqcount].rpn.type=string;
	meq[meqcount].rpn.u.stringval=text;
	meq[meqcount].rpn.allocsize=StrLen(text)+1;
	meqcount++;
	return i+2+slashcount;
}

/* Converts a mathematical string to an array of 'tokens', where
 * every token is a number, operator or function. To every operator 
 * is assigned a priority based on the input parameter.
 * Input:
 * buf - string to decode
 * priority - starting priority
 * param_count - exact number of expected parameters delimited by ':'
 * meqcount - counter of used fields on the meq[] 
 * Output:
 * Int16 - 0 - error, otherwise number of characters decoded
 * Output is written to global array 'meq' with a
 * counter 'meqcount'
 * 
 * Basic prioritization:
 *   pr+3 : number, function
 *   pr+2 : '-' on the nonstandard place, connected to following number
 *   pr+1   : operators, they have added oper-priority, that is 
 *          later (meq_prioritize) used to reprioritize the neighbour items
 *          
 *   pr   : ':' -        
 *        : ';' - terminates completely whole expression,
 *                it's also given a special type 'discard',
 *                that is later recognized at the level of konvert_to_rpn
 * 
 * Algorithm:
 * Every mathematical expression consists of:
 * TOK1 - number|function
 * OPER - operator
 *  - TOK1[OPER TOK2[OPER TOK3...]]
 * with some exceptions, of course...
 */
static Int16
konvert_to_meq(char *buf,Int16 priority,Int16 param_count)
{	
	Int16 tmp;
	Int16 length=0;
	Int16 param_encountred = 1;
	Int16 (*try_num)(char *,Int16);
	
	if (calcPrefs.int_mode==binary || calcPrefs.int_mode==octal ||
	    calcPrefs.int_mode==hexa || calcPrefs.int_mode==decimal) 
	  try_num=try_int_num;
	else if (calcPrefs.int_mode==floating)
	  try_num=try_real_num;	
	else
	  return 0;
	
	/* We should allow an empty beginning */
	if (*buf==';' && priority==0) {
		buf++;length++;
	}	  
	
	/* We can begin with '-',number or function */
	if (*buf=='-') {
		buf++;
		length+=1;

		meq[meqcount].priority=priority+2;
		meq[meqcount].rpn.type=function;
		meq[meqcount].rpn.u.funcval=FUNC_NEGATE;
		meq[meqcount].rpn.allocsize = 0;
		meqcount++;
 
	}
	
	while (1) {		
		if ((tmp=try_num(buf,priority+3))) {
			buf+=tmp;
			length+=tmp;
		}
		else if ((tmp=try_degree(buf,priority+3))) {
			buf+=tmp;
			length+=tmp;
		}
		else if ((tmp=try_func(buf,priority+3))) {
			buf+=tmp;
			length+=tmp;
		}
		else if ((tmp=try_variable(buf,priority+3))) {
			buf+=tmp;
			length+=tmp;
		}
		else if ((tmp=try_string(buf,priority+3))) {
			buf+=tmp;
			length+=tmp;
		}
		else
		  return 0; /* Syntax error */
		/* End of expression?, priority=0 means we are at starting level */
		if (!*buf || (*buf==')' && priority>0) 
		    || (*buf==';' && priority>0)) {
			if (*buf==')')
			  length++;
			if (param_encountred != param_count)			  
			  return 0; /* Bad parameter count */
			
			return length;
		}
		
		
		if ((tmp=try_op(buf,priority+1))) {
			buf+=tmp;
			length+=tmp;
		}
		/* Next parameter */
		else if (*buf==':') {
			param_encountred++;
			buf++;
			length++;
			meq[meqcount].priority=priority;
			meq[meqcount].rpn.type=function;
			meq[meqcount].rpn.u.funcval=FUNC_EMPTY;
			meq[meqcount].rpn.allocsize = 0;
			meqcount++;
		}
		/* This is for syntax like '3x'=>'3*x' and 3(3+4)*/
		else if (IS_FUNC_LETTER(*buf) || *buf=='(') {
			try_op("*",priority+1);
		}		
		else if (*buf==';' && priority==0) {
			buf++;
			length++;
			meq[meqcount].priority=0;
			meq[meqcount].rpn.type=discard;
			meq[meqcount].rpn.u.funcval=FUNC_DISCARD;
			meq[meqcount].rpn.allocsize = 0;
			meqcount++;
		}
		else 
		  return 0; /* Syntax error */
		
		/* Allow syntax like 3^-4 */
		if (*buf=='-') {
			buf++;
			length+=1;
			
			meq[meqcount].priority=priority+2;
			meq[meqcount].rpn.type=function;
			meq[meqcount].rpn.u.funcval=FUNC_NEGATE;
			meq[meqcount].rpn.allocsize = 0;
			meqcount++;
		}
	}
}

/* Finds operators and raises priority of things with higher priority
 * backwards and forwards from the operator 
 * 
 * Input:
 * meq - global array that is filled with decoded expression from 
 *       konvert_to_meq
 * meqcount
 * Output:
 * meq
 */
static void
meq_prioritize(void)
{
	Int16 i,j;

	/* Prioritize from operators everything, that has
	 * pri>pri_of_the_operator */
	for (i=0;i<meqcount;i++) 
	  if (meq[i].rpn.type==oper && meq[i].operpri>0) {
		  /* Prioritize backwards */
		  for (j=i-1;j>=0 && meq[j].priority>meq[i].priority;j--)
		    meq[j].priority+=meq[i].operpri;
		  /* Prioritize forwards */
		  for (j=i+1;j<meqcount && meq[j].priority>meq[i].priority;j++)
		    meq[j].priority+=meq[i].operpri;
	  }
	
	/* Prioritize the particular operators*/
	for (i=0;i<meqcount;i++) {
		if (meq[i].rpn.type==oper)
		  meq[i].priority+=meq[i].operpri;
	}
	/* Prioritize forward +1 for FUNC_POWER (handle 2^3^4) 
	 * This shouldn't break all other priorities, as it only
	 * raises priority of things, that are already higher then
	 * the priority of the POWER operator */
	for (i=0;i<meqcount;i++) {
		if (meq[i].rpn.type==oper && meq[i].rpn.u.funcval==FUNC_POWER)
		  for (j=i+1;j<meqcount && meq[j].priority>=meq[i].priority;j++)
		    meq[j].priority++;
	}
	/* Convert operators to ordinary function */
	for (i=0;i<meqcount;i++) {
		if (meq[i].rpn.type==oper)
		  meq[i].rpn.type=function;
	}
}


/* Converts a prioritized array to Reverse Polish Notation (at least I hope
 * it is) stack. 
 * 
 * Meq is now prioritized after konvert_to_meq and meq_prioritize,
 * so we just 'sort' it following way:
 * for item in meq[]:
 *   if operstack empty - put on operstack
 *   else flush everything with higher priority from operstack to rpnstack
 *   put item on operstack
 * flush operstack to rpn
 * 
 * Input:
 * meq, meqcount
 * Output:
 * rpn, rpncount - global array, resulting rpn
 */
static void
konvert_to_rpn(void)
{
	Int16 i;

	opercount=0;
	for (i=0;i<meqcount;i++) {
		if (meq[i].rpn.type==discard) {
			/* The ';' terminates the everything processed till now,
			 * so we flush the operstack and append a ';' function */
			while (opercount>0)
			  rpn[rpncount++]=operstack[--opercount].rpn;
			meq[i].rpn.type=function;
			rpn[rpncount++]=meq[i].rpn; 
		} 
		else {
			if (opercount==0) 
			  operstack[opercount++]=meq[i];		       
			else {
				while (opercount>0 && 
				       operstack[opercount-1].priority>=meq[i].priority &&
					/* Handle case -2^3 -> -(2^3), another exception :-( */
					!(operstack[opercount-1].priority-meq[i].priority==1 &&
					  operstack[opercount-1].rpn.u.funcval==FUNC_NEGATE &&
					  meq[i].rpn.u.funcval==FUNC_POWER &&
						 meq[i].rpn.u.funcval==FUNC_POWER))
				  rpn[rpncount++]=operstack[--opercount].rpn;
				operstack[opercount++]=meq[i];
			}
		}
	}
	/* Flush the rest on the rpn stack */
	for (i=opercount-1;i>=0;i--) 
	  rpn[rpncount++]=operstack[i].rpn;	
}

void
delete_rpn(Trpn *myrpn,Int16 rpnsize,Boolean freerpn)
{
	Int16 i;
	
	for (i=0;i<rpnsize;i++)
	  if (myrpn[i].allocsize)
	    MemPtrFree(myrpn[i].u.data);
	if (freerpn && rpn!=myrpn)
	  MemPtrFree(myrpn);
}

Trpn *
allocate_rpn(Int16 elem)
{
	return (Trpn *)MemPtrNew(sizeof(Trpn)*elem);
}

/* Convert a string to CodeStack, for conversion is used a global
 * array, the result is then copied to a newly allocated space 
 */
CodeStack *
text_to_stack(char *buf,CError *err)
{
	Int16 i;
	CodeStack *result;
	
	meqcount=0;
	rpncount=0;
	if (!konvert_to_meq(buf,0,1)) {
		/* Free the alocated variables */
		for (i=meqcount-1;i>=0;i--)
		  rpn_delete(meq[i].rpn);
		*err = c_syntax;
		return NULL;
	}
	meq_prioritize();
	konvert_to_rpn();
	result = stack_new(rpncount);
	for (i=rpncount-1;i>=0;i--) 
	  stack_push(result,rpn[i]);
	
	
	*err = c_noerror;
	return result;
}

/* Error texts, corresponding to constants defined in konvert.h in
 * enum CError 
 */
char errtext[][20]={
	"No Error","Syntax error","Bad function","Missing argument",
	"Bad arg type",
	"Internal error","Bad mode selected","No result","Division by 0",
	"No such var","Bad result","Too deep recurs","Impossible calc."
};

char *
print_error(CError err)
{
	char *res;
	
	res = MemPtrNew(25);
	StrCopy(res,errtext[err]);
	return res;
}

Err
mlib_init_mathlib(void)
{
	Err error;

	rpn=allocate_rpn(MAX_RPNSIZE);
	meq=MemPtrNew(sizeof(*meq)*MAX_RPNSIZE);
	operstack=MemPtrNew(sizeof(*operstack)*MAX_OSTACK);
	
	error = SysLibFind(MathLibName, &MathLibRef);
	if (error)
	  error = SysLibLoad(LibType, MathLibCreator, &MathLibRef);
	ErrFatalDisplayIf(error, "Can't load Mathlib");
	error = MathLibOpen(MathLibRef, MathLibVersion);
	ErrFatalDisplayIf(error, "Can't open MathLib");
	return 0;
}

void mlib_close_mathlib(void)
{
	Err error;
	UInt16 usecount;

	MemPtrFree(rpn);MemPtrFree(meq);MemPtrFree(operstack);
	
	error = MathLibClose(MathLibRef,&usecount);
	ErrFatalDisplayIf(error, "Can't close MathLib");
	if (usecount == 0)
	  SysLibRemove(MathLibRef);
}
