/*
 *   $Id: stack.c,v 2.13 2000/09/21 21:08:17 ondra Exp $
 * 
 *   Scientific Calculator for Palm's and PilmPilot's.
 *   Copyright (C) 2000 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *  You can contact me at 'ondrap@penguin.cz'.
*/


#include <PalmOS.h>
#include <string.h>

#include "konvert.h"
#include "funcs.h"
#include "stack.h"
#include "calcDB.h"
#include "MathLib.h"

dbStackItem *
rpn_pack_record(Trpn item)
{
	dbStackItem *packed;	

	packed = MemPtrNew(item.allocsize + sizeof(*packed));
	packed->rpn = item;
	packed->datasize = item.allocsize;
	if (item.allocsize)
	  memcpy(packed->data,item.u.data,item.allocsize);

	return packed;
}

Trpn
rpn_unpack_record(dbStackItem *record)
{
	Trpn result;

	result = record->rpn;
	result.allocsize = record->datasize;
	if (record->datasize) {
		result.u.data = MemPtrNew(record->datasize);
		memcpy(result.u.data,record->data,record->datasize);
	}
	
	return result;
}

void
rpn_delete(Trpn item)
{
	if (item.allocsize)
	  MemPtrFree(item.u.stringval);
}

void
rpn_duplicate(Trpn *dest,Trpn source)
{
	*dest = source;
	if (source.allocsize) {
		dest->u.data = MemPtrNew(source.allocsize);
		memcpy(dest->u.data,source.u.data,source.allocsize);
	}
}	

CodeStack *
stack_unpack(dbPackedStack *packed)
{
	CodeStack *result;
	Trpn item;
	Int16 i,ptr;
	void *tmp;
	dbStackItem *stmp;
	
	result = stack_new(packed->header.count);
	
	ptr=0;
	for (i=0;i<packed->header.count;i++) {
		tmp = packed->data + ptr;
		stmp = (dbStackItem *)tmp;
		item = rpn_unpack_record(stmp);
		stack_push(result,item);
		ptr+=sizeof(dbStackItem)+stmp->datasize;
	}	
	return result;
}

/* Create one-block representation of stack */
dbPackedStack *
stack_pack(CodeStack *stack,Int16 *size)
{
	Int16 asize=0;
	Int16 i,ptr,isize;
	dbPackedStack *result;
	dbStackItem *item;	
	
	for (i=0;i<stack->size;i++)
	  asize+=sizeof(*item)+stack->stack[i].allocsize;
	result = MemPtrNew(asize+sizeof(*result));
	result->header.count = stack->size;
	result->header.orig_offset = asize;
	*size = asize+sizeof(*result);
	
	ptr = 0;
	for (i=0;i<stack->size;i++) {
		item = rpn_pack_record(stack->stack[i]);
		isize = sizeof(*item) + item->datasize;
		memcpy(result->data+ptr,item,isize);
		ptr+=isize;
		MemPtrFree(item);
	}
	return result;
}

void
stack_copy(CodeStack *dest,CodeStack *orig)
{
	Int16 i;
	if (dest->allocated < orig->size) 
	  ErrFatalDisplayIf(1,"Trying to copy larger stack into smaller.");
	for (i=0;i<orig->size;i++)
	  rpn_duplicate(&dest->stack[i],orig->stack[i]);
	dest->size = orig->size;
}

/* 
 * Get numeric value from RPN stack
 * perform automatic conversion, when needed,
 * reqtype is the Requested Type
 */
CError
stack_get_val(CodeStack *stack,void *arg1,rpntype reqtype)
{
	UInt32 intarg;
	double realarg;
	Complex *cplxarg;
	CError err=c_noerror;
	Trpn tmpitem;
	
	if (stack->size<1)
	  return c_noarg;

	tmpitem = stack_pop(stack);
	if (tmpitem.type == variable) {
		err = rpn_eval_variable(&tmpitem,tmpitem);
		if (err)
		  return err;
	}
	  
	if (reqtype==string) {
		char *tmp;
		
		if (tmpitem.type!=string) {
			rpn_delete(tmpitem);
			return c_badarg;
		}
		tmp = MemPtrNew(StrLen(tmpitem.u.stringval)+1);
		StrCopy(tmp,tmpitem.u.stringval);
		*((char **)arg1)=tmp;

		rpn_delete(tmpitem);
		return c_noerror;
	}

	/* Now follows numeric types */
	if (tmpitem.type!=integer &&
		tmpitem.type!=real &&
		tmpitem.type!=complex) {
		rpn_delete(tmpitem);
		return c_badarg;
	}
	
	if (reqtype==complex) {
		cplxarg = (Complex *) arg1;
		cplxarg->imag = 0.0;
		if (tmpitem.type==complex) 
		  *cplxarg = *(tmpitem.u.cplxval);
		else if (tmpitem.type==real) 
		  cplxarg->real = tmpitem.u.realval;
		else if (tmpitem.type==integer)
		  cplxarg->real = tmpitem.u.intval;
	}	
	else if (reqtype==real) {
		if (tmpitem.type==real)
		  *((double *)arg1)=tmpitem.u.realval;
		else if (tmpitem.type==integer)
		  *((double *)arg1)=tmpitem.u.intval;
		else if (tmpitem.type==complex) {
			if (IS_ZERO(tmpitem.u.cplxval->imag))
			  *((double *)arg1)=tmpitem.u.cplxval->real;
			else
			  err=c_badarg;
		}
	}
	else if (reqtype==integer) {
		switch (tmpitem.type) {
		 case integer:
			*((UInt32 *)arg1)=tmpitem.u.intval;
			break;
		 case complex:
			if (IS_ZERO(tmpitem.u.cplxval->imag))
			  realarg = tmpitem.u.cplxval->real;
			else {
				err = c_badarg;
				break;
			}
			/* PASS THROUGH, generates warning*/
		 case real:
			if (tmpitem.type==real)
			  realarg = tmpitem.u.realval;
			if (realarg<=4294967296.0 &&
				realarg>=0.0 ) {
				if (IS_ZERO(round(realarg)-realarg)) {
					intarg=(double)tmpitem.u.realval;
					*((UInt32 *)arg1)=intarg;
				}
				else 
				  err = c_badarg;
			}
			else
			  err = c_badarg;
			break;
		 default:
			err = c_badarg;
		}
	}
	else	  
	  err=c_badarg;
	
	rpn_delete(tmpitem);	
	return err;
}

CError
stack_get_val2(CodeStack *stack,
	     void *arg1,void *arg2,rpntype reqtype)
{
	CError err=c_noerror;
	
	if ((err=stack_get_val(stack,arg2,reqtype)))
	  return err;       
	err=stack_get_val(stack,arg1,reqtype);
	return err;;
}


CError
stack_add_val(CodeStack *stack,void *arg,rpntype reqtype)
{
	Trpn value;
	
	value.type=reqtype;
	value.allocsize=0;
	if (reqtype==real)
	  value.u.realval=*(double *)arg;
	else if (reqtype==integer)
	  value.u.intval=*(UInt32 *)arg;
	else if (reqtype==complex) {
		value.u.cplxval = MemPtrNew(sizeof(Complex));
		value.allocsize = sizeof(Complex);
		*(value.u.cplxval) = *(Complex *)arg;
	}
	else if (reqtype==string) {
		value.allocsize = StrLen((char *)arg)+1;
		value.u.stringval = MemPtrNew(value.allocsize);
		StrCopy(value.u.stringval,(char *)arg);
	}
	else
	  return c_badmode;
	
	stack_push(stack,value);
	return c_noerror;
}

CError
stack_item_type(CodeStack *stack,rpntype *type,Int16 itnum)
{
	Trpn tmp;
	CError err;
	
	if (stack->size<itnum)
	  return c_internal;
	tmp = stack->stack[stack->size - itnum -1];
	
	if (tmp.type!=variable) {
		*type = tmp.type;
		return c_noerror;
	}
	/* Get the type of the result of the variable */
	err = rpn_eval_variable(&tmp,tmp);
	if (err)
	  return err;
	
	*type = tmp.type;
	rpn_delete(tmp);
	
	return c_noerror;
}

CError
stack_compute(CodeStack *instack)
{
	CodeStack *varstack;
	Trpn result;
	Int16 retval=c_noerror;
	Trpn item;
	/* allocate rpnstack to maximum of input elements */
	varstack = stack_new(instack->size);
	while (instack->size) {
		item = stack_pop(instack);
		/* Handle non-operator types */
		if (item.type==integer || item.type==real ||
		    item.type==variable || item.type==funcempty
		    || item.type==string || item.type == complex) {

			stack_push(varstack,item);
		}
		else if (item.type==function) {
			Int16 fce=item.u.funcval;
			if ((retval=defined_funcs[fce].
			     function(defined_funcs[fce].funcnum,varstack)))
			  break;
		}
		else if (item.type==functext) {
			if ((retval=text_function(item.u.funcname,varstack)))
			  break;
		}
		else {
			retval=c_internal;
			break;
		}
	}       
	if (varstack->size<1) {
		stack_delete(varstack);
		return retval?retval:c_noresult;
	}
	result = stack_pop(varstack);		
	stack_delete(varstack);	
	stack_push(instack,result);

	return retval;
}

Trpn
stack_pop(CodeStack *stack)
{
	ErrFatalDisplayIf(stack->size==0,"Null stack size");
	stack->size--;
	
	return stack->stack[stack->size];
}
  
void
stack_push(CodeStack *stack,Trpn rpn)
{
	ErrFatalDisplayIf(stack->size==stack->allocated,
					  "RPN Stack overflow");
	stack->stack[stack->size]=rpn;
	stack->size++;
}

CodeStack *
stack_new(Int16 count)
{
	CodeStack *result;
	
	result = MemPtrNew(sizeof(*result));
	result->stack = MemPtrNew(sizeof(*result->stack)*count);
	result->size = 0;
	result->allocated = count;
	
	return result;
}

void
stack_delete(CodeStack *stack)
{
	while (stack->size) {
		stack->size--;
		rpn_delete(stack->stack[stack->size]);
	}
	
	MemPtrFree(stack->stack);
	MemPtrFree(stack);
}
