 /*   
 * $Id: prefs.c,v 2.5 2000/09/27 00:01:54 ondra Exp $
 * 
 *   Scientific Calculator for Palms and PilmPilots.
 *   Copyright (C) 1999,2000 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *  You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>

#include "calcrsc.h"
#include "calc.h"
#include "prefs.h"
#include "calcDB.h"

tPrefs calcPrefs;

void
prefs_read_preferences()
{
	int i;
	UInt16 size;
	
	size=sizeof(calcPrefs);
	if (PrefGetAppPreferences(APP_ID,PREF_DEFAULT,&calcPrefs,&size,
				  true)!=PREF_VERSION) {
		calcPrefs.input[0]='\0';
		calcPrefs.form=tdFormBasic;
		calcPrefs.insertPos=0;
		calcPrefs.selPosStart=calcPrefs.selPosEnd=0;
		calcPrefs.finBegin=false;
		calcPrefs.reducePrecision=true;
		calcPrefs.forceComplex=false;
		calcPrefs.defFrac=9;
		calcPrefs.stripZeros=true;
		calcPrefs.matchParenth=false;
		calcPrefs.sciMode=0;
		for (i=0;i<MAX_HISTORY;i++) 
		  calcPrefs.history[i][0]='\0';
		calcPrefs.int_mode=floating;
		calcPrefs.trigo_mode=radian;
		
		FrmAlert(tdAlertAbout);
		FrmHelp(tdHelp2);
		db_recompile_all();
	}
}
	
void 
prefs_save_preferences()
{
	PrefSetAppPreferences(APP_ID,PREF_DEFAULT,PREF_VERSION,&calcPrefs,
			      sizeof(calcPrefs),true);
}

Boolean
PreferencesHandleEvent(EventPtr event)
{
	Boolean handled=false;
	int controlID;
	FormPtr frm;
	int formId;
	ControlPtr knoflik;
	ListPtr list;
	
	switch (event->eType) {
	 case frmOpenEvent:
		frm=FrmGetActiveForm();		
		formId=frm->formId;
		FrmDrawForm(frm);
		
		switch (calcPrefs.int_mode) {
		 default:
		 case floating:
			knoflik=GetObjectPtr(prefFloating);
			break;
		 case decimal:
			knoflik=GetObjectPtr(prefDecimal);
			break;
		 case binary:
			knoflik=GetObjectPtr(prefBinary);
			break;
		 case octal:
			knoflik=GetObjectPtr(prefOctal);
			break;
		 case hexa:
			knoflik=GetObjectPtr(prefHexa);
			break;
		}
		CtlSetValue(knoflik,1);
		
		switch (calcPrefs.trigo_mode) {
		 case degree:
			knoflik=GetObjectPtr(prefDegree);
			break;
		 case grad:
			knoflik=GetObjectPtr(prefGrad);
			break;
		 case radian:
		 default:
			knoflik=GetObjectPtr(prefRadian);
			break;
		}
		CtlSetValue(knoflik,1);
		
		
		switch (calcPrefs.sciMode) {
		 case 0:
			knoflik=GetObjectPtr(prefNormal);
			break;
		 case 1:		
			knoflik=GetObjectPtr(prefSci);
			break;
		 case 2:
		 default:
			knoflik=GetObjectPtr(prefEng);
			break;
		}
		CtlSetValue(knoflik,1);
/* Set the precision */
		list = GetObjectPtr(prefPrecList);
		LstSetSelection(list,calcPrefs.defFrac);
		  {
			  static char tmp[4]; /* It's simpler than MemPtrNew/Free.. */
			  StrPrintF(tmp,"%d",calcPrefs.defFrac);
			  CtlSetLabel(GetObjectPtr(prefPrecPopup),tmp);
		  }
		/* Set the topItem of the list */
		LstMakeItemVisible(list,calcPrefs.defFrac);
/* Set the checkboxes */	
		knoflik = GetObjectPtr(prefReducePrecision);
		CtlSetValue(knoflik,calcPrefs.reducePrecision);
		
		knoflik=GetObjectPtr(prefStrip);
		CtlSetValue(knoflik,calcPrefs.stripZeros);
		
		knoflik=GetObjectPtr(prefForceComplex);
		CtlSetValue(knoflik,calcPrefs.forceComplex);
		
		knoflik=GetObjectPtr(prefParenth);
		CtlSetValue(knoflik,calcPrefs.matchParenth);

		handled=true;
		break;
	 case ctlSelectEvent:
		controlID=event->data.ctlSelect.controlID;  
		switch (controlID) {		       
		 case prefOK:
			if (CtlGetValue(GetObjectPtr(prefNormal)))
			  calcPrefs.sciMode=0;
			else if (CtlGetValue(GetObjectPtr(prefSci)))
			  calcPrefs.sciMode=1;
			else if (CtlGetValue(GetObjectPtr(prefEng)))
			  calcPrefs.sciMode=2;
			
			if (CtlGetValue(GetObjectPtr(prefDegree)))
			  calcPrefs.trigo_mode = degree;
			else if (CtlGetValue(GetObjectPtr(prefRadian)))
			  calcPrefs.trigo_mode = radian;
			else if (CtlGetValue(GetObjectPtr(prefGrad)))
			  calcPrefs.trigo_mode = grad;
			
			if (CtlGetValue(GetObjectPtr(prefFloating)))
			  calcPrefs.int_mode = floating;
			else if (CtlGetValue(GetObjectPtr(prefDecimal)))
			  calcPrefs.int_mode = decimal;
			else if (CtlGetValue(GetObjectPtr(prefBinary)))
			  calcPrefs.int_mode = binary;
			else if (CtlGetValue(GetObjectPtr(prefOctal)))
			  calcPrefs.int_mode = octal;
			else if (CtlGetValue(GetObjectPtr(prefHexa)))
			  calcPrefs.int_mode = hexa;
			
			calcPrefs.defFrac = LstGetSelection(GetObjectPtr(prefPrecList));
			calcPrefs.stripZeros=CtlGetValue(GetObjectPtr(prefStrip));
			calcPrefs.matchParenth=CtlGetValue(GetObjectPtr(prefParenth));
			calcPrefs.reducePrecision=CtlGetValue(GetObjectPtr(prefReducePrecision));
			calcPrefs.forceComplex=CtlGetValue(GetObjectPtr(prefForceComplex));

			SetDisplayMode(calcPrefs.sciMode,calcPrefs.defFrac,
				       !calcPrefs.stripZeros,calcPrefs.reducePrecision);
			FrmReturnToForm(0);
			FrmUpdateForm(calcPrefs.form,frmUpdateVars);
			handled=true;
			break;
		 case prefCancel:
			FrmReturnToForm(0);
			handled=true;
			break;
		}
	}	
	  
	return handled;
}

