/*
 *   $Id: result.c,v 2.26 2000/09/28 23:12:30 ondra Exp $
 * 
 *   Scientific Calculator for Palm's and PilmPilot's.
 *   Copyright (C) 2000 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *  You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>
#include "calcrsc.h"
#include "konvert.h"
#include "stack.h"
#include "funcs.h"
#include "defuns.h"
#include "result.h"
#include "prefs.h"
#include "display.h"
#include "ansops.h"
#include "calc.h"

/* This handle gets freed from the PalmOS at the end,
 * or it would have to be called from some other routine than AppHandler */
WinHandle txtWindow = NULL;
Coord position;
RectangleType resultBounds;
char *displayedText=NULL;

TresultPrefs resultPrefs;

char * popupDescription[] = {
	"Copy","Save as","DataMgr","GuessIt","->Degree","->Radian","->e^(ix)",
	 "->c()+isin()"
};
/***********************************************************************
 *
 * FUNCTION:     result_popup
 * 
 * DESCRIPTION:  Popup a list with operations related to the result field
 *
 * PARAMETERS:   None
 *
 * RETURN:       None
 *      
 ***********************************************************************/
void
result_popup(void) IFACE;
void
result_popup(void)
{
	Int16 selection,i;
	Int16 count=0;
	resSelection choices[SELECTION_COUNT];
	char *description[SELECTION_COUNT];
	ListPtr list; 
	
	choices[count++] = COPYRESULT;
	choices[count++] = DEFMGR;
	if (resultPrefs.ansType!=notype) {
		choices[count++] = VARSAVEAS;
		choices[count++] = GUESSIT;
		if (calcPrefs.trigo_mode == degree)
		  choices[count++] = TODEGREE;
		else if (calcPrefs.trigo_mode == radian)
		  choices[count++] = TORADIAN;
		if (resultPrefs.ansType==complex || calcPrefs.forceComplex) {
			choices[count++] = TOGONIO;
			if (calcPrefs.trigo_mode!=radian)
			  choices[count++] = TOCIS;
		}
	}

	for (i=0;i<count;i++)
	  description[i] = popupDescription[choices[i]];
	

	list = GetObjectPtr(calcResList);
	LstSetListChoices(list,description,count);
	LstSetHeight(list,count);
	selection=LstPopupList(list);

	switch (choices[selection]) {
	 case COPYRESULT:
		result_copy();
		break;
	 case VARSAVEAS:
		FrmPopupForm(varEntryForm);
		break;
	 case DEFMGR:
		FrmPopupForm(defForm);
		break;
	 case TODEGREE:
		ans_redisplay("todeg(ans)");
		break;
	 case GUESSIT:
		ans_guess();
		break;
	 case TOGONIO:
		ans_redisplay("togonio(ans)");
		break;
	 case TOCIS:
		ans_redisplay("tocis(ans)");
		break;
	 case TORADIAN:
		ans_redisplay("torad(ans)");
		break;
	}
}


/***********************************************************************
 *
 * FUNCTION:    result_copy
 * 
 * DESCRIPTION: Copy contents of Result field to clipboard
 *
 * PARAMETERS:  None
 *
 * RETURN:      None
 *      
 ***********************************************************************/
void result_copy(void) IFACE;
void
result_copy(void)
{
	if (displayedText)
	  ClipboardAddItem(clipboardText,displayedText,StrLen(displayedText));
}

/***********************************************************************
 *
 * FUNCTION:     result_init
 * 
 * DESCRIPTION:  Initialize resultField gadget, allocate txtWindow
 *
 * PARAMETERS:   id - resource id of the gadget
 *
 * RETURN:       None
 *      
 ***********************************************************************/
void result_init(Int16 id) IFACE;
void
result_init(Int16 id)
{
	UInt16 err;
	FormPtr frm = FrmGetActiveForm();
	UInt16 resIndex = FrmGetObjectIndex(frm,id);
	UInt16 size=0;
	Int16 version;
	void *prefData;
	
	FrmGetObjectBounds(frm,resIndex,&resultBounds);	
	if (!txtWindow) {
		txtWindow = WinCreateOffscreenWindow(resultBounds.extent.x,
											 resultBounds.extent.y,
											 screenFormat,&err);
		ErrFatalDisplayIf(err,"Error while creating txtWindow.");	
		position = 0;
		/* Read the stored text last time used in this gadget */
		version=PrefGetAppPreferences(APP_ID,PREF_RESULT,NULL,&size,false);
		if (version==PREF_RES_VERSION) {
			prefData = MemPtrNew(size);
			PrefGetAppPreferences(APP_ID,PREF_RESULT,prefData,&size,false);
			resultPrefs = *(TresultPrefs *)prefData;
			
			if (resultPrefs.formatType==plaintext)
			  result_print((char *)prefData+sizeof(resultPrefs));
			else
			  result_print_pow((char *)prefData+sizeof(resultPrefs));
			MemPtrFree(prefData);
		} else {
			result_print("0");
			resultPrefs.formatType=plaintext;
			resultPrefs.ansType=notype;
		}
	}
}

/***********************************************************************
 *
 * FUNCTION:     result_destroy
 * 
 * DESCRIPTION:  Free all structures asociated with resultField
 *
 * PARAMETERS:   None
 *
 * RETURN:       None
 *      
 ***********************************************************************/
void result_destroy(void) IFACE;
void
result_destroy(void)
{
	void *data;
	
	if (!txtWindow) /* The gadget was not initialized */
	  return;

	WinDeleteWindow(txtWindow,false);
	
	data = MemPtrNew(sizeof(resultPrefs)+StrLen(displayedText)+1);
	*(TresultPrefs *)data = resultPrefs;
	StrCopy((char *)data+sizeof(resultPrefs),displayedText);
	
	PrefSetAppPreferences(APP_ID,PREF_RESULT,PREF_RES_VERSION,
						  data,StrLen(displayedText)+1+sizeof(resultPrefs),
						  false);
	MemPtrFree(data);
	if (displayedText)
	  MemPtrFree(displayedText);
}

/***********************************************************************
 *
 * FUNCTION:     result_draw
 * 
 * DESCRIPTION:  Redraw a gadget, depending on the position variable
 *
 * PARAMETERS:   None
 *
 * RETURN:       None
 *      
 ***********************************************************************/
void result_draw(void) IFACE;
void
result_draw(void)
{
	RectangleType tmpbounds;
	static Boolean larr_drawn=false,rarr_drawn=false;
	MemHandle bmpRes;
	BitmapType *bmpPtr;
	Coord width,height;
	RectangleType re;
	WinHandle oldwindow;

	tmpbounds.topLeft.x = position;
	tmpbounds.topLeft.y = 0;
	tmpbounds.extent.x = resultBounds.extent.x;
	tmpbounds.extent.y = resultBounds.extent.y;
	
	WinCopyRectangle(txtWindow,NULL,&tmpbounds,
					 resultBounds.topLeft.x,resultBounds.topLeft.y,
					 winPaint);

	
	re.extent.x = 3;
	re.extent.y = 5;
	re.topLeft.y = resultBounds.topLeft.y+4;
	
	oldwindow = WinSetDrawWindow(txtWindow);
	WinGetWindowExtent(&width,&height);
	WinSetDrawWindow(oldwindow);
	
	
	if (position>3 && !larr_drawn) {
		bmpRes = DmGetResource(bitmapRsc,bmpLArrow);
		bmpPtr = MemHandleLock(bmpRes);
		WinDrawBitmap(bmpPtr,resultBounds.topLeft.x-4,resultBounds.topLeft.y+4);
		MemHandleUnlock(bmpRes);
		larr_drawn = true;
	}
	if (position <=3 && larr_drawn) {
		re.topLeft.x = resultBounds.topLeft.x-4;
		WinEraseRectangle(&re,0);
		larr_drawn = false;
	}
	
	if (position>=width-resultBounds.extent.x-3 && rarr_drawn) {
		re.topLeft.x = resultBounds.topLeft.x+resultBounds.extent.x+1;
		WinEraseRectangle(&re,0);
		rarr_drawn = false;
	}
	if (position<width-resultBounds.extent.x-3 && !rarr_drawn) {
				bmpRes = DmGetResource(bitmapRsc,bmpRArrow);
		bmpPtr = MemHandleLock(bmpRes);
		WinDrawBitmap(bmpPtr,resultBounds.topLeft.x+resultBounds.extent.x+1,
					  resultBounds.topLeft.y+4);
		MemHandleUnlock(bmpRes);
		rarr_drawn = true;
	}							
}

/***********************************************************************
 *
 * FUNCTION:     result_track
 * 
 * DESCRIPTION:  track the pen movement over result gadget
 *
 * PARAMETERS:   x,y - starting pen position
 *
 * RETURN:       true - pen movemtn handled by this gadget
 *               false - the tap wasn't for this gadget
 *      
 ***********************************************************************/
Boolean result_track(Coord x,Coord y) IFACE;
Boolean
result_track(Coord x,Coord y)
{
	Coord width,height;
	Coord oldx=x,oldy=y;
	Boolean penDown;
	Int16 diff;
	WinHandle oldwindow;
	
	if (!RctPtInRectangle(x,y,&resultBounds))
	  return false;
	
	oldwindow = WinSetDrawWindow(txtWindow);
	WinGetWindowExtent(&width,&height);
	WinSetDrawWindow(oldwindow);
	
	do {
		EvtGetPen(&x,&y,&penDown);
		if (oldx!=x && RctPtInRectangle(x,y,&resultBounds)) {
			diff = oldx-x;
			if (position+diff>=0
				&& position+diff+resultBounds.extent.x<=width) {
				position+=diff;
				result_draw();
			}
			oldx = x;
			oldy = y;
		}		
	}while(penDown);
	return true;
}

static void
result_clear(Coord width) IFACE;
static void
result_clear(Coord width)
{
	UInt16 err;
	
	if (txtWindow)
	  WinDeleteWindow(txtWindow,false);
	txtWindow = WinCreateOffscreenWindow(width,resultBounds.extent.y,
										 screenFormat,&err); 
	ErrFatalDisplayIf(err,"Cannot create window."); 
}

/***********************************************************************
 *
 * FUNCTION:     result_print_pow
 * 
 * DESCRIPTION:  Formats a a^b in a nice way on the display
 *
 * PARAMETERS:   text to display, should contain a '^' character
 *
 * RETURN:       None
 *      
 ***********************************************************************/
void result_print_pow(char *text) IFACE;
void
result_print_pow(char *text)
{
	char *expon;
	UInt16 baselen,explen;
	Coord expwidth,basewidth,width;
	FontID oldfont;
	WinHandle oldwindow;
	
	expon = StrChr(text,'^');
	if (!expon) {
		result_print(text);
		return;
	}
	resultPrefs.formatType = powtext;
	
	if (displayedText) 
	  MemPtrFree(displayedText);
	displayedText = MemPtrNew(StrLen(text)+1);
	StrCopy(displayedText,text); 
	
	expon++;	
	baselen = expon - text - 1;
	if (expon[0]=='(') {
		expon++;
		explen = StrLen(expon)-1;
	} 
	else
	  explen = StrLen(expon);		
	
	oldfont = FntSetFont(largeFont);
	basewidth = FntCharsWidth(text,baselen);
	FntSetFont(stdFont);
	expwidth = FntCharsWidth(expon,explen);
		
	if (basewidth+expwidth+8 < resultBounds.extent.x)
	  width = resultBounds.extent.x;
	else
	  width = basewidth+expwidth+8;
	
	result_clear(width);

	oldwindow = WinSetDrawWindow(txtWindow);
	WinEraseWindow();
	
	FntSetFont(largeFont);
	WinDrawChars(text,baselen,width-(basewidth+expwidth)-5,2);
	FntSetFont(stdFont);
	WinDrawChars(expon,explen,width-expwidth-3,0);
	
	position = 0;
	FntSetFont(oldfont);
	WinSetDrawWindow(oldwindow);
}
  
/***********************************************************************
 *
 * FUNCTION:     result_print
 * 
 * DESCRIPTION:  Prints a given text on display
 *
 * PARAMETERS:   None
 *
 * RETURN:       None
 *      
 ***********************************************************************/
void result_print(char *text) IFACE;
void
result_print(char *text)
{
	Coord width,chrwidth;	
	FontID oldfont;
	WinHandle oldwindow;
	
	resultPrefs.formatType = plaintext;
	
	if (displayedText)
	  MemPtrFree(displayedText);
	displayedText = MemPtrNew(StrLen(text)+1);
	StrCopy(displayedText,text); 
	/* Set the position for printing to 0 */
	oldfont = FntSetFont(largeFont);
	chrwidth = FntCharsWidth(text,StrLen(text));
	if (chrwidth +5 < resultBounds.extent.x)
	  width = resultBounds.extent.x;
	else
	  width = chrwidth + 5;
	
	result_clear(width);
	
	oldwindow = WinSetDrawWindow(txtWindow);
	WinEraseWindow();
	WinDrawChars(text,StrLen(text),width-chrwidth-2,0);
	WinSetDrawWindow(oldwindow);
	
	position = 0;
	FntSetFont(oldfont);
}

void
result_error(CError errcode)
{
	char *text;
	
	resultPrefs.ansType = notype;
	
	text = print_error(errcode);
	result_print(text);
	MemPtrFree(text);
}

/***********************************************************************
 *
 * FUNCTION:     result_set
 * 
 * DESCRIPTION:  display a given parameter according to it's type
 *
 * PARAMETERS:   Trpn item - the object to display
 *
 * RETURN:       None
 *      
 ***********************************************************************/
void result_set(Trpn item) IFACE;
void
result_set(Trpn item)
{
	char *text;
	CError err;
	
	if (item.type==variable) {
		err = rpn_eval_variable(&item,item);
		if (err) {
			result_error(err);
			return;
		}
		text = display_default(item);
		resultPrefs.ansType = item.type;
		rpn_delete(item);
	}
	else {
		resultPrefs.ansType = item.type;
		text = display_default(item);
	}
	result_print(text);
	MemPtrFree(text);
}
