/**
* MakeDocJ
*
* Compresses text files into a format that is ready to export to a Pilot
* and work with Rick Bram's PilotDOC reader.
*
*/

/**
 *	Format of PilotDOC file:
 *
 *	Database Header (databaseHeader)
 *	Record Index (recordIndex, 1 to n based on databaseHeader.numRecords)
 *	1 short (unknown as to why currently setting to 0)
 *	Document Header (documentHeader recordIndex[0])
 *	Text Records (recordIndex[1] to recordIndex[documentHeader.numRecords])
 *	Bookmarks Records (optional, recordIndex[documentHeader.numRecords+1] to recordIndex[databaseHeader.numRecords])
 *
 */

// Byte		= (1 byte)	byte
// Word		= (2 bytes)	short
// DWord	= (4 bytes)	int

import java.io.*;
import java.util.*;

import COM.stevesoft.pat.*;

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////                                  //////////////////////
/////////////////////      databaseHeader class        //////////////////////
/////////////////////                                  //////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class databaseHeader {
	public final static int TealDocID = 0x546c4463;	// 'TlDc'
	public final static int ReaderID = 0x52454164;	// 'REAd'
	public final static int TEXt = 0x54455874;		// 'TEXt'

	public String name = "";					// 32
	public short attribute = 0;					// 2 Word
	public short version = 0;					// 2 Word
	public int creationDate = 0;				// 4 DWord
	public int modificationDate = 0;			// 4 DWord
	public int lastBackupDate = 0;				// 4 DWord
	public int modificationNumber = 0;			// 4 DWord
	public int appInfoID = 0;					// 4 DWord
	public int sortInfoID = 0;					// 4 DWord
	public int typeID = 0;						// 4 DWord
	public int creatorID = 0;					// 4 DWord
	public int uniqueIDSeed = 0;				// 4 DWord
	public int nextRecordListID = 0;			// 4 DWord
	public short numRecords = 0;				// 2 Word
												// 78 bytes for Database Header

	public static int getSize() {
		return(78);
	}

	public void read(DataInput di) throws IOException {
		byte[] b = new byte[32];
		di.readFully(b);
		name = new String(b);
		int i = name.indexOf('\0');
		if(i >= 0) {
			name = name.substring(0,i);
		}
		attribute = di.readShort();
		version = di.readShort();
		creationDate = di.readInt();
		modificationDate = di.readInt();
		lastBackupDate = di.readInt();
		modificationNumber = di.readInt();
		appInfoID = di.readInt();
		sortInfoID = di.readInt();
		creatorID = di.readInt();
		typeID = di.readInt();
		uniqueIDSeed = di.readInt();
		nextRecordListID = di.readInt();
		numRecords = di.readShort();
	}

	public void write(DataOutput out) throws IOException {
		byte[] b = new byte[32];
		byte[] bName = name.getBytes();
		for(int x=0; x < b.length; x++) {
			if(x < bName.length) {
				b[x] = bName[x];
			}
			else {
				b[x] = 0;
			}
		}
		out.write(b);
		out.writeShort(attribute);
		out.writeShort(version);
		out.writeInt(creationDate);
		out.writeInt(modificationDate);
		out.writeInt(lastBackupDate);
		out.writeInt(modificationNumber);
		out.writeInt(appInfoID);
		out.writeInt(sortInfoID);
		out.writeInt(creatorID);
		out.writeInt(typeID);
		out.writeInt(uniqueIDSeed);
		out.writeInt(nextRecordListID);
		out.writeShort(numRecords);
	}

	public void dump() {
		char[] text = new char[4];
		System.err.println(">> Database Header <<");
		System.err.println("name = `" + name + "`");
		System.err.println("attribute = " + attribute);
		System.err.println("version = " + version);
		System.err.println("creationDate = " + creationDate);
		System.err.println("modificationDate = " + modificationDate);
		System.err.println("lastBackupDate = " + lastBackupDate);
		System.err.println("modificationNumber = " + modificationNumber);
		System.err.println("appInfoID = " + appInfoID);
		System.err.println("sortInfoID = " + sortInfoID);
		text[0] = (char)((creatorID >> 24) & 0xff);
		text[1] = (char)((creatorID >> 16) & 0xff);
		text[2] = (char)((creatorID >> 8) & 0xff);
		text[3] = (char)(creatorID & 0xff);
		System.err.println("creatorID = " + creatorID + " 0x" + Integer.toHexString(creatorID) + " `" + text + "`");
		text[0] = (char)((typeID >> 24) & 0xff);
		text[1] = (char)((typeID >> 16) & 0xff);
		text[2] = (char)((typeID >> 8) & 0xff);
		text[3] = (char)(typeID & 0xff);
		System.err.println("typeID = " + typeID + " 0x" + Integer.toHexString(typeID) + " `" + text + "`");
		System.err.println("uniqueIDSeed = " + uniqueIDSeed);
		System.err.println("nextRecordListID = " + nextRecordListID);
		System.err.println("numRecords = " + numRecords);
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////                                  //////////////////////
/////////////////////      recordIndex class           //////////////////////
/////////////////////                                  //////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class recordIndex {
	public int fileOffset = -1;	// 4 DWord
	public byte attribute = 0;	// 1 Byte
	public int uniqueID = 0;	// 3 Bytes
								// 8 bytes for Record Index

	public static int getSize() {
		return(8);
	}

	public void read(DataInput di) throws IOException {
		fileOffset = di.readInt();
		attribute = di.readByte();
		uniqueID = ((0xff & di.readByte()) << 16) + ((0xff & di.readByte()) << 8) + (di.readByte() & 0xff);
	}

	public void write(DataOutput out) throws IOException {
		out.writeInt(fileOffset);
		out.write(attribute);
		out.write((byte)(uniqueID >> 16));
		out.write((byte)(uniqueID >> 8));
		out.write((byte) uniqueID);
	}

	public void dump(int i) {
		System.err.println(">> Record Index " + i + " <<");
		System.err.println("fileOffset = " + fileOffset);
		System.err.println("attribute = " + attribute);
		System.err.println("uniqueID = " + uniqueID);
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////                                  //////////////////////
/////////////////////      documentHeader class        //////////////////////
/////////////////////                                  //////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class documentHeader {
	public final static short UNCOMPRESSED = 1;
	public final static short COMPRESSED = 2;
	public final static short textRecordSize = 4096;

	public short version = 0;					// 2 Word	1=plain text, 2=compressed
	public short spare1 = 0;					// 2 Word
	public int storyLen = 0;					// 4 DWord	in bytes, when decompressed
	public short numRecords = 0;				// 2 Word	text records
	public short recordSize = textRecordSize;	// 2 Word	usually 0x1000
	public int spare2 = 0;						// 4 DWord
												// 16 bytes for Document Header

	public static int getSize() {
		return(16);
	}

	public void read(DataInput di) throws IOException {
		version = di.readShort();
		spare1 = di.readShort();
		storyLen = di.readInt();
		numRecords = di.readShort();
		recordSize = di.readShort();
		spare2 = di.readInt();
	}

	public void write(DataOutput out) throws IOException {
		out.writeShort(version);
		out.writeShort(spare1);
		out.writeInt(storyLen);
		out.writeShort(numRecords);
		out.writeShort(recordSize);
		out.writeInt(spare2);
	}

	public void dump() {
		System.err.println(">> Document Header <<");
		System.err.println("version = " + version);
		System.err.println("spare = " + spare1);
		System.err.println("storyLen = " + storyLen);
		System.err.println("numRecords = " + numRecords);
		System.err.println("RecSize = " + recordSize);
		System.err.println("spare2 = " + spare2);
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////                                  //////////////////////
/////////////////////      bookMark class              //////////////////////
/////////////////////                                  //////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class bookMark {
	public String name = ""; 	// 16 Bytes	Bookmark name
	public int fileOffset = -1;	// 4 DWord	file offset
								// 20 bytes bookmark

	public static int getSize() {
		return(20);
	}

	public void read(DataInput di) throws IOException {
		byte[] b = new byte[16];
		di.readFully(b);
		name = new String(b);
		int i = name.indexOf('\0');
		if(i >= 0) {
			name = name.substring(0,i);
		}
		fileOffset = di.readInt();
	}

	public void write(DataOutput out) throws IOException {
		byte[] b = new byte[16];
		byte[] bName = name.getBytes();
		for(int x=0; x < b.length; x++) {
			if(x < bName.length) {
				b[x] = bName[x];
			}
			else {
				b[x] = 0;
			}
		}
		out.write(b);
		out.writeInt(fileOffset);
	}

	public void dump() {
		System.err.println(">> Bookmark <<");
		System.err.println("name = `" + name + "`");
		System.err.println("fileOffset = " + fileOffset);
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////                                  //////////////////////
/////////////////////      bookMarkHelper class        //////////////////////
/////////////////////                                  //////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class bookMarkHelper extends bookMark {
	public String searchText = "";
	public Regex searchRegex = null;

	public void prepareRegex() {
		searchRegex = new Regex(searchText);
		searchRegex.optimize();
	}

	public void dump() {
		super.dump();
		System.err.println("searchText = `" + searchText + "`");
		System.err.println("searchRegex = `" + searchRegex.toString() +"`");
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////                                  //////////////////////
/////////////////////      textBuffer class            //////////////////////
/////////////////////                                  //////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/**
* This little class handles in-memory documentHeader.textRecordSize byte buffers. It can
* compress, decompress, remove binary codes, duplicate \n codes
*
*/
class textBuffer {
	final static int DISP_BITS = 11;
	final static int COUNT_BITS = 3;
	final static int DEF_LEN = 6000;

	final static int EOLLF = 0;
	final static int EOLCR = 1;
	final static int EOLCRLF = 2;

	// the main storage buf during translations, this is the source buffer
	byte[] buf = null;
	// during translations, this it the destination buffer
	private byte[] destBuf = null;
	private int destIndex = 0;
	private boolean space = false;

	textBuffer() { this(DEF_LEN); }
	textBuffer(int len) { buf = new byte[len]; }
	public int length() { return buf.length; }

	/**
	* Remove the binary codes throw away really low ASCII or above '~' from the buffer
	* @return the resulting buffer length
	*/
	public int removeBinary() {
		destBuf = new byte[(buf.length*3)/2];
		int k, j;
		for(j = k = 0; j < buf.length; j++, k++) {
			// copy each byte
			destBuf[k] = buf[j];

			// throw away really low ASCII or above '~'
			if((destBuf[k] >= 0x00 && destBuf[k] < 0x09) || destBuf[k] > 0x7e) {
				k--;
			}
		}
		assign(destBuf, k);
		return k;
	}

	/**
	* Change all 0x0A into 0x0D, 0x0A pairs or 0x0A into 0x0D
	* @return the resulting buffer length
	*/
	public int convertEOL(final int eolType) {
		destBuf = new byte[(buf.length * 3) / 2];
		destIndex = 0;
		int k, j;
		for(j = k = 0; j < buf.length; j++, k++) {
			destBuf[k] = buf[j];
			if(destBuf[k] == 0x0A) {
				if(eolType == EOLCRLF) {
					destBuf[k++] = 0x0D;
					destBuf[k] = 0x0A;
				}
				else {	// EOLCR
					destBuf[k] = 0x0D;
				}
			}
		}
		assign(destBuf, k);
		return k;
	}

	/**
	* Decompress the buffer
	* action: make a new buffer
	*				run through the source data
	*				check the 4 cases:
	*					0,9...7F represent self
	*					1...8		escape n chars
	*					80...bf reference earlier run
	*					c0...ff	space+ASCII
	*
	* @return the resulting buffer length
	*/
	public int decompress() {
		// we "know" that all decompresses fit within documentHeader.textRecordSize, right?
		destBuf = new byte[(buf.length*10)];
		destIndex = 0;

		int i,j;
		for(j = i = 0; j < buf.length;) {
			int c;

			// take a char from the input buffer
			c = buf[j++] & 0xFF;

			// separate the char into zones: 0, 1...8, 9...0x7F, 0x80...0xBF, 0xC0...0xFF

			// codes 1...8 mean copy that many bytes; for accented chars & binary
			if(c > 0 && c < 9) {
				while(c-- > 0) {
					destBuf[destIndex++] = buf[j++];
				}
			}

			// codes 0, 9...0x7F represent themselves
			else if(c < 0x80) {
				destBuf[destIndex++] = (byte) c;
			}

			// codes 0xC0...0xFF represent "space + ascii char"
			else if(c >= 0xC0) {
				destBuf[destIndex++] = ' ';
				destBuf[destIndex++] = (byte) (c ^ 0x80);
			}

			// codes 0x80...0xBf represent sequences
			else {
				int m,n;
				c <<= 8;
				c += buf[j++] & 0xFF;
				m = (c & 0x3FFF) >> COUNT_BITS;
				n = c & ((1 << COUNT_BITS) - 1);
				n += 3;
				while(n-- > 0) {
					destBuf[destIndex] = destBuf[destIndex - m];
					destIndex++;
				}
			}
		}
		assign(destBuf, destIndex);
		return destIndex;
	}

	/**
	* compress the buffer
	* @return the resulting buffer length
	*/
	public int compress() {
		int hit = 0;		// points to a walking test hit; works upwards on successive matches
		int prevHit = 0; 	// previous value of hit
		int testHead = 0;	// current test string
		int testTail = 0;	// current walking pointer; one past the current test buffer
		int end = 0;		// 1 past the end of the input buffer

		// run through the input buffer
		hit = prevHit = testHead = 0;
		testTail = testHead + 1;
		end = buf.length;

		space = false;

		// make a destination buffer
		destBuf = new byte[(buf.length*3)/2];
		destIndex = 0;	// used to walk through the output buffer

		// make a source string, so we can search it quickly
		String src = new String(buf);
		String testStr = "";

		// loop, absorbing one more char from the input buffer on each pass
		for(hit = 0; testHead != end; testTail++) {
			// scan in the previous data for a match
			testStr = src.substring(testHead, testTail);
			hit = src.lastIndexOf(testStr, testHead - 1);

			// on a mismatch or end of buffer, issue codes
			if(hit == -1
				|| testHead - hit > ((1 << DISP_BITS) - 1)
				|| hit == testHead
				|| testTail - testHead > (1 << COUNT_BITS) + 2
				|| testTail == end) {
				// issue the codes

				// first, check for short runs
				if(testTail - testHead < 4) {
					issue(buf[testHead]);
					testHead++;
				}

				// for inter runs, issue a run-code
				else {
					// issue space char if required
					if(space) {
						destBuf[destIndex++] = ' ';
						space = false;
					}

					int dist = testHead - prevHit;
					int compound = (dist << COUNT_BITS) + testTail - testHead - 4;

					destBuf[destIndex++] = (byte) (0x80 + (compound >> 8));
					destBuf[destIndex++] = (byte) (compound & 0xFF);
					testHead = testTail - 1;
				}
				// start the search again
				prevHit = testHead;
			}
			// got a match
			else {
				prevHit = hit;
			}
			// when we get to the end of the buffer, don't inc past the end
			// this forces the residue chars out one at a time
			if(testTail == end) {
				testTail--;
			}
		}

		// clean up any dangling spaces
		if(space) {
			destBuf[destIndex++] = ' ';
		}

		// final scan to merge consecutive high chars together
		int i, k;
		byte[] cbuf = new byte[destBuf.length];
		for(i = k = 0; i < destIndex; i++, k++) {
			cbuf[k] = destBuf[i];
			// skip the run-length codes
			if(cbuf[k] >= 0x80 && cbuf[k] < 0xC0) {
				cbuf[++k] = destBuf[++i];
			}

			// if we hit a high char marker, look ahead for another
			else if(cbuf[k] == 1) {
				cbuf[k + 1] = destBuf[i + 1];
				while(i + 2 < destIndex && destBuf[i + 2] == 1 && cbuf[k] < 8) {
					cbuf[k]++;
					cbuf[k + buf[k]] = destBuf[i + 3];
					i += 2;
				}
				k += cbuf[k];
				i++;
			}
		}

		// free up resources
		destBuf = null;
		destIndex = 0;

		assign(cbuf, k);
		return k;
	}

	/**
	* Handle the details of writing a single
	* character to the compressed stream
	* @return the resulting buffer length
	*/
	private void issue(byte src) {
		// if there is an outstanding space char, see if
		// we can squeeze it in with an ASCII char
		if(space) {
			if(src >= 0x40 && src <= 0x7F) {
				destBuf[destIndex++] = (byte)(src ^ 0x80);
			}
			else {
				// couldn't squeeze it in, so issue the space char by itself
				// most chars go out simple, except the range 1...8, 0x80...0xFF
				destBuf[destIndex++] = ' ';
				if(src < 0x80 && (src == 0 || src > 8)) {
					destBuf[destIndex++] = src;
				}
				else {
					destBuf[destIndex++] = 1;
					destBuf[destIndex++] = src;
				}
			}
			// knock down the space flag
			space = false;
		}
		else {
			// check for a space char
			if(src == ' ') {
				space = true;
			}
			else {
				if(src < 0x80 && (src == 0 || src > 8)) {
					destBuf[destIndex++] = src;
				}
				else {
					destBuf[destIndex++] = 1;
					destBuf[destIndex++] = src;
				}
			}
		}
	}

	public void assign(byte[] b, int n) {
		buf = new byte[n];
		System.arraycopy(b, 0, buf, 0, n);
	}

	/**
	* if a cr appears before a lf, then remove the cr
	* if a cr appears in isolation, change to a lf
	* @return the resulting buffer length
	*/
	public int convertEOL() {
		destBuf = new byte[(buf.length*3)/2];
		int k, j;
		for(j = k = 0; j < buf.length; j++, k++) {
			// copy each byte
			destBuf[k] = buf[j];

			if(destBuf[k] == 0x0D) {
				// if next is LF, then drop it
				if(j < buf.length - 1 && buf[j + 1] == 0x0A) {
					k--;
				}
				else { // turn it into a LF
					destBuf[k] = 0x0A;
				}
			}
		}
		assign(destBuf, k);
		return k;
	}

}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////                                  //////////////////////
/////////////////////      MakeDocJ class              //////////////////////
/////////////////////                                  //////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

public class MakeDocJ {
	final static float version = (float) 2.0;

////////////////////////////////////////////////////////////////////////////////

	final public static void decode(final String src, final String dest, final int convertEOL, final boolean bookMarks) throws IOException {
		RandomAccessFile fin = null;
		FileOutputStream fout = null;
		try {
			fin = new RandomAccessFile(src, "r");
			fout = new FileOutputStream(dest);
		} catch (FileNotFoundException e) {
			System.err.println("the file " + src + " could not be found");
			return;
		} catch (IOException e) {
			System.err.println("the file " + dest + " could not be created");
			return;
		}

		databaseHeader dbHeader = new databaseHeader();
		dbHeader.read(fin);

		if(dbHeader.typeID != databaseHeader.ReaderID && dbHeader.typeID != databaseHeader.TealDocID
			|| dbHeader.creatorID != databaseHeader.TEXt) {
			System.err.println("file is unknown format");
			dbHeader.dump();
			return;
		}

		recordIndex[] ri = new recordIndex[dbHeader.numRecords];
		for(int i = 0; i < dbHeader.numRecords; i++) {
			ri[i] = new recordIndex();
			ri[i].read(fin);
		}
		short unknown = fin.readShort();

		documentHeader docHeader = new documentHeader();
		fin.seek(ri[0].fileOffset);
		docHeader.read(fin);

		if(docHeader.version != documentHeader.UNCOMPRESSED && docHeader.version != documentHeader.COMPRESSED) {
			System.err.println("WARNING: unknown file compression type: " + docHeader.version);
		}

		boolean compressed = (docHeader.version == documentHeader.COMPRESSED);

		/////////////////////////////////////////////////////////////
		// Handle the text (compressed or not)
		textBuffer textBuf;
		int dwRecLen;
		for(int i = 1; i <= docHeader.numRecords; i++) {
			System.out.print("\rconverting `" + dbHeader.name + "`: text record " + i + " of " + docHeader.numRecords);
			if(i == ri.length - 1) {
				// for the last, use the file len
				dwRecLen = (int) fin.length() - ri[i].fileOffset;
			}
			else {
				dwRecLen = ri[i + 1].fileOffset - ri[i].fileOffset;
			}
			fin.seek(ri[i].fileOffset);
			textBuf = new textBuffer(dwRecLen);
			fin.read(textBuf.buf);
			if(compressed) {
				textBuf.decompress();
			}
			if(convertEOL != textBuffer.EOLLF) {
				textBuf.convertEOL(convertEOL);
			}

			fout.write(textBuf.buf);
		}

		/////////////////////////////////////////////////////////////
		// Handle Bookmarks if any
		if(bookMarks && docHeader.numRecords + 1 < dbHeader.numRecords) {
			PrintWriter bout;
			try {
				bout = new PrintWriter(new FileOutputStream(dest + ".bm"));
			} catch (IOException e) {
				System.err.println("the file " + dest + ".bm" + " could not be created");
				return;
			}
			bookMark bm = new bookMark();
			for(int i = docHeader.numRecords + 1; i < dbHeader.numRecords; i++) {
				fin.seek(ri[i].fileOffset);
				bm.read(fin);
				bout.println(bm.name + "," + bm.fileOffset);
			}
			bout.close();
		}
		fin.close();
		fout.close();
	}

////////////////////////////////////////////////////////////////////////////////

	final public static void encode(final String src, final String dest, final String title, final boolean compress, final boolean binary, final boolean quiet, final int docID, final String bookMarkFile) throws IOException {
		final String tempBookMark = "$$BKMK$$.TMP";
		boolean processBookMarks = false;
		Vector bookMarks = null;
		RandomAccessFile bmf = null;

		FileInputStream fin = null;
		RandomAccessFile fout = null;
		documentHeader docHeader = new documentHeader();
		try {
			fin = new FileInputStream(src);
			docHeader.storyLen = fin.available();
			(new File(dest)).delete();
			fout = new RandomAccessFile(dest, "rw");
		}
		catch (FileNotFoundException e) {
			System.err.println("the file " + src + " could not be found");
			return;
		}
		catch (IOException e) {
			System.err.println("the file " + dest + " could not be created");
			return;
		}

		///////////////////////////////////////////////////
		// deal with bookmarks if necessary
		if(bookMarkFile != null) {
			bookMarkHelper bmh = null;
			processBookMarks = true;
			bookMarks = new Vector(10, 10);
			LineNumberReader finb;
			try {
				finb = new LineNumberReader(new FileReader(bookMarkFile));
			}
			catch(FileNotFoundException e) {
				System.err.println("the file " + bookMarkFile + " could not be found");
				return;
			}
			String line;
			StringTokenizer lineToken;
			while((line = finb.readLine()) != null) {
				lineToken = new StringTokenizer(line, ",");
				if(lineToken.countTokens() == 2) {
					bmh = new bookMarkHelper();
					bmh.name = lineToken.nextToken();
					bmh.searchText = lineToken.nextToken();
					bmh.prepareRegex();
					try {
						bmh.fileOffset = Integer.decode(bmh.searchText).intValue();
					}
					catch(NumberFormatException e) {
					}
					bookMarks.addElement(bmh);
				}
			}
			finb.close();
			try {
				(new File(tempBookMark)).delete();
				bmf = new RandomAccessFile(tempBookMark, "rw");
			}
			catch (IOException e) {
				System.err.println("the file " + tempBookMark + " could not be created");
				return;
			}
		}

		System.out.println("Saving `" + dest + "` as `" + title + "`," +
			(binary ? " binary," : "")  +
			(compress ? "" : " not") + " compressed");

		databaseHeader dbHeader = new databaseHeader();
		dbHeader.name = title;
		dbHeader.creationDate = (int) (System.currentTimeMillis() / 1000);
		dbHeader.modificationDate = (int) (System.currentTimeMillis() / 1000);
		dbHeader.creatorID = databaseHeader.TEXt;
		dbHeader.typeID = docID;

		docHeader.version = (compress ? documentHeader.COMPRESSED : documentHeader.UNCOMPRESSED);
		docHeader.recordSize = documentHeader.textRecordSize;

		docHeader.numRecords = (short)(docHeader.storyLen / documentHeader.textRecordSize);
		if(docHeader.numRecords * documentHeader.textRecordSize < docHeader.storyLen) {
			docHeader.numRecords++;
		}

		if(processBookMarks) {
			dbHeader.numRecords = (short)(docHeader.numRecords + 1 + bookMarks.size());
		}
		else {
			dbHeader.numRecords = (short)(docHeader.numRecords + 1);
		}
		dbHeader.write(fout);

		int bookMarkIndex = docHeader.numRecords + 1;	// this will only be used if we are bookmarking

		recordIndex[] ri = new recordIndex[dbHeader.numRecords];
		ri[0] = new recordIndex();
		ri[0].attribute = 0x40;		// not sure as to the whys for this value
		ri[0].uniqueID = 0x6f8000;	// not sure as to the whys for this value
		ri[0].write(fout);
		for(int i = 1; i < dbHeader.numRecords; i++) {
			ri[i] = new recordIndex();
			ri[i].fileOffset = 0;
			ri[i].attribute = 0x40;	// not sure as to the whys for this value
			ri[i].uniqueID = ri[i - 1].uniqueID + 1;
			ri[i].write(fout);
		}

		fout.writeShort(0);	// one more short.....don't know why
		ri[0].fileOffset = (int) fout.getFilePointer();

		docHeader.write(fout);

		// dump the whole story into the new file
		textBuffer buf = new textBuffer(documentHeader.textRecordSize);
		int nOrg = 0;
		int totalBytes = 0;
		int totalCompressedBytes = 0;
		byte[] bl = new byte[documentHeader.textRecordSize];

		int n;

		for(int recNum = 1; recNum <= docHeader.numRecords; recNum++) {
			ri[recNum].fileOffset = (int)fout.getFilePointer();
			n = nOrg = fin.read(bl);
			buf.assign(bl, n);
			if(nOrg == -1) {	// read error?
				break;
			}
			if(binary == false) {
				n = buf.removeBinary();
			}
			n = buf.convertEOL();
			if(processBookMarks) {	// bookmarks
				bmf.write(buf.buf);
			}
			if(compress) {
				n = buf.compress();
				totalCompressedBytes += n;
			}
			fout.write(buf.buf);
			totalBytes += nOrg;
			System.out.print("\rconverting record " + recNum + " of " + docHeader.numRecords);
			if(quiet == false && n > 0 && compress) {
				System.out.print(" original " + nOrg + " compressed to "
					+ n + " ratio: " + (int)(100. * n / nOrg + .5) + "%  ");
			}
		}

		bl = null;	// free resource

		// process the bookmarks
		if(processBookMarks) {
			System.out.println("");
			if(quiet == false) {
				System.out.println("Processing bookmarks");
			}
			bmf.seek(0);
			bookMarkHelper bmh;
			String line;
			int filePos = 0;
			int bookMarks2Match = bookMarks.size();
			while(bookMarks2Match > 0 && (line = bmf.readLine()) != null) {
				for(int bm = 0; bm < bookMarks.size(); bm++) {
					bmh = (bookMarkHelper)bookMarks.elementAt(bm);
					if(bmh.fileOffset == -1 && bmh.searchRegex.search(line)) {
						bookMarks2Match--;
						bmh.fileOffset = filePos + bmh.searchRegex.result().matchedFrom();
						if(quiet == false) {
							System.out.println("We found `" + bmh.searchText + "` bookmark");
						}
					}
				}
				filePos = (int)bmf.getFilePointer();
			}
			for(int bm = 0; bm < bookMarks.size(); bm++) {
				ri[bookMarkIndex + bm].fileOffset = (int)fout.getFilePointer();;
				((bookMarkHelper)bookMarks.elementAt(bm)).write(fout);
			}
		}

		// re-write the record index array, now with the correct file offsets
		fout.seek(databaseHeader.getSize());
		for(int i = 0; i < ri.length; i++) {
			ri[i].write(fout);
		}
		if(compress && quiet == false) {
			System.out.println("\nProcessed " + totalBytes +
				" bytes, compressed to " + totalCompressedBytes +
				" bytes, ratio: " + (int)(100. * totalCompressedBytes / totalBytes + .5) + "%");
		}
		fin.close();
		fout.close();
		if(processBookMarks) {
			bmf.close();
			(new File(tempBookMark)).delete();
		}
	}

////////////////////////////////////////////////////////////////////////////////

	public static void main(String args[]) {
		System.out.println("MakeDocJ v" + version);
		if(args.length < 3) {
			System.out.println("Usage: MakeDocJ [-options] ...");
			System.out.println("");
			System.out.println("MakeDocJ [-options] <text-file> <PilotDOC-file> <story-name>");
			System.out.println("  encode text file into PilotDOC format file");
			System.out.println("    -n builds the PilotDOC file without compression");
			System.out.println("    -r remove binary");
			System.out.println("    -q quiet (i.e. minimal reporting during encoding process)");
			System.out.println("    -t database ID is TealDoc instead of generic reader");
			System.out.println("    -i <database ID> four character database ID");
			System.out.println("");
			System.out.println("MakeDocJ -d [-options] <PilotDOC-file> <text-file> [<bookmark-file>]");
			System.out.println("  decode PilotDOC format file into the text file");
			System.out.println("    -1 end of line is CR/LF (i.e. DOS)");
			System.out.println("    -2 end of line is CR    (i.e. MAC)");
			System.out.println("    -b generate a bookmark file (<text-file>.bm) if there are any bookmarks");
			System.out.println("    -d decodes the PilotDOC file into a text file");
			return;
		}

		int argNum = 0;
		boolean decode = false;	// default to compress
		boolean binary = true;	// keep binary data
		boolean quiet = false;	// report mucho information
		boolean compress = true;	// we are compressing
		boolean bookmarks = false;	// don't generate .bm file
		int eolType = textBuffer.EOLLF;	// UNIX => LF
		int docID = databaseHeader.ReaderID;	// 'REAd' database creator ID, generic reader

		while(args[argNum].charAt(0) == '-' || args[argNum].charAt(0) == '\\' || args[argNum].charAt(0) == '/') {
			if(args[argNum].charAt(1) == '1') {	// DOS => CR/LF
				eolType = textBuffer.EOLCRLF;
			}
			if(args[argNum].charAt(1) == '2') {	// MAC => CR
				eolType = textBuffer.EOLCR;
			}
			if(args[argNum].charAt(1) == 'b') {
				bookmarks = true;
			}
			if(args[argNum].charAt(1) == 'd') {
				decode = true;
			}
			if(args[argNum].charAt(1) == 'i') {
				argNum++;
				byte[] id = args[argNum].getBytes();
				if(id.length != 4) {
					System.err.println(args[argNum] + " is incorrect length, must be 4 characters");
					System.exit(1);
				}
				else {
					docID = ((0xff & id[0]) << 24) + ((0xff & id[1]) << 16) + ((0xff & id[2]) << 8) + (0xff & id[3]);
				}
			}
			if(args[argNum].charAt(1) == 'n') {
				compress = false;
			}
			if(args[argNum].charAt(1) == 'q') {
				quiet = true;
			}
			if(args[argNum].charAt(1) == 'r') {
				binary = false;
			}
			if(args[argNum].charAt(1) == 't') {
				docID = databaseHeader.TealDocID;
			}
			argNum++;
		}

		try {
			if(decode) {
				decode(args[argNum], args[argNum + 1], eolType, bookmarks);
			}
			else {
				String bookMarkFile = null;
				try {
					bookMarkFile = args[argNum + 3];
				}
				catch(java.lang.ArrayIndexOutOfBoundsException e) {
					bookMarkFile = null;
				}
				encode(args[argNum], args[argNum + 1], args[argNum + 2], compress, binary, quiet, docID, bookMarkFile);
			}
		}
		catch (Exception e) {
			System.out.println(e.toString());
		}
	}
}

