/**************************************************************************

	Yanoff - the free UseNet news reader for the Palm Computing Platform
    Copyright (C) 1999 Matthias Jordan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Reach the author via 
	email: mjordan@sensenet.wirepool.free.de

**************************************************************************/
//
//
// artdb.c
//
// article database routines for use with yanoff
// Matthias Jordan, 4-1999
//
//


#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include <System/DataMgr.h>


#include "general.h"
#include "yanoff.h"
#include "artdb.h"


#define artdb_error 0
#define artdb_ok 1


char articledb_name[] = {"NewsArts"};

DmOpenRef artdb = NULL;


DmOpenRef ArtOpenArticles(void)
{
	if (artdb == NULL)
	{
	    artdb = NPOpenDB(articledb_name);
	}
	return artdb;
} /* ArtOpenArticles */



void ArtCloseArticles(void)
{
	if (artdb != NULL)
	{
		DmCloseDatabase(artdb);
	}
} /* ArtOpenArticles */





int ArtReadArticle(int pos, struct articledb_item *art)
{
    VoidHand h;
    char *p;

    h = DmQueryRecord(artdb, pos);
    if (h == 0)
    {
        return 1;
    }
    p = (char *)MemHandleLock(h);

    art->flags = *(UInt *)p;
    p += sizeof(UInt);

    art->rcvdate = *(ULong *)p;
    p += sizeof(ULong);

    art->xpostings = *(UInt *)p;
    p += sizeof(UInt);


    art->from = p;
    p += StrLen(p) + 1;

    art->date = p;
    p += StrLen(p) + 1;

    art->subject = p;
    p += StrLen(p) + 1;

    art->newsgroups = p;
    p += StrLen(p) + 1;

    art->msgid = p;
    p += StrLen(p) + 1;

	
    art->body = p;

    MemHandleUnlock(h);
    return 0;
} /* ArtReadArticle */



//
//
// Create new article in database and append article header
// without body. This is appended by NPReadArticle2DB
// the database chunk is left open and its pointer, handle, offset and OpenRef
// are given back to NPReadArticle2DB
//
// Called by latter function without the body-part of *art (art->body == NULL)
//
// Returns 0 in case of error
// Returns 1 in case of success

int ArtAppendArticleHeader(struct articledb_item *art, UInt *pos)
{
    UInt from_len,
        date_len,
        subject_len,
        newsgroups_len,
        msgid_len,
        header_len;
    char *p;
    VoidHand h;
	Word bodypos, size;
	

    *pos = -1; // Append chunk to database
    size = 0;

    from_len = StrLen(art->from) + 1;
    date_len = StrLen(art->date) + 1;
    subject_len = StrLen(art->subject) + 1;
    newsgroups_len = StrLen(art->newsgroups) + 1;
    msgid_len = StrLen(art->msgid) + 1;

    header_len = sizeof(ULong) + sizeof(UInt);
    header_len += from_len + date_len + subject_len 
	    		+ newsgroups_len + msgid_len + 1; // One byte for empty body


	// Allocate the largest chunk available
    size = ART_SIZE_START;
    do
    {
        h = DmNewRecord(artdb, pos, size);
        if (h == 0)
        {
            size -= 1000;
        }
    }
    while ((h == 0) && (size > header_len));
    if (h == 0)
    {
        // Mem low
        FrmCustomAlert(alertID_dum, "Memory low", " ", " ");
        return 1;
    }
    p = (char *)MemHandleLock(h);

    DmWrite(p, 0, &(art->flags), sizeof(UInt));
    bodypos = sizeof(UInt);

    DmWrite(p, bodypos, &(art->rcvdate), sizeof(ULong));
    bodypos += sizeof(ULong);

    DmWrite(p, bodypos, &(art->xpostings), sizeof(UInt));
    bodypos += sizeof(UInt);

    DmStrCopy(p, bodypos, art->from);
    bodypos += from_len;

    DmStrCopy(p, bodypos, art->date);
    bodypos += date_len;

    DmStrCopy(p, bodypos, art->subject);
    bodypos += subject_len;

    DmStrCopy(p, bodypos, art->newsgroups);
    bodypos += newsgroups_len;

    DmStrCopy(p, bodypos, art->msgid);
    bodypos += msgid_len;
	
	DmStrCopy(p, bodypos, "");
	bodypos++;

    MemHandleUnlock(h);
    DmReleaseRecord(artdb, *pos, 1);
	ArtResizeArticle(*pos, bodypos);
    return 0;
} /* ArtAppendArticleHeader */










//
//
// Copy message body from buffer to art structure - without
// the annoying '\r'
// len is size of the complete chunk dest
//
// Returns pointer to destination
//

char *NPCopyBody2DB(char *dest, Word destpos, Word len, char *data)
{
    len -= 1; // save place for 0-terminator
    while ((*data) && (destpos < len))
    {
        if (*data == '\r')
        {
            // skip CR - do nothing
        }
        else
        {
            DmSet(dest, destpos++, 1, *data);
        }
        data++;
    }
    DmSet(dest, destpos, 1, 0);
    return &dest[destpos];
} /* NPCopyBody2DB */






int ArtAppendArticleBody(int pos, char *buf)
{
	VoidHand h;
	char *p, *end;
	Word bodypos,
		len;
	
	
	len = BUF_START_LEN;
	do
	{
		h = ArtResizeArticle(pos, len);
		if (h == NULL)
		{
			len -= BUF_LEN_DOWN;
		}
	}
	while ((h == NULL) && (len > 1024 + BUF_LEN_DOWN)); // arbitrary floor value

	if (h == NULL)
	{
		return 1;
	}
	
	h = DmGetRecord(artdb, pos);
	if (h == 0)
	{
		return 2;
	}
	p = (char *) MemHandleLock(h);
	bodypos = 8; // 2 + 4 + 2 
	bodypos += StrLen(p + bodypos) + 1;
	bodypos += StrLen(p + bodypos) + 1;
	bodypos += StrLen(p + bodypos) + 1;
	bodypos += StrLen(p + bodypos) + 1;
	bodypos += StrLen(p + bodypos) + 1;
	end = NPCopyBody2DB(p, bodypos, len, buf);
	MemHandleUnlock(h); 
	DmReleaseRecord(artdb, pos, 1);
	ArtResizeArticle(pos, end - p + 1);
	return 0;
} /* ArtAppendArticleBody */






//
//
// Change an articles xposting count with ID uid
// 

UInt ArtChangeArticleXP(ULong uid, int dcount)
{
	VoidHand h;
    char *p;
    UInt *xp, xpdum, pos;
	
	if (DmFindRecordByID(artdb, uid, &pos) != 0)
	{
		return -1;
	}
	h = DmQueryRecord(artdb, pos);
	if (h == 0)
	{
		return -1;
	}
    p = (char *)MemHandleLock(h);
    xp = (UInt *)(p + sizeof(UInt) + sizeof(ULong));
    xpdum = (*xp) + dcount;
    DmWrite(p, sizeof(UInt) + sizeof(ULong), &xpdum, sizeof(UInt));
    MemHandleUnlock(h);
	return xpdum;
} /* ArtChangeArticleXP */






//
//
// Delete Article from NewsArts
//
// First reduce appropriate xposting-count. If it reaches zero: remove article.
//

int ArtDeleteArticle(ULong uid)
{
    UInt xpdum, pos;


	if (DmFindRecordByID(artdb, uid, &pos) != 0)
	{
		return 1;
	}

	xpdum = ArtChangeArticleXP(uid, -1);	

    if (xpdum == 0)
    {
        DmRemoveRecord(artdb, pos);
    }
    return 0;
} /* ArtDeleteArticle */



//
//
// Find article with unique ID uid in the global article database
//

int ArtSeekArticle(ULong uid, UInt *pos)
{
	return DmFindRecordByID(artdb, uid, pos);
} /* ArtSeekArticle */



VoidHand ArtResizeArticle(UInt pos, Word len)
{
	return DmResizeRecord(artdb, pos, len);
} /* ArtResizeArticle */


ULong ArtGetArticleUID(UInt pos)
{
	ULong uid;
	
	if (DmRecordInfo(artdb, pos, NULL, &uid, NULL))
	{
		return 0;
	}
	return uid;
} /* ArtGetArticleUID */



//
//
// Set an article flag on the article with unique ID uid
// value -1: toggle, value 0: set to 0, value 1: set to 1
//

int ArtSetArticleFlag(ULong uid, int flag, int value)
{
	VoidHand h;
	UInt *f, f2;
	UInt pos;
	
	if (DmFindRecordByID(artdb, uid, &pos) != 0)
	{
		return 1;
	}
	h = DmQueryRecord(artdb, pos);
	if (h == 0)
	{
		return 2;
	}
	f = (UInt *) MemHandleLock(h);
	f2 = *f;
	if (value == -1)
	{
		f2 ^= flag;
	}
	else if (value == 1)
	{
		f2 |= flag;
	}
	else if (value == 0)
	{
		f2 &= (~flag);
	}
	DmWrite(f, 0, &f2, sizeof(f2));
	MemHandleUnlock(h);
	return 0;
} /* ArtSetArticleFlag */



UInt ArtArticlesCount(void)
{
	return DmNumRecords(artdb);
} /* ArtArticlesCount */



//
//
// Init article data to NULL values
//

void ArtInitArticleData(struct articledb_item *a)
{
    a->flags = 0;
    a->rcvdate = 0;
    a->xpostings = 0;
    a->newsgroups = NULL;
    a->from = NULL;
    a->subject = NULL;
    a->date = NULL;
    a->body = NULL;
} /* ArtInitArticleData */



//
//
// Free article data based on article structure
// Does NOT free the article struct itself!
//

void ArtFreeArticleData(struct articledb_item *a)
{
    FreePointer((Ptr) a->newsgroups);
    FreePointer((Ptr) a->from);
    FreePointer((Ptr) a->subject);
    FreePointer((Ptr) a->date);
    FreePointer((Ptr) a->body);
    ArtInitArticleData(a);
} /* ArtFreeArticleData */



void ArtPurgeArticles(ULong deltadays)
{
	UInt count, act;
	struct articledb_item art;
	ULong max;
	
	max = TimGetSeconds() - (deltadays * 86400);
	count = ArtArticlesCount();
	for (act=0; (act < count);)
	{
		ArtReadArticle(act, &art);
		if ((art.rcvdate < max) && !(art.flags & yfArticleFlagKeep))
		{
			DmRemoveRecord(artdb, act);
			count--;
		}
		else
		{
			act++;
		}
	}
} /* ArtPurgeArticles */

