/**************************************************************************

	Yanoff - the free UseNet news reader for the Palm Computing Platform
    Copyright (C) 1999 Matthias Jordan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Reach the author via 
	email: mjordan@sensenet.wirepool.free.de

**************************************************************************/
//
//
// frmart.c
//
// Handle the article form
// Matthias Jordan, 4-1999
//
//


#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>


#include "yanoff.h"
#include "general.h"
#include "ngroup.h"
#include "network.h"

#include "frmartlist.h"
#include "frmart.h"

#include "mainmenu.h"




int top_line, old_top_line;
FontID ArtFont = stdFont;
int artmode = artModeBody;
struct articledb_item art;
int active_article;
char *header;
Word artpagesize;




static void ScrollArtField(Short linesToScroll)
{
	Short min, max, value, pageSize;
	FieldPtr fld;
	ScrollBarPtr bar;
	RectangleType r;
	Word textheight, fieldheight, next_top;
	
    fld = GetObjectPtr(fldArtString);
 	FldGetBounds(fld, &r);
	fieldheight = r.extent.y / FntLineHeight();
	textheight = FldCalcFieldHeight(FldGetTextPtr(fld), r.extent.x);
	next_top = GetNewTopline(top_line, textheight, fieldheight, linesToScroll);

	if (linesToScroll < 0)
	{
        FldScrollField(fld, top_line - next_top, up);
	}		
	else if (linesToScroll > 0)
	{
		FldScrollField (fld, next_top - top_line, down);
	}

	top_line = next_top;
    bar = GetObjectPtr(sbArticle);
    SclGetScrollBar(bar, &value, &min, &max, &pageSize);
    SclSetScrollBar(bar, top_line, min, max, pageSize);
} /* ScrollArtField */







void ConstructHeader(char **head)
{
	VoidHand h;
	UInt size;
	
	size = StrLen(art.subject) + StrLen(art.from) + StrLen(art.date)
		+ StrLen(art.newsgroups) + StrLen(art.msgid) + 49 + 1;
	h = MemHandleNew(size);
	if (h == 0)
	{
		*h = NULL;
		return;
	}
	*head = (char *) MemHandleLock(h);
	StrCopy(*head, "From: ");
	StrCat(*head, art.from);
	StrCat(*head, "\n\nSubject: ");
	StrCat(*head, art.subject);
	StrCat(*head, "\n\nDate: ");
	StrCat(*head, art.date);
	StrCat(*head, "\n\nMsgID: ");
	StrCat(*head, art.msgid);
	StrCat(*head, "\n\nNewsgroups: ");
	StrCat(*head, art.newsgroups);
	StrCat(*head, "\n");
} /* ConstructHeader */





void InitArtField(int show_mode)
{
	FieldPtr fld;
	FormPtr frm;
	FieldAttrType attr;
	ControlType *btnH, *btnS;
	Word fieldheight,
		textheight;
	RectangleType r;
	char *fieldtext;
	int top_dum;
	
    btnS = GetObjectPtr(btnSelect);
    if (art.flags & yfArticleFlagMarked)
    {
    	CtlSetLabel(btnS, "s");
    }
    else
    {
    	CtlSetLabel(btnS, "S");
    }

    btnS = GetObjectPtr(btnKeep);
    if (art.flags & yfArticleFlagKeep)
    {
    	CtlSetLabel(btnS, "k");
    }
    else
    {
    	CtlSetLabel(btnS, "K");
    }

    if (artmode == show_mode)
    {
    	// show_mode did not change - so update only Select button
//    	CtlDrawControl(btnS);
    	return;
    }
    btnH = GetObjectPtr(btnHeader);
	if (show_mode == artModeBody)
	{
		fieldtext = art.body;
		if (!(*art.body) && NPOnlineMode())
		{
			// Body empty - so get it while we are online
			NPInvokePoll(pollSingleBody, active_article);
			return;
		}
		CtlSetLabel(btnH, "Header");
	}
	else if (header != NULL)
	{
		fieldtext = header;
		CtlSetLabel(btnH, "Body");
	}
	else
	{
		fieldtext = "Header not available";
	}

	artmode = show_mode;

	top_dum = top_line;
    top_line = old_top_line;
    old_top_line = top_dum;

//	top_line = 0;

    fld = GetObjectPtr(fldArtString);
    FldEraseField(fld);
	FldSetTextPtr(fld, fieldtext);
	FldRecalculateField(fld, 0);
	FldSetScrollPosition(fld, 0);
	if (top_line)
	{
	    FldScrollField(fld, top_line, down);
	}

	frm = FrmGetActiveForm();
	NPSetFormTitle(frm, art.subject, 0);

 	FldGetBounds(fld, &r);
	fieldheight = r.extent.y / FntLineHeight();
	textheight = FldCalcFieldHeight(fieldtext, r.extent.x);
    FldGetAttributes(fld, &attr);


	artpagesize = fieldheight - 1;
	if (textheight > fieldheight)
	{
	    attr.hasScrollBar = 1;
		SclSetScrollBar(GetObjectPtr(sbArticle), top_line, 0,  textheight - fieldheight, artpagesize);
	}
	else
	{
	    attr.hasScrollBar = 0;
		SclSetScrollBar(GetObjectPtr(sbArticle), 0, 0,  0, 0);
	}
    FldSetAttributes(fld, &attr);
	RefreshForm();
} /* InitArtField */





void CloseArtField(void)
{
	FieldPtr fld;
	
    fld = GetObjectPtr(fldArtString);
    FldEraseField(fld);
	MemPtrFree(header);
} /* InitArtField */






Boolean HandleArticle(EventPtr event)
{
  int handled = 0;

    switch (event->eType)
    {
		case frmUpdateEvent:
		{
			NGReadArticle(active_article, &art);
			InitArtField(event->data.frmUpdate.updateCode);
			handled = 1;
			break;
		}

        case frmOpenEvent:
        {
			InitArtField(*(art.body) ? artModeBody : artModeHeader);
            handled = 1;
            break;
        }

		case sclRepeatEvent:
		{
			ScrollArtField(event->data.sclRepeat.newValue -
						event->data.sclRepeat.value);
			break;
		}

        case menuEvent:
        {
			handled = HandleMainMenu(event);
			break;
        }
		
		case keyDownEvent:
		{
			switch (event->data.keyDown.chr)
			{
				case backspaceChr:
				{
					CloseArtField();
					NPExitForm();
					FrmUpdateForm(frmArtList, 0);
					handled = 1;
					break;
				}
				
				case pageUpChr:
				{
					ScrollArtField(-artpagesize);
					handled = 1;
					break;
				}

				case pageDownChr:
				{
					ScrollArtField(artpagesize);
					handled = 1;
					break;
				}
				
				default:
			}
			break;
		}
		
		case ctlSelectEvent:
		{
			switch (event->data.ctlSelect.controlID)
			{
				case btnDone:
				{
					CloseArtField();
					NPExitForm();
					FrmUpdateForm(frmArtList, 0);
					handled = 1;
					break;
				}
				
				case btnHeader:
				{
					if (artmode == artModeBody)
					{
						FrmUpdateForm(frmArticle, artModeHeader);
					}
					else
					{
						FrmUpdateForm(frmArticle, artModeBody);
					}
					handled = 1;
					break;
				}
				
				case btnSelect:
				{
					NGSetArticleFlag(active_article, yfArticleFlagMarked, -1);
					FrmUpdateForm(frmArticle, artmode);
					handled = 1;
					break;
				}

				case btnKeep:
				{
					NGSetArticleFlag(active_article, yfArticleFlagKeep, -1);
					FrmUpdateForm(frmArticle, artmode);
					handled = 1;
					break;
				}
			}
		}

        default:
    }
    return handled;
} /* HandleArticle */






void ArtFGotoArticle(UInt n)
{
	active_article = n;
	artmode = -1;
	top_line = 0;
	old_top_line = 0;
	NGReadArticle(n, &art);
	ConstructHeader(&header);
	NPGotoForm(frmArticle);
} /* ArtFGotoArticle */
