/**************************************************************************

	Yanoff - the free UseNet news reader for the Palm Computing Platform
    Copyright (C) 1999 Matthias Jordan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Reach the author via 
	email: mjordan@sensenet.wirepool.free.de

**************************************************************************/
//
//
// frmartlist.c
//
// Handle the article list form
// Matthias Jordan, 4-1999
//
//


#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include "yanoff.h"
#include "general.h"
#include "ngroup.h"

#include "frmartlist.h"
#include "frmart.h"

#include "mainmenu.h"


int top_article,
	artlistpagesize;
FontID ArtListFont = stdFont;






//
//
//  Init article list table
//
//



static void ArticlesListDrawItemString(CharPtr dum, short x, short y, 
		short width)
{
	Int titlelen;
	short titlewidth;
	Boolean stringfit;
	
	titlewidth = width;
	titlelen = StrLen(dum);
	FntCharsInWidth(dum, &titlewidth, &titlelen, &stringfit);
	if (stringfit)
	{
		WinDrawChars(dum, titlelen, x, y);
	}
	else
	{
		width -= (FntCharWidth('.') * 3);
		while (titlewidth > width)
		{
			titlewidth -= FntCharWidth(dum[--titlelen]);
		}
		WinDrawChars(dum, titlelen, x, y);
		x += titlewidth;
		WinDrawChars("..", 2, x, y);
	}
} /* ArticlesListDrawItemString */




#define namex 38
#define colsepx 2

static void ArticlesListDrawItem(VoidPtr table, Word row, Word column, 
	RectanglePtr bounds)
{
	Word recnum;
	char dum[41];
	short x, y;
	struct articledb_item a;
	
	recnum = TblGetRowID(table, row);
	if (recnum == -1)
	{
		// empty table entry
		return;
	}
	x = bounds->topLeft.x + 1;
	y = bounds->topLeft.y;
	FntSetFont(ArtListFont);
	NGReadArticle(recnum, &a);
	switch (column)
	{
		case 0:
		{
			// Chars possible: .:il!|
			WinEraseRectangle(bounds, 0);
			if (a.flags & yfArticleFlagMarked)
			{
				ArticlesListDrawItemString("!", x, y, bounds->extent.x);
			}
			if (a.flags & yfArticleFlagKeep)
			{
				ArticlesListDrawItemString("i", x+4, y, bounds->extent.x);
			}
			else
			{
				if (a.flags & yfArticleFlagRead)
				{
					ArticlesListDrawItemString(".", x+4, y, bounds->extent.x);
				}
			}
			if (*a.body)
			{
				ArticlesListDrawItemString("|", x+8, y, bounds->extent.x);
			}
 			break;
		}

		case 1:
		{
			StrNCopy(dum, a.from, 40);
			ArticlesListDrawItemString(dum, x, y, namex);

			StrNCopy(dum, a.subject, 40);			
			ArticlesListDrawItemString(dum, x + namex + colsepx, y, bounds->extent.x - colsepx - namex);
			
			break;
		}
		default:
	}
} /* ArticlesListDrawItem */



//
//
//  Initialize the newsgroups list table
//	lines: number of rows scrolled - to calculate what rows have to
//  be marked invalid to be redrawn.
//  lines == 0 -> init of table
//

static void InitArticlesTable(int lines)
{
	TablePtr table;
	RectangleType r;
	FontID curFont;
	Word row,
		 numrows,
		 tableheight,
		 dataheight,
		 lineheight,
		 currecord,
		 lastrecord;
	struct articledb_item a;
	
	table = GetObjectPtr(tblArtList);
	TblGetBounds(table, &r);
	tableheight = r.extent.y;

	curFont = FntSetFont(ArtListFont);
	lineheight = FntLineHeight();
	FntSetFont(curFont);
	

	dataheight = 0;
	currecord = top_article;

	numrows = TblGetNumberOfRows(table);
	for (row = 0; row < numrows; row++)
	{
		if (!NGReadArticle(currecord, &a))
		{
			// newsgroup number currecord exists
			TblSetRowID(table, row, currecord);
			TblSetItemStyle(table, row, 0, customTableItem);
			TblSetItemStyle(table, row, 1, customTableItem);
// OS2			TblSetItemFont(table, row, 2, ArtListFont);
			TblSetRowHeight(table, row, lineheight);
			TblSetRowData(table, row, currecord);
			TblSetRowUsable(table, row, true);
//			TblSetRowSelectable(table, row, true);
			if ((!lines || (!row && (lines < 0)) || (lines >= (int) numrows))
				|| ((lines > 0) && ((numrows - 1 - lines) < row) && (row < numrows))
				|| ((lines < 0) && (0 <= row) && (row < -lines)))
			{
				TblMarkRowInvalid(table, row);
			}

			lastrecord = currecord;
			if (row+1 < numrows)
			{
				currecord++;
			}
			dataheight += lineheight;
		}
		else
		{
			TblSetRowID(table, row, -1);
			TblSetRowUsable(table, row, false);
		}
	}
	TblSetCustomDrawProcedure(table, 0, ArticlesListDrawItem);
	TblSetCustomDrawProcedure(table, 1, ArticlesListDrawItem);
	TblSetColumnUsable(table, 0, true);
	TblSetColumnUsable(table, 1, true);
	
	artlistpagesize = numrows - 1;

	if (NGArticlesCount() > numrows)
	{
		TblHasScrollBar(table, true);
		SclSetScrollBar(GetObjectPtr(sbArtList), top_article, 0, NGArticlesCount() - numrows, artlistpagesize);
	}
	else
	{
		TblHasScrollBar(table, false);
		SclSetScrollBar(GetObjectPtr(sbArtList), 0, 0, 0, 0);
	}
	TblRedrawTable(table);
} /* InitArticlesTable */








static void ScrollArtList(int lines)
{
	Word lastRow, rows;
	Word scrollAmount;
	Int i, 
//		new_top,
		count;
	TablePtr table;
	RectangleType scrollR;
	RectangleType vacated;
	DirectionType direction;


	table = GetObjectPtr(tblArtList);
	rows = TblGetNumberOfRows(table);
	count = NGArticlesCount();
	top_article = GetNewTopline(top_article, count, rows, lines);

	if (((lines > 0) && (lines < rows)) ||
		 ((lines < 0) && (-lines < rows)))
	{
		scrollAmount = 0;
	
		if (lines > 0)
		{
			lastRow = TblGetLastUsableRow(table) - 1;
			for (i = 0; i < lines; i++)
			{
				scrollAmount += TblGetRowHeight(table, lastRow);
				TblRemoveRow(table, 0);
			}
			direction = up;
		}
		else
		{
			for (i = 0; i < -lines; i++)
			{
				scrollAmount += TblGetRowHeight(table, 0);
				TblInsertRow(table, 0);
			}
			direction = down;
		}

		TblGetBounds(table, &scrollR);
		WinScrollRectangle(&scrollR, direction, scrollAmount, &vacated);
		WinEraseRectangle(&vacated, 0);
	}

	InitArticlesTable(lines);
	TblRedrawTable(table);
} /* ScrollArtList */




void HandleArtEnterEvent(EventPtr event)
{
	TablePtr table;
	Short row, col;
	UInt n;

	table = event->data.tblSelect.pTable;
	row = event->data.tblSelect.row;
	col = event->data.tblSelect.column;
	n = TblGetRowID(table, row);
	if (col >= 1)
	{
		NGSetArticleFlag(n, yfArticleFlagRead, 1);
		TblMarkRowInvalid(table, row);
//		TblSelectItem(table, row, 1);
		ArtFGotoArticle(n);
	}
	else
	{
		NGSetArticleFlag(n, yfArticleFlagMarked, -1);
		TblMarkRowInvalid(table, row);
		FrmUpdateForm(frmArtList, 0);
	}	
} /* HandleArtEnterEvent */



void InitArticlesForm(void)
{
	InitArticlesTable(0);
	NPSetFormTitle(FrmGetActiveForm(), NGActiveNewsgroup(), 1);
} /* InitArticlesForm */



#ifdef COMMENT
void RefreshSelection(void)
{
	Word row, col;
	TablePtr tbl;
	
	tbl = GetObjectPtr(tblArtList);
	TblGetSelection(tbl, &row, &col);
	TblSelectItem(tbl, row, 1);
	TblMarkRowInvalid(tbl, row);
} /* RefreshSelection */
#endif



Boolean HandleArtList(EventPtr event)
{
	int handled = 0;
	FormPtr frm;
	
	frm = FrmGetActiveForm();

    switch (event->eType)
    {
		case frmGotoEvent:
		case frmUpdateEvent:
        {
        	if ((event->eType == frmUpdateEvent) 
        		&& (event->data.frmUpdate.updateCode == 1))
        	{
				InitArticlesForm();
			}
            RefreshForm();
            handled = 1;
            break;
        }

        case frmOpenEvent:
        {
			InitArticlesForm();
            RefreshForm();
            handled = 1;
            break;
        }

		case sclRepeatEvent:
		{
			ScrollArtList(event->data.sclRepeat.newValue -
						event->data.sclRepeat.value);
			break;
		}

        case menuEvent:
        {
			handled = HandleMainMenu(event);
			break;
        }
		
		case tblEnterEvent:
		{
			HandleArtEnterEvent(event);
			handled = 1;
		}


		case keyDownEvent:
		{
			switch (event->data.keyDown.chr)
			{
				case backspaceChr:
				{
					NGCloseNewsgroup();
					NPExitForm();
					handled = 1;
					break;
				}
				
				case pageUpChr: 
				{
					ScrollArtList(-artlistpagesize);
					handled = 1;
					break;
				}

				case pageDownChr: 
				{
					ScrollArtList(artlistpagesize);
					handled = 1;
					break;
				}
			}	
			break;
		}

        default:
    }
    return handled;
} /* HandleArtList */






void ArtListGotoNewsgroup(char *ng)
{
	NGOpenNewsgroup(ng);
	top_article = 0;
	NPGotoForm(frmArtList);
} /* ArtListGotoNewsgroup */
