/**************************************************************************

	Yanoff - the free UseNet news reader for the Palm Computing Platform
    Copyright (C) 1999 Matthias Jordan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Reach the author via 
	email: mjordan@sensenet.wirepool.free.de

**************************************************************************/
//
//
// frmnglist.c
//
// handle the newsgroups list form
// Matthias Jordan, 4-1999
//
//


#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>


#include "yanoff.h"
#include "general.h"
#include "subscrdb.h"
#include "frmartlist.h"

#include "mainmenu.h"


static FontID ListFont = stdFont;
UInt top_newsgroup = 0;





//
//
// Newsgroups Form Handling
//
//
//
//
//







//
//
//  Init newsgroups list table
//
//



void NewsgroupsListDrawItemString(CharPtr ng, short x, short y, 
		short width)
{
	Int titlelen;
	short titlewidth;
	Boolean stringfit;
	
	titlewidth = width;
	titlelen = StrLen(ng);
	FntCharsInWidth(ng, &titlewidth, &titlelen, &stringfit);
	if (stringfit)
	{
		WinDrawChars(ng, titlelen, x, y);
	}
	else
	{
		width -= (FntCharWidth('.') * 3);
		while (titlewidth > width)
		{
			titlewidth -= FntCharWidth(ng[--titlelen]);
		}
		WinDrawChars(ng, titlelen, x, y);
		x += titlewidth;
		WinDrawChars("...", 3, x, y);
	}
} /* NewsgroupsListDrawItemString */



void NewsgroupsListDrawItem(VoidPtr table, Word row, Word column, 
	RectanglePtr bounds)
{
	Word recnum;
	char *dum;
	short x, y;

	recnum = TblGetRowID(table, row);
	x = bounds->topLeft.x + 1;
	y = bounds->topLeft.y;
	FntSetFont(ListFont);
	SubSeekSubscription2(recnum, &dum);
	NewsgroupsListDrawItemString(dum, x, y,	bounds->extent.x);
} /* NewsgroupsListDrawItem */



void GetRowBounds(TablePtr tbl, int row, RectanglePtr r)
{
	RectangleType item, table;
	
	TblGetBounds(tbl, &table);
	TblGetItemBounds(tbl, row, 0, &item);
	r->topLeft.x = item.topLeft.x;
	r->topLeft.y = item.topLeft.y;
	r->extent.x = table.extent.x;
	r->extent.y = item.extent.y;
} /* GetRowBounds */



//
//
//  Initialize the newsgroups list table
//	lines: number of rows scrolled - to calculate what rows have to
//  be marked invalid to be redrawn.
//  lines == 0 -> init of table
//

static void InitNewsgroupsTable(int lines)
{
	TablePtr table;
	RectangleType r;
	FontID curFont;
	Word row,
		 numrows,
		 tableheight,
		 dataheight,
		 lineheight,
		 currecord,
		 lastrecord;
	char *ng;
	
	table = GetObjectPtr(tblNewsgroups);
	TblGetBounds(table, &r);
	tableheight = r.extent.y;

	curFont = FntSetFont(ListFont);
	lineheight = FntLineHeight();
	FntSetFont(curFont);
	

	dataheight = 0;
	currecord = top_newsgroup;

	numrows = TblGetNumberOfRows(table);
	for (row = 0; row < numrows; row++)
	{
		if (!SubSeekSubscription2(currecord, &ng))
		{
			// newsgroup number currecord exists
			TblSetRowID(table, row, currecord);
			TblSetItemStyle(table, row, 0, customTableItem);
// OS2			TblSetItemFont(table, row, 0, ListFont);
			TblSetRowHeight(table, row, lineheight);
			TblSetRowData(table, row, currecord);
			TblSetRowUsable(table, row, true);
			if (!lines || (!row && (lines < 0))|| (lines >= (int) numrows))
			{
				TblMarkRowInvalid(table, row);
			}
			else if ((lines > 0) && ((numrows - 1 - lines) < row) && (row < numrows))
			{
				TblMarkRowInvalid(table, row);
			}
			else if ((lines < 0) && (0 <= row) && (row < -lines))
			{
				TblMarkRowInvalid(table, row);
			}
			lastrecord = currecord;
			if (row+1 < numrows)
			{
				currecord++;
			}
			dataheight += lineheight;
		}
		else
		{
			TblSetRowUsable(table, row, false);
		}
	}
	TblSetCustomDrawProcedure(table, 0, NewsgroupsListDrawItem);
	TblSetColumnUsable(table, 0, true);
	if (SubSubscriptionsCount() > numrows)
	{
		TblHasScrollBar(table, true);
		SclSetScrollBar(GetObjectPtr(sbNewsgroups), top_newsgroup, 0, SubSubscriptionsCount() - numrows, numrows - 1);
	}
	else
	{
		TblHasScrollBar(table, false);
		SclSetScrollBar(GetObjectPtr(sbNewsgroups), 0, 0, 0, 0);
	}
	TblRedrawTable(table);
} /* InitNewsgroupsTable */








static void ScrollNGList(int lines)
{
	Int 				i;
	Word				rows;
	Word				lastRow;
	Word 				scrollAmount;
	Int 				new_top,
						count;
	TablePtr 		table;
	RectangleType	scrollR;
	RectangleType	vacated;
	DirectionType	direction;


	table = GetObjectPtr (tblNewsgroups);
	rows = TblGetNumberOfRows(table);

	// Find the new top visible record
	new_top = top_newsgroup + lines;

	if (lines > 0)
	{
		// ensure that the last newsgroup is not above the last row
		count = SubSubscriptionsCount();
		if (new_top > (count - rows))
		{
			new_top = count - rows;
			return;
		}
	}
	else if (lines < 0)
	{
		if (new_top < 0)
		{
			new_top = 0;
		}
		if (top_newsgroup == new_top)
		{
			// nothing happened to happen
			return;
		}
	}


	top_newsgroup = new_top;

	if (((lines > 0) && (lines < rows)) ||
		 ((lines < 0) && (-lines < rows)))
	{
		scrollAmount = 0;
	
		if (lines > 0)
		{
			lastRow = TblGetLastUsableRow(table) - 1;
			for (i = 0; i < lines; i++)
			{
				scrollAmount += TblGetRowHeight(table, lastRow);
				TblRemoveRow(table, 0);
			}
			direction = up;
		}
		else
		{
			for (i = 0; i < -lines; i++)
			{
				scrollAmount += TblGetRowHeight(table, 0);
				TblInsertRow(table, 0);
			}
			direction = down;
		}

		TblGetBounds(table, &scrollR);
		WinScrollRectangle(&scrollR, direction, scrollAmount, &vacated);
		WinEraseRectangle(&vacated, 0);
	}

	InitNewsgroupsTable(lines);
	TblRedrawTable(table);
} /* ScrollNGList */






void EnterNewsgroup(EventPtr event)
{
	TablePtr table;
	Short row;
	Word ngnum;
	char *ng;

	row = event->data.tblSelect.row;
	table = event->data.tblSelect.pTable;
	ngnum = TblGetRowID(table, row);
	SubSeekSubscription2(ngnum, &ng);
	ArtListGotoNewsgroup(ng);
} /* EnterNewsgroup */





Boolean HandleNewsgroups(EventPtr event)
{
    int handled = 0;
    
    switch (event->eType)
    {
		case frmUpdateEvent:
        case frmGotoEvent:
        case frmOpenEvent:
        {
			InitNewsgroupsTable(0);
            RefreshForm();
            handled = 1;
            break;
		}


        case menuEvent:
        {
			handled = HandleMainMenu(event);
			break;
        }

		case keyDownEvent:
		{
			switch (event->data.keyDown.chr)
			{
				case pageUpChr:
				{ 
					ScrollNGList(-1); 
					handled = 1; 
					break;
				}
				
				case pageDownChr:
				{
					ScrollNGList(1); 
					handled = 1; 
					break;
				}
				
				default :
			}
			break;
		}        

		case sclRepeatEvent:
		{
			ScrollNGList(event->data.sclRepeat.newValue -
						event->data.sclRepeat.value);
			break;
		}

		case tblEnterEvent:
		{
			EnterNewsgroup(event);
			handled = 1;
		}

        default :
    }
    return handled;
} /* HandleNewsgroups */



