/**************************************************************************

	Yanoff - the free UseNet news reader for the Palm Computing Platform
    Copyright (C) 1999 Matthias Jordan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Reach the author via 
	email: mjordan@sensenet.wirepool.free.de

**************************************************************************/
//
//
// frmprefs.c
//
// handle the prefs form of yanoff
// Matthias Jordan, 4-1999
//
//


#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include <System/DataMgr.h>
#include <System/NetMgr.h>
#include <System/DateTime.h>


#include "yanoff.h"
#include "general.h"
#include "frmprefs.h"
#include "prefs.h"
#include "ngroup.h"


VoidHand hs, hp, hid, hart, hsam, htim, hmax;




int GetFormValue(char *name, Word id, UInt *val)
{
	char *str;
	UInt dum;
	
	str = GetFormParameter(name, id);
	if (str == NULL)
	{
		return 1;
	}
	dum = StrAToI(str);
	if (dum != 0)
	{
		*val = dum;
		return 0;
	}
	return 1;
} /* GetFormValue */


int SavePrefs(void)
{
	UInt dum;
	char *str;
		
	str = GetFormParameter("Newsserver", fldPrefsServer);
	if (str == NULL)
	{
		return 1;
	}
	StrNCopy(yprefs.server, str, 40);
	
	if (GetFormValue("Port", fldPrefsPort, &yprefs.port))
	{
		return 1;
	}
	
	if (GetFormValue("ID Purge", fldPrefsIDPurge, &yprefs.idpurgedays))
	{
		return 1;
	}

	if (GetFormValue("Art Purge", fldPrefsArtPurge, &yprefs.artpurgedays))
	{
		return 1;
	}

	if (GetFormValue("Sample Arts", fldPrefsSampleArts, &yprefs.samplearts))
	{
		return 1;
	}

	if (GetFormValue("Timeout", fldPrefsTimeout, &yprefs.timeoutsecs))
	{
		return 1;
	}

	dum = yprefs.artmaxkb;
	if (GetFormValue("Article max", fldPrefsArtMax, &yprefs.artmaxkb))
	{
		return 1;
	}
	if (yprefs.artmaxkb > 31)
	{
		yprefs.artmaxkb = dum;
		return 2;
	}
	
	PrSavePrefs();
	return 0;
} /* SavePrefs */



void InitForm(void)
{
	FieldPtr fld;
	char *s, *p, *id, *art, *sam, *tim, *max;
	
	hs = MemHandleNew(StrLen(yprefs.server) + 1);
	hp = MemHandleNew(6);
	hid = MemHandleNew(6);
	hart = MemHandleNew(6);
	hsam = MemHandleNew(6);
	htim = MemHandleNew(6);
	hmax = MemHandleNew(6);
	
	if (!(hs && hp && hid && hart && hsam && htim && hmax))
	{
		return;
	}
	s = (char *) MemHandleLock(hs);
	p = (char *) MemHandleLock(hp);
	id = (char *) MemHandleLock(hid);
	art = (char *) MemHandleLock(hart);
	sam = (char *) MemHandleLock(hsam);
	tim = (char *) MemHandleLock(htim);
	max = (char *) MemHandleLock(hmax);
	
	StrCopy(s, yprefs.server);
	StrIToA(p, yprefs.port);
	StrIToA(id, yprefs.idpurgedays);
	StrIToA(art, yprefs.artpurgedays);
	StrIToA(sam, yprefs.samplearts);
	StrIToA(tim, yprefs.timeoutsecs);
	StrIToA(max, yprefs.artmaxkb);
	
	MemHandleUnlock(hs);
	MemHandleUnlock(hp);
	MemHandleUnlock(hid);
	MemHandleUnlock(hart);
	MemHandleUnlock(hsam);
	MemHandleUnlock(htim);
	MemHandleUnlock(hmax);

	fld = GetObjectPtr(fldPrefsServer);
	FldSetTextHandle(fld, (Handle) hs);
	
	fld = GetObjectPtr(fldPrefsPort);
	FldSetTextHandle(fld, (Handle) hp);
	
	fld = GetObjectPtr(fldPrefsIDPurge);
	FldSetTextHandle(fld, (Handle) hid);

	fld = GetObjectPtr(fldPrefsArtPurge);
	FldSetTextHandle(fld, (Handle) hart);

	fld = GetObjectPtr(fldPrefsSampleArts);
	FldSetTextHandle(fld, (Handle) hsam);

	fld = GetObjectPtr(fldPrefsTimeout);
	FldSetTextHandle(fld, (Handle) htim);

	fld = GetObjectPtr(fldPrefsArtMax);
	FldSetTextHandle(fld, (Handle) hmax);
} /* InitForm */



void ClearFields(void)
{
	FieldPtr fld;

	fld = GetObjectPtr(fldPrefsServer);
	FldSetTextHandle(fld, 0);
	
	fld = GetObjectPtr(fldPrefsPort);
	FldSetTextHandle(fld, 0);
	
	fld = GetObjectPtr(fldPrefsIDPurge);
	FldSetTextHandle(fld, 0);
	
	fld = GetObjectPtr(fldPrefsArtPurge);
	FldSetTextHandle(fld, 0);
	
	fld = GetObjectPtr(fldPrefsSampleArts);
	FldSetTextHandle(fld, 0);
	
	fld = GetObjectPtr(fldPrefsTimeout);
	FldSetTextHandle(fld, 0);

	fld = GetObjectPtr(fldPrefsArtMax);
	FldSetTextHandle(fld, 0);
	
	MemHandleFree(hs);
	MemHandleFree(hp);
	MemHandleFree(hid);
	MemHandleFree(hart);
	MemHandleFree(hsam);
	MemHandleFree(htim);
	MemHandleFree(hmax);
} /* ClearFields */



Boolean HandlePrefs(EventPtr event)
{
    int handled = 0;
    int res = 0;

    switch (event->eType)
    {
        case frmOpenEvent:
			InitForm();
            RefreshForm();
            handled = 1;
            break;

		case ctlSelectEvent:
		{
			switch (event->data.ctlSelect.controlID)
			{
				case btnPrefsCancel:
				{
					ClearFields();
					NPExitForm();
					handled = 1;
					break;
				}
				
				case btnPrefsOk:
				{
					res = SavePrefs();
					if (!res)
					{
						ClearFields();
						NPExitForm();
					}
					else if (res == 1)
					{
						FrmCustomAlert(altInfo, "Values of 0 are not allowed!", " ", " ");
					}
					else if (res == 2)
					{
						FrmCustomAlert(altInfo, "Max article size 31 KB. Sorry.", " ", " ");
					}
					handled = 1;
					break;
				}
				
				case btnPrefsIDPurge:
				{
					NGPurge(purgeMsgIDs);
					handled = 1;
					break;
				}

				case btnPrefsArtPurge:
				{
					NGPurge(purgeArticles);
					handled = 1;
					break;
				}
				
				default:
			}
			break;
		}
		
        default:
		
    }
    return handled;
} /* HandlePrefs */




