/**************************************************************************

	Yanoff - the free UseNet news reader for the Palm Computing Platform
    Copyright (C) 1999 Matthias Jordan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Reach the author via 
	email: mjordan@sensenet.wirepool.free.de

**************************************************************************/
//
//
// general.c
//
// general routines for use with yanoff
// Matthias Jordan, 4-1999
//
//


#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include <System/DataMgr.h>
#include <System/StringMgr.h>

#include "general.h"




//
//
// Opens database by name. If is doesnt exist: create it!
//
// returns 0 in case of error
// returns DmOpenRef > 0 in case of success
//

DmOpenRef NPOpenDB(char *name)
{
    LocalID lid;
    Err err;
    DmOpenRef db;

    lid = DmFindDatabase(0, name);
    if (lid == 0)
    {
        // not found - so create it
        err = DmCreateDatabase(0, name, CREATOR, TYPE, 0);
        if (err)
        {
            return 0;
        }
        lid = DmFindDatabase(0, name);
    }
    db = DmOpenDatabase(0, lid, dmModeReadWrite);
    return db;
} /* NPOpenDB */









//
//
// Checks whether there is a letter in a string
// Used to determine if address is dotted quad or FQDN
//

int StrHasLetter(char *c)
{
    char *testfield;
    VoidHand h;
    int res = 0;

    h = MemHandleNew(StrLen(c) + 1);
    if (h == 0)
    {
        return 0;
    }
    testfield = (char *) MemHandleLock(h);
    StrToLower(testfield, c);
    while (*testfield && !res)
    {
        if ((*testfield >= 'a') && (*testfield <= 'z'))
        {
            res = 1;
        }
        testfield++;
    }
    MemHandleUnlock(h);
    MemHandleFree(h);
    return res;
} /* StrHasLetter */





//
//
// Free chunk based on pointer
//

void FreePointer(Ptr p)
{
    VoidHand h;

    if (p == NULL)
        return;
    h = MemPtrRecoverHandle(p);
    if (h == 0)
    {
        return;
    }
    MemHandleUnlock(h);
    MemHandleFree(h);
} /* FreePointer */





//
//
// Common UI routines
//
//


VoidPtr GetObjectPtr(Word objectID)
{
	FormPtr frm;
	
	frm = FrmGetActiveForm();
	return (FrmGetObjectPtr (frm, FrmGetObjectIndex(frm, objectID)));
} /* GetObjectPtr */




char *GetFormParameter(char *fieldname, Word id)
{
    FieldPtr field;
    FormPtr form;
    char *contents;

    form = FrmGetActiveForm();
    field = FrmGetObjectPtr(form, FrmGetObjectIndex(form, id));
    contents = FldGetTextPtr(field);
    if (!FldGetTextLength(field))
    {
        FrmCustomAlert(alertID_dum, "Field empty:", fieldname, " ");
        return NULL;
    }
    return contents;
} /* GetFormParameter */





void RefreshForm(void)
{
	FormPtr frm;
	
	frm = FrmGetActiveForm();
    FrmDrawForm(frm);
} /* RefreshForm */



//
//  Routines for the forms stack
//  formstacktop is the index of the form that is currently active
//


#define FORMSTACKMAX 10
Word formstack[FORMSTACKMAX];
int formstacktop; 


void NPInitFormStack(Word formID)
{
    FrmGotoForm(formID);
    formstacktop = 0;
    formstack[formstacktop] = formID;
} /* NPInitFormStack */



void NPGotoForm(Word formID)
{
	if (formstacktop < (FORMSTACKMAX - 1))
	{
		formstack[++formstacktop] = formID;
		FrmPopupForm(formID);
	}
	else
	{
		FrmCustomAlert(alertID_dum, "Form stack overflow", " ", " ");
	}
} /* NPGotoForm */


void NPExitForm(void)
{
	if (formstacktop > 0) // don't let the stack get empty
	{
		FrmReturnToForm(formstack[--formstacktop]);
	}
	else
	{
		FrmCustomAlert(alertID_dum, "Form stack underrun", " ", " ");
	}
} /* NPExitForm */




static void MakeNewNGName(char *new, char *ng, SWord len)
{
	SWord act = 0;

	while (*ng && (act < len))
	{
		*new++ = *ng++;
		act++;
		while (*ng && (*ng != '.'))
		{
			ng++;
		}
		if (*ng == '.')
		{
			*new++ = *ng++;
			act++;
		}
	}
	*new = 0;
} /* MakeNewNGName */



void NPSetFormTitle(FormPtr frm, char *title, int is_ng)
{
	char dum[41];
	RectangleType r;
	SWord width, len;
	Boolean fits = 0;
	FontID font;
	
	FrmGetFormBounds(frm, &r);
	width = r.extent.x - 3;
	len = StrLen(title);
	font = FntSetFont(boldFont);
	FntCharsInWidth(title, &width, &len, &fits);
	if (!fits)
	{
		len = min(len, 40);
		if (is_ng)
		{
			MakeNewNGName(dum, title, len);
		}
		else
		{
			StrNCopy(dum, title, len); dum[len] = 0;
		}
		FrmCopyTitle(frm, dum);
	}
	else
	{
		FrmCopyTitle(frm, title);
	}
	FntSetFont(font);
} /* NPSetFormTitle */




Word GetNewTopline(int currenttop, Word linestotal, 
	Word linesdisplayed, Short scllines)
{
	Word new_top;

	new_top = currenttop + scllines;
	if (currenttop < -scllines) // "new_top < 0" for Word next (that cannot be < 0)
	{
		new_top = 0;
	}
	if (new_top > (linestotal - linesdisplayed))
	{
		new_top = linestotal - linesdisplayed;
	}
	return new_top;
} /* GetNewTopline */
