/**************************************************************************

	Yanoff - the free UseNet news reader for the Palm Computing Platform
    Copyright (C) 1999 Matthias Jordan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Reach the author via 
	email: mjordan@sensenet.wirepool.free.de

**************************************************************************/
//
//
// network.c
//
// Network routines for use with yanoff
// Matthias Jordan, 4-1999
//
//


#pragma pack(2)


#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include <System/DataMgr.h>
#include <System/NetMgr.h>

#include "general.h"
#include "yanoff.h"
#include "network.h"
#include "artdb.h"
#include "msgiddb.h"
#include "ngroup.h"
#include "prefs.h"
#include "subscrdb.h"
#include "frmart.h"




//
//
// Some definitions. Initially, we want a buffer of size buf_len bytes.
// NPInitNetwork will try to allocate a chunk of this size, sizing down
// in the event of an error (i.e. mem low).
// The real size of the allocated buffer is stored in buf_len.
// The same for the skip buffer into which data to be skipped is read
//

DmOpenRef buf_db;
Ptr buf;
VoidHand buf_h;
Word buf_len;
Ptr skipbuf;
VoidHand skipbuf_h;
Word skipbuf_len = 4069;
char bufferdb_name[] = {"Newsbuffer"};


char zeroterm = 0;
char none[] = {"-None-"};
int pollmode;
int pollart;

Err err;
char errname[30],
	lastresultcode[4];
Word nl = 0; /* Library-Reference */
NetSocketRef sock = -1;
NetSocketAddrINType remote;
Long timeoutticks;

char superfluous;

int con_to_server = 0; // a server connection is available
int mode_online = 0;   // we operate online (that is, don't close network after work)


//
// ------------------------------------------------------------------
//
//  Routines for initializing or closing the network connection
//  for use with Yanoff. 
//  There are two modes: Online operation and offline.
//
//  Online operation works as follows:
//
//  Before the user can get articles from the net she has to
//  establish a connection: First initialize the network interface, 
//  then connect to the server. Maybe the Pilot already has a usable
//  network interface connection, maybe the link gets established
//  with all the UI-action this implicates. We don't care about that.
//  The server connection is maintained all the time until the user
//  decides to cut it off or she quits the application. In the latter
//  case only the server is quit and the socket is shut down so 
//  other apps can use the link. 
//  
//  Offline operation goes like this:
//
//  The user may be offline or not. Anyway, she decides to get some
//  information from the UseNet and selects "Get Headers", "Get Articles" or
//  "Get Bodies" without a usable network interface. So the interface is
//  brought up, the data is retrieved and the interface is closed again
//  along with net lib. So if no other app uses the interface within 
//  the system timeout, the connection is cut (modem hangs up, whatever).
//
// ------------------------------------------------------------------
//


int NPOnlineMode(void)
{
	return mode_online;
} /* NPOnlineMode */


void NPWorkOnline(int mode)
{
	mode_online = mode;
} /* NPWorkOnline */



//
//
// Returns const char with error name based on error number
//

const char *NPErrorName(Err err)
{
    switch (err)
    {
        case netErrTimeout: StrCopy(errname, "Timeout"); break;
        case netErrNotOpen: StrCopy(errname, "Not open"); break;
        case netErrParamErr: StrCopy(errname, "Param error"); break;
        case netErrSocketNotOpen: StrCopy(errname, "Socket not open"); break;
        case netErrUnimplemented: StrCopy(errname, "Unimplemented"); break;
        case netErrWrongSocketType: StrCopy(errname, "Wrong socket type"); break;
        case netErrWouldBlock: StrCopy(errname, "Would block"); break;
        case netErrDNSNameTooLong: StrCopy(errname, "DNS: Name too long"); break;
        case netErrDNSBadName: StrCopy(errname, "DNS: Bad name"); break;
        case netErrSocketClosedByRemote: StrCopy(errname, "Socket closed by remote"); break;
//        case netErr: StrCopy(errname, ""); break;
        default:
            {
                StrCopy(errname, "Unknown Error #");
                StrIToA(&errname[StrLen(errname)], (Long) err);
            }
    }
    return errname;
} /* NPErrorName */





//
//
// Delete database that served as buffer dummy (and is empty)
// Qualifies as candidate for StopApplication
//

void NPDeleteBufferDatabase(void)
{
    LocalID lid;

    lid = DmFindDatabase(0, bufferdb_name);
	if (lid != 0)
	{
	    DmDeleteDatabase(0, lid);
	}
} /* NPDeleteBufferDatabase */





//
//
// Allocate buffers with intended size *len and minimum size min
// Real size and the associated handle are stored in *len and *h, resp.
//
// Returns pointer to buffer chunk or NULL in case of error.
//

Ptr AllocateBuffer(Word *len, Word min, VoidHand *h)
{
    char numdum[20];
    Ptr b;

    do
    {
        *h = DmNewHandle(buf_db, *len);
        if (*h == 0)
        {
            *len /= 2; // Buffer too big, try half sized one
        }
    }
    while ((*h == 0) && (*len > min));

    if (*h == 0)
    {
        StrIToA(numdum, min);
        FrmCustomAlert(alertID_dum, "Memory low. Could not allocate", numdum, "bytes!");
        return NULL;
    }
    b = (Ptr) MemHandleLock(*h);
    return b;
} /* AllocateBuffer */



//
//
// Opens NetLibrary if availiable and socket.
// NPInitNetwork will try to allocate a chunk of size buf_len, sizing down
// in the event of an error (i.e. mem low).
// The real size of the allocated buffer is stored in buf_len.
//
// returns 0 if successful
// returns err > 0 if error and pops up an alert
//

int NPInitNetwork(void)
{
    Word openerror;
//    char numdum[20];
//    UInt pos = -1;


	if (sock != -1)
	{
		FrmCustomAlert(alertID_dum, "Socket already open and usable.", " ", " ");
		// Socket already open
		return 0;
	}

	timeoutticks = yprefs.timeoutsecs * sysTicksPerSecond;

    err = SysLibFind("Net.lib", &nl);
    if (err)
    {
        FrmCustomAlert(alertID_dum, "Network library not found",
            "Maybe this is a PalmPilot Personal?", " ");
        return 1;
    }
    err = NetLibOpen(nl, &openerror);
    if (openerror || (err && (err != netErrAlreadyOpen)))
    {
        FrmCustomAlert(alertID_dum, "Network library open failed", " ", " ");
		if (openerror)
		{
			NetLibClose(nl, 1);
		}
        return 2;
    }

    sock = NetLibSocketOpen(nl, netSocketAddrINET, netSocketTypeStream, NetHToNS(6), timeoutticks, &err);
    if (sock == -1)
    {
        FrmCustomAlert(alertID_dum, "Socket open error", " ", " ");
        NetLibClose(nl, 0);
        return 3;
    }

    // Allocate transfer buffer database and let buf point to the buffer
    buf_db = NPOpenDB(bufferdb_name);
    if (buf_db == 0)
    {
        FrmCustomAlert(alertID_dum, "Could not allocate buffer DB", " ", " ");
        NetLibClose(nl, 0);
        sock = -1;
        return 4;
    }
    buf_len = yprefs.artmaxkb * 1024;
    buf = AllocateBuffer(&buf_len, 1024, &buf_h);
    if (buf == NULL)
    {
        NetLibClose(nl, 0);
        sock = -1;
        return 5;
    }
    skipbuf = AllocateBuffer(&skipbuf_len, 1024, &skipbuf_h);
    if (skipbuf == NULL)
    {
        NetLibClose(nl, 0);
        sock = -1;
        return 6;
    }
    err = 0;
    return 0;
} /* NPInitNetwork */




//
//
// Closes NetLibrary and socket, frees transfer buffer
//
// returns 0 if successful
// returns err > 0 if error and pops up an alert
//

int NPCloseNetwork(int immediately)
{
	if (sock == -1)
	{
//		FrmCustomAlert(alertID_dum, "Network already down", " ", " ");
		return 0;
	}
    NetLibSocketClose(nl, sock, timeoutticks, &err);
    // bring down interface (i.e. hang up modem)
    err = NetLibClose(nl, immediately);
    nl = 0;
    sock = -1;
	con_to_server = 0;
	mode_online = 0;
    MemHandleUnlock(buf_h);
    MemHandleFree(buf_h);
    DmCloseDatabase(buf_db);
    NPDeleteBufferDatabase();
    return err;
} /* NPCloseNetwork */





//
// ------------------------------------------------------------------
//
//    Routines for connecting to a server and exiting the server
//
// ------------------------------------------------------------------
//



//
//
// Quits from server.
//
// returns 0 if success
// returns err > 0 in case of error
//

int NPShutdown(void)
{
	SWord num;
	
    num = NetLibSocketShutdown(nl, sock, netSocketDirBoth, timeoutticks, &err);
    if (num == -1)
    {
        NPErrorName(err);
        FrmCustomAlert(alertID_dum, "Socket Close Error", errname, " ");
        return 3;
    }
    con_to_server = 0;
    return 0;
} /* NPShutdown */



//
//
// Connects Device to specified server. If server contains an FQDN
// the function gets the IP address by a DNS lookup.
//
// returns 0 if success
// returns err > 0 in case of error
//

int NPConnect(void)
{
	char *server;
	Word port;
    NetHostInfoBufType hostinfo;
    SWord num;

	server = yprefs.server;
	port = yprefs.port;
    err = 0;


    if (StrHasLetter(server))
    {
        if (0 != NetLibGetHostByName(nl, server, &hostinfo, timeoutticks, &err))
        {
            remote.addr = hostinfo.address[0];
        }
        else
        {
            NPErrorName(err);
            FrmCustomAlert(alertID_dum, "AddressError", errname, server);
            return 1;
        }
    }
    else
    {
        remote.addr = NetLibAddrAToIN(nl, server);
        if (remote.addr == -1)
        {
            err = netErrDNSBadName;
            NPErrorName(err);
            FrmCustomAlert(alertID_dum, "AddressError", errname, server);
            return 2;
        }
    }

    remote.family = netSocketAddrINET;
    remote.port = port;

    num = NetLibSocketConnect(nl, sock, (NetSocketAddrType *)&remote, sizeof(remote), timeoutticks, &err);
    if (num == -1)
    {
	    NetLibSocketClose(nl, sock, timeoutticks, &err);
	    sock = NetLibSocketOpen(nl, netSocketAddrINET, netSocketTypeStream, NetHToNS(6), timeoutticks, &err);
	    if (sock == -1)
	    {
	        FrmCustomAlert(alertID_dum, "Socket open error", " ", " ");
//	        NetLibClose(nl, 0);
//			NPCloseNetwork(1);
	        return 3;
	    }
	    num = NetLibSocketConnect(nl, sock, (NetSocketAddrType *)&remote, sizeof(remote), timeoutticks, &err);
	    if (num == -1)
	    {
    	    NPErrorName(err);
			NPShutdown();
	        FrmCustomAlert(alertID_dum, "Socket Connect Error", errname, " ");
	        return 3;
	    }
	}
    con_to_server = 1;
    NPGetAnswer(buf, buf_len, "200", 0); // Skip server status message
    return 0;
} /* NPConnect */









int NPConnected2Server(void)
{
	return con_to_server;
} /* NPConnected2Server */






//
// ------------------------------------------------------------------
//
//         Routines for network read operations
//
// ------------------------------------------------------------------
//


int ErrorCodeNonSevere(char *code)
{
	return (!(StrCompare(code, "423") 
        	&& StrCompare(code, "430")));
} /* ErrorCodeNonSevere */


//
//
// Reads size bytes from network socket
// Requires a buffer buf of size bytes in size
//
// returns number of bytes read
// returns 0 if socket has been shut down remotely
// returns -1 in case of general error
//

SWord NPReadN(char *buf, Word pos, Word size)
{
    SWord num;

    num = NetLibDmReceive(nl, sock, (VoidPtr) buf, pos, size, 0, 0, 0, timeoutticks, &err);
    if (num == -1)
    {
        NPErrorName(err);
        FrmCustomAlert(alertID_dum, "Read Error", errname, " ");
    }
    else if (num == 0)
    {
        FrmCustomAlert(alertID_dum, "Socket shut down", " ", " ");
    }
    return num;
} /* NPReadN */




//
//
// Reads string from socket - data is 0-terminated
// returns number of bytes read, not counting 0-terminator
//
// returns 0 if socket has been shut down remotely
// returns -1 in case of general error
//

SWord NPRead(char *buf, Word size)
{
    SWord len;

    len = NPReadN(buf, 0, size - 1);
    if (len > 0)
    {
        DmWrite(buf, len, &zeroterm, 1);
    }
    return len;
} /* NPRead */





//
//
// Read first 3 byte that contain the server answer code and
// compare it (i.e. "411") with expected answer code
//
// This function assumes for simplicity the allocated buffer is >= 4 byte large.
// Else it would simply make no sense to keep on working.
//
// returns 1 in case of match
// returns 0 in case of mismatch or error
//

int AnswerCodeOK(char *c, char *exp_answer)
{
    SWord len;

	// Skip linebreaks if any
	// Don't ask me why this has to be but it prevents some errors
	// I experienced during a multi-hundred-articles download.
	do
	{
		NPReadN(c, 0, 1);
	}
	while ((*c == '\n') || (*c == '\r'));

    DmSet(c, 1, 1, 'X');
    DmSet(c, 2, 1, 'X');
    DmSet(c, 3, 1, 'X');
    DmSet(c, 4, 1, 'X');
    len = NPReadN(c, 1, 2);
    if (len > 0)
    {
        // Set 0-terminator on eventual continue-dash of the
        // servers response code
        DmSet(c, 3, 1, 0);
    }
    else
    {
        // Error
        FrmCustomAlert(alertID_dum, "ServerError:", c, "(Socket)");
        return 0;
    }
    return (!StrCompare(c, exp_answer));
} /* AnswerCodeOK */




//
//
// Skip server error msg in the form
//
// -blablabla
// 441-foobar
// 441 end of rubbish
//
// The first numerical code is expected not to exist because
// AnswerCodeOK read it already when this function is called.
//

void SkipResponse(char *buf, Word size)
{
    SWord len;
    UInt pos = 0;

    len = NPReadN(buf, 0, 1);
    if ((len == 0) || (len == -1))
    {
        return;
    }
    if (buf[0] == '-')
    {
        do
        {
            // msg continued on next line so skip line
            do
            {
                len = NPReadN(buf, 0, 1);
                if ((len == 0) || (len == -1))
                {
                    return;
                }
            }
            while (buf[0] != '\n');
            NPReadN(buf, 0, 4);
        }
        while (buf[3] == '-');
    }
    // skip last line
    do
    {
        if (pos < (buf_len - 1))
        {
            pos++;
        }
        len = NPReadN(buf, pos, 1);
        if ((len == 0) || (len == -1))
        {
            return;
        }
    }
    while (buf[pos] != '\n');
} /* SkipResponse */




//
//
// Read server answer
// into buffer buf that is size byte long and 0-terminate it
// If wait_on_dotterm: Read server answer that is terminated by a full stop (".")
//
// The status answer is NOT stored in buf. It is skipped. The data in
// buf after this function returns successfully is the complete
// server response without status lines and trailing dot-line.
//
// exp_answcode points to the expected answer code for ok response
//
// Algorithm: Read answer into buffer until either a . is read
// or the buffer is full.
// In the first case: delete the .
// In the second case: memorize end of buffer, skip rest of answer
// At the end: 0-terminate buffer
//
// returns number of bytes read
// returns 0 in case of error
//

Word NPGetAnswer(char *b, Word size, char *exp_answcode, int wait_on_dotterm)
{
    ULong p;
    int dotterm_found = 0;
    char *dotterm = NULL;
    SWord len,
        context = 50;
    Word total = 0;
//    char cdum;


    if (!AnswerCodeOK(b, exp_answcode))
    {
        // Server error - msg other than exp_answcode recieved so skip rest
        // (probably continued)
        FrmCustomAlert(alertID_dum, "ServerError:", b, "(Response)");
        StrCopy(lastresultcode, b);
        SkipResponse(b, size);
        return 0;
    }

    if (!wait_on_dotterm)
    {
        return (Word) NPRead(b, size);
    }


    SkipResponse(b, size);
    p = 0; // Point to the beginning of b
    do
    {
        len = NPReadN(b, p, size - 1);
        if ((len == -1) || (len == 0))
        {
            return 0;
        }
        p += len;
        total += len;
        size -= len;

        // Now determine whether terminating full stop has been read
        // For this, look after "\n.\r" in the last "context" bytes
        if (len < context)
        {
            // in case that less then 50 bytes have been read, reduce "context"
            context = len;
        }
        if (p == len)
        {
            // p was 0 - so this is the first answer part we have got
            // from server. If answer starts with ".\r\n" the answer is empty
            dotterm = StrStr(b, ".\r\n");
            dotterm_found = (dotterm == b);
        }
        if (!dotterm_found)
        {
            dotterm = StrStr(&b[p]-context, "\n.\r");
            dotterm_found = (dotterm != NULL);
        }
    }
    while (((size - 1) > 0) && (!dotterm_found));
    if (dotterm_found)
    {
        // We have the whole response - including the traling dot-line
        // which we don't need. So kill the dot.
        DmSet(b, dotterm - b + 1, 1, zeroterm);
    }
    else
    {
        // Buffer is full - so skip rest of that junk
        do
        {
            len = NPReadN(skipbuf, 0, skipbuf_len);
            if ((len != -1) && (len != 0))
            {
                dotterm = StrStr(skipbuf, "\n.\r");
            }
        }
        while ((len != -1) && (len != 0) && (dotterm == NULL));
    }
    DmWrite(b, p, &zeroterm, 1);
    // for this 0-terminator there is reserved space
    // (the "-1" in NPReadN(p, 0, size - 1) above)
    return total;
} /* NPGetAnswer */




//
// ------------------------------------------------------------------
//
//         Routines for network write operations
//
// ------------------------------------------------------------------
//





//
//
// Writes data to network socket
//
// returns number of bytes written
// returns 0 if socket has been shut down remotely
// returns -1 in case of general error
//

SWord NPWriteN(char *buf, Word size)
{
    SWord num;

    num = NetLibSend(nl, sock, (VoidPtr) buf, size, 0, (VoidPtr) &remote, sizeof(remote), timeoutticks, &err);
    if (num == -1)
    {
        NPErrorName(err);
        FrmCustomAlert(alertID_dum, "Send Error", errname,  " ");
    }
    else if (num == 0)
    {
        FrmCustomAlert(alertID_dum, "Socket shut down", " ", " ");
    }
    return num;
} /* NPWriteN */



//
//
// Writes String to socket
//

SWord NPWrite(char *buf)
{
    return NPWriteN(buf, StrLen(buf));
} /* NPWrite */



//
//
// Writes three strings to socket. 0-string to omitt parameter
//

SWord NPWriteP(char *b1, char *b2, char *b3)
{
    SWord res,
          total = 0;

    if (*b1)
    {
        res = NPWrite(b1);
        if (res < 0)
            return res;
        else
            total += res;
    }
    if (*b2)
    {
        res = NPWrite(b2);
        if (res < 0)
            return res;
        else
            total += res;
    }
    if (*b3)
    {
        res = NPWrite(b3);
        if (res < 0)
            return res;
        else
            total += res;
    }

    return total;
} /* NPWriteP */






//
// ------------------------------------------------------------------
//
//           (Very many) Newsgroups related routines
//
// ------------------------------------------------------------------
//





ULong NextNum(char *p, int *pos)
{
    char dum;
    ULong num;
    int start;

    // nchstes Zeichen suchen
    while (p[*pos] && (p[*pos] != '\n') && (p[*pos] == ' '))
    {
        (*pos)++;
    }
    start = *pos;
    while (p[*pos] && (p[*pos] >= '0') && (p[*pos] <= '9'))
    {
        (*pos)++;
    }
    dum = p[*pos];
    DmWrite(p, *pos, "", 1);
    num = StrAToI(&p[start]);
    DmWrite(p, *pos, &dum, 1);
    return num;
} /* NextNum */







//
//
// Go to newsgroup
//
// The answer parameters of the server are stored in num, min, max:
// number of articles in group (estd)
// min name of articles
// max name of articles
//
// returns 0 in case of error
// returns 1 in case of success
//

int NPGroup(char *newsgroup, ULong *num, ULong *min, ULong *max)
{
    int res,
        pos = 0;

    NPWriteP("GROUP ", newsgroup, crlf);

    *num = *min = *max = 0;
    res = AnswerCodeOK(buf, "211");
    SkipResponse(buf, buf_len);
    if (!res)
    {
        // Group unknown
        FrmCustomAlert(alertID_dum, "Group unknown", newsgroup, " ");
        return 0;
    }
    *num = NextNum(buf, &pos);
    *min = NextNum(buf, &pos);
    *max = NextNum(buf, &pos);
    return 1;
} /* NPGroup */








//
// ------------------------------------------------------------------
//
//               Routines for downloading news articles
//
// ------------------------------------------------------------------
//



//
//
// Copy memory portion between start and end (including) to
// the memory region dest
//
// Returns pointer to destination area for convenience
//

char *NPMemCopy(char *dest, char *start, char *end)
{
    char *out;

    out = dest;
    while (start <= end)
    {
        *dest++ = *start++;
    }
    return out;
} /* NPMemCopy */





//
//
// Takes header data and fetches the content of a specific header field.
//
// returns a pointer to a movable chunk containing the header content.
// If the header has no content (standard violation - so what?) the
// chunk contains "-None-"
//
// The chunk has to be freed! Done in NPFreeArticleData
//
//

char *GetHeaderField(char *header, char *buf)
{
    VoidHand h;
    Ptr p, start, end;
//    char c;

    start = StrStr(buf, header);
    if (start != NULL)
    {
        // Header tag found - skip to colon
        while (*start && (*start != '\r') && (*start != ':'))
        {
            start++;
        }
        // Colon found - skip to content
        start++;
        while (*start && (*start != '\r') && ((*start == ' ') || (*start == '\t')))
        {
            start++;
        }
        // Content found - find end of line and terminate string temporarily
        end = start;
        while (*end && (*end != '\r'))
        {
            end++;
        }
        end--;
        // end now points to the last char of header content
        // if the header has no content (i.e. "From: \r\n"),
        // start points to the '\r' and end points to the char before '\r'
        // that is: end < start

        if (end < start)
        {
            start = none; // point to standard string
            end = &start[5];
        }
    }
    else
    {
        start = none;
        end = &start[5];
    }
    h = MemHandleNew((ULong) (end - start + 1) + 1);
    if (h == 0)
    {
        FrmCustomAlert(alertID_dum, "Mem low", " ", " ");
        return NULL;
    }
    p = MemHandleLock(h);
    NPMemCopy(p, start, end);
    p[(end - start + 1)] = 0;
    return p;
} /* GetHeaderField */








//
//
// Find MsgID in server reply in response to STAT command
//
// Returns pointer to 0-terminated MsgID in case of success
// Returns NULL else
//


char *FindMsgID(char *buf)
{
    char *start, *end;

    start = StrChr(buf, '<');
    if (start == NULL)
    {
        return &zeroterm;
    }
    end = StrChr(buf, '>');
    if (end == NULL)
    {
        return &zeroterm;
    }
    DmSet(buf, (end - buf) + 1, 1, 0);
    return start;
} /* FindMsgID */







//
//
// Fetch article from server by number
//
// Store article including body in the article database
// If complete body doesnt fit, allocate a smaller amount and retry
//
// The article is downloaded from the current newsgroup on the server
// The article is read into buf (global)
//
// returns 0 in case of success
// returns 1 in case that article is already in database (non-severe error)
// returns 2 in case that article is not on server (non-sever error)
// returns >2 for severe errors
//

#define yfErrSuccess 0
#define yfErrAlreadyRead 1
#define yfErrNotOnServer 2
#define yfErrSevere 3


int NPGetArticleHeader2DB(Word num, UInt *pos)
{
    struct articledb_item art;
    char numdum[11],
         *id;
    Word size;
    int res = 0;

    StrIToA(numdum, num);

    // Get msg stats

    NPWriteP("STAT ", numdum, crlf);
    if ((size = NPGetAnswer(buf, buf_len, "223", 0)) == 0)
    {
        // Error
        if (ErrorCodeNonSevere(lastresultcode))
        {
        	// Article not on server - so return corresponding result code
        	// so that PollNewsgroup continues work but without attempting
        	// to download the body.
        	return yfErrNotOnServer;
        }
        else
        {
	        return yfErrSevere;
	    }
    }

    id = FindMsgID(buf);
    if (MidMsgIDExists(id))
    {
		// return non-severe result code
        return yfErrAlreadyRead;
    }

    // Read and process header
    NPWriteP("HEAD ", numdum, crlf);
    if ((size = NPGetAnswer(buf, buf_len, "221", 1)) == 0)
    {
        // Error
        return yfErrSevere;
    }

    art.flags = 0;
    art.rcvdate = TimGetSeconds();
    art.xpostings = 0;
    art.newsgroups = GetHeaderField("Newsgroups:", buf);
    art.from = GetHeaderField("From:", buf);
    art.subject = GetHeaderField("Subject:", buf);
    art.date = GetHeaderField("Date:", buf);
    art.msgid = GetHeaderField("Message-ID:", buf);
    art.body = NULL;

    if (ArtAppendArticleHeader(&art, pos))
    {
        FrmCustomAlert(alertID_dum, "Memory low", " ", " ");
        res = yfErrSevere + 1; // just to emphasize the severeness...
    }
    else
    {
	    MidAddMsgID(art.msgid); // Store MsgID in database
	}
	ArtFreeArticleData(&art);
	return res;
} /* NPGetArticleHeader2DB */




int NPGetArticleBody2DB(UInt pos)
{
    struct articledb_item art;
		
	if (ArtReadArticle(pos, &art))
	{
		FrmCustomAlert(alertID_dum, "Article not in database.", "This should NEVER happen.", "Contact maintainer, please!");
		return yfErrSevere;
	}
	if (*art.body != 0)
	{
		// body already in database
		return yfErrAlreadyRead;
	}
	if (*art.msgid == 0)
	{
		// Article probably not read correctly
		return yfErrNotOnServer;
	}
    NPWriteP("BODY ", art.msgid, crlf);
    if (NPGetAnswer(buf, buf_len, "222", 1) == 0)
    {
        // Error
        FrmCustomAlert(alertID_dum, "BODY error", " ", " ");
        return yfErrSevere;
    }
    else
    {
		ArtAppendArticleBody(pos, buf);
    }
    return yfErrSuccess;
} /* NPGetArticleBody2DB */







int AbortButtonPressed(void)
{
	int stop = 0;
	EventType event;
	
	do 
	{
	    EvtGetEvent(&event, sysTicksPerSecond / 2);
	    if (!SysHandleEvent(&event))
	    {
			CtlHandleEvent(GetObjectPtr(btnPollAbort), &event);
			if ((event.eType == ctlSelectEvent)
				&& (event.data.ctlSelect.controlID == btnPollAbort))
			{
				stop = 1;
			}
		}
	}
	while ((!stop) && (event.eType != nilEvent));
	
	return stop;
} /* AbortButtonPressed */




static void InitPollField(FieldPtr *fld, VoidHand *h)
{
	FormPtr frm;
	char *p;

	frm = FrmGetActiveForm();
	*fld = GetObjectPtr(fldPoll);
	*h = MemHandleNew(1);
	if (*h == 0)
	{
		return;
	}
	p = MemHandleLock(*h);
	*p = 0;
	MemHandleUnlock(*h);
	FldSetTextHandle(*fld, (Handle) *h);
} /* InitPollField */


static void FieldOutput(FieldPtr fld, VoidHand h, char *str)
{
	char *p;
	UInt size;
	Err res;

	size = MemHandleSize(h);
	FldSetTextHandle(fld, 0);
	res = MemHandleResize(h,  size + StrLen(str) + 1);
	switch (res)
	{
		case memErrInvalidParam : break;
		case memErrNotEnoughSpace : break;
		case memErrChunkLocked : break;
		default:
	}
	if (res)
	{
		return;
	}
	p = MemHandleLock(h);
	StrCat(p, str);
	MemHandleUnlock(h);
	FldSetTextHandle(fld, (Handle) h);
} /* FieldOutput */


void FieldFlush(FieldPtr fld)
{
	char *p;

	p = FldGetTextPtr(fld);
	FldRecalculateField(fld, 0);
	FldSetScrollPosition(fld, StrLen(p));
	FldDrawField(fld);
} /* FieldFlush */




void FieldOutputRes(FieldPtr fld, VoidHand h, int header, int res)
{
	if (res == yfErrSuccess)
	{
       	FieldOutput(fld, h, header ? " h" : " b");
    }
	if (res == yfErrAlreadyRead)
	{
		FieldOutput(fld, h, " -");
	}
	if (res == yfErrNotOnServer)
	{
		FieldOutput(fld, h, " ?");
	}
	FieldFlush(fld);
} /* FieldOutputRes */



//
//
// Download body for article # act in article database
//


int NPPollNewsgroupBodyNG(FieldPtr fld, VoidHand h, ULong act)
{
	UInt pos;
	int res = yfErrAlreadyRead;
	ULong uid;
	char actdum[20];

	uid = NGGetArticleUID(act);
	if (!ArtSeekArticle(uid, &pos))
	{
	   	StrIToA(actdum, pos + 1);
		FieldOutput(fld, h, "\nDownloading article body ");
		FieldOutput(fld, h, actdum);
		FieldFlush(fld);
		res = NPGetArticleBody2DB(pos);
		FieldOutputRes(fld, h, 0, res);
	}
	return res;
} /* NPPollNewsgroupBodyNG */



int NPPollNewsgroupBodies(FieldPtr fld, VoidHand h)
{
	UInt max,
		act;
	int res = 0,
		stop = 0;
	char actdum[20];
	
	err = 0;
	max = NGArticlesCount() - 1;
    for (act=0; (!stop && (res < yfErrSevere) && !err && (act <= max)); act++)
    {
    	stop = AbortButtonPressed();
		if (!stop) 
		{
			res = NPPollNewsgroupBodyNG(fld, h, act);
		}
    }
    // err gets its value by the network low level functions above!
    if (err)
    {
        StrIToA(actdum, act);
        NPErrorName(err);
        FrmCustomAlert(alertID_dum, "Network error", errname, actdum);
    }
    else
    {
        FrmUpdateForm(frmArtList, 1);
    }
    return stop;
} /* NPPollNewsgroupBodies */





#define withoutBody 0
#define withBody 1


int NPPollNewsgroupHeaders(FieldPtr fld, VoidHand h, ULong last, 
		UInt min, UInt maxart, int mode)
{
	UInt act, 
		res = 0, 
		stop = 0,
		pos;
	char actdum[20];

	err = 0;
	for (act=min; (!stop && (res < yfErrSevere) && !err 
		&& (act <= maxart)); act++) 
	{
		stop = AbortButtonPressed();
		if (!stop)
		{
	    	StrIToA(actdum, act);
	    	FieldOutput(fld, h, "\nDownloading article ");
	    	FieldOutput(fld, h, actdum);
	    	FieldFlush(fld);
	        // Get article and store it in Newsarts
	        res = NPGetArticleHeader2DB(act, &pos);
	        FieldOutputRes(fld, h, 1, res);
	        if (res < yfErrSevere) // success or non-severe error
	        {
	        	if ((mode == withBody) && (res < yfErrNotOnServer))
	        	{
	        		// Header received ok or non-severe error
					if (act == 13742)
					{
						act += 1; act -= 1; // breakpoint dummy
					}
					res = NPGetArticleBody2DB(pos);
					FieldOutputRes(fld, h, 0, res);
		       	}
		       	last = act;
	        }
		}
    }
    // err gets its value by the network low level functions above!
    if (err)
    {
        StrIToA(actdum, act);
        NPErrorName(err);
        FrmCustomAlert(alertID_dum, "Network error", errname, actdum);
        stop = 1;
    }
    else
    {
        FrmUpdateForm(frmArtList, 1);
    }
	NGUpdateLastField(last);    
	return stop;
} /* NPPollNewsgroupHeaders */








int NPPollNewsgroup(FieldPtr fld, VoidHand h, char *newsgroup, int open)
{
    ULong count,
        amin,
        amax;
    ULong last;
    int stop;


	if (newsgroup == NULL)
	{
		return 1;
	}
	if (!open)
	{
		NGOpenNewsgroup(newsgroup);
	}

	FieldOutput(fld, h, "\nPolling ");
	FieldOutput(fld, h, newsgroup);
	if (pollmode == pollHeaders)
	{
		FieldOutput(fld, h, "\nGetting only headers");
	}
	FieldFlush(fld);
	
//    NPGetAnswer(buf, buf_len, "200", 0); // Skip server status message

    if (!NPGroup(newsgroup, &count, &amin, &amax))
    {
    	if (!open)
    	{
			NGCloseNewsgroup();
		}
//        NPCloseNetwork();
        return 0;
    }

    FieldOutput(fld, h, "\nNewsgroup entered");
    FieldFlush(fld);

	SubReadSubscription(newsgroup, NULL, &last);
	if (last == 0)
	{
		// See whether 
		amin = max(amin, amax - yprefs.samplearts + 1);
	}
	else
	{
		amin = max(amin, last + 1); // get first article that we don't already have 
	}

	switch (pollmode)
	{
		case pollHeaders: 
		{
			stop = NPPollNewsgroupHeaders(fld, h, last, amin, amax, withoutBody); 
			break;
		}
		case pollArticles: 
		{
			stop = NPPollNewsgroupHeaders(fld, h, last, amin, amax, withBody); 
			break;
		}
		case pollBodies: 
		{
			stop = NPPollNewsgroupBodies(fld, h); 
			break;
		}
		default: stop = 1;
	}
	if (!open)
	{
		NGCloseNewsgroup();
	}
	return stop;
} /* NPPollNewsgroup */




void NPPollNewsgroups(void)
{
	char *newsgroup;
//    ULong count,
//        min,
//        max;
    UInt pos, start, count;
	VoidHand h;
	FieldPtr fld;
	UInt oldautooff;	
	int stop = 0;

	
	newsgroup = NGActiveNewsgroup();
	if ((newsgroup == NULL) && (SubSubscriptionsCount() > 1))
	{
		if (FrmCustomAlert(altPollAll, " ", " ", " "))
		{
			return;
		}
	}

	InitPollField(&fld, &h);

	if (!NPOnlineMode())
	{
	    if (NPInitNetwork() || NPConnect())
	    {
	        // Error
	        FrmCustomAlert(alertID_dum, "Maybe wrong server name or port number.", " ", " ");
//	        NPCloseNetwork(0);
			// Let network open - user can close it herself if she wants to
			FldSetTextHandle(fld, 0);
		    MemHandleFree(h);
	        return;
	    }
	}

	oldautooff = SysSetAutoOffTime(0);

	FieldOutput(fld, h, yprefs.server);
	FieldOutput(fld, h, " connected");
	FieldFlush(fld);

	start = ArtArticlesCount();
	if (newsgroup == NULL)
	{
		count = SubSubscriptionsCount();
		for (pos=0; (!stop && (pos < count)); pos++)
		{
			SubSeekSubscription2(pos, &newsgroup);
			stop = NPPollNewsgroup(fld, h, newsgroup, 0);
		}
	}
	else
	{
		NPPollNewsgroup(fld, h, newsgroup, 1);
	}

	if (!NPOnlineMode())
	{
//		NPShutdown();
		NPCloseNetwork(1);
	}


	if (pollmode != pollBodies)
	{
		count = ArtArticlesCount();
		if (count != start)
		{
		    // Fetch record ID and enter it into the appropriate databases
		    // of the newsgroups that recieved this article and that we subscribed to
		   	FieldOutput(fld, h, "\nInstalling articles");
		   	FieldFlush(fld);
		    for (pos=start; (pos < count); pos++)
		    {
		        NGWeaveInArticle(pos);
		    }
		}
		else
		{
		   	FieldOutput(fld, h, "\nNo new articles.");
		   	FieldFlush(fld);
		   	SysTaskDelay(sysTicksPerSecond/2);
		}
	}
	FldSetTextHandle(fld, 0);
    MemHandleFree(h);
    SysSetAutoOffTime(oldautooff);
} /* NPPollNewsgroups */






void NPPollSingleBody(void)
{
	char *newsgroup;
	VoidHand h;
	FieldPtr fld;
	ULong count, min, max;
	UInt oldautooff;
	

	oldautooff = SysSetAutoOffTime(0);
	newsgroup = NGActiveNewsgroup();

	InitPollField(&fld, &h);

	FieldOutput(fld, h, yprefs.server);
	FieldOutput(fld, h, " connected");
	FieldFlush(fld);

    if (!NPGroup(newsgroup, &count, &min, &max))
    {
		FieldOutput(fld, h, "Could not enter ");
		FieldOutput(fld, h, newsgroup);
		FieldFlush(fld);
		SysSetAutoOffTime(oldautooff);
        return;
    }

	NPPollNewsgroupBodyNG(fld, h, pollart);

	FldSetTextHandle(fld, 0);
    MemHandleFree(h);
    FrmUpdateForm(frmArticle, artModeBody);
    SysSetAutoOffTime(oldautooff);
} /* NPPollSingleBody */









int HandlePoll(EventPtr event)
{
  int handled = 0;

    switch (event->eType)
    {
        case frmOpenEvent:
        {
            RefreshForm();
			if (pollmode == pollSingleBody)
			{
				NPPollSingleBody();
			}
			else
			{
				NPPollNewsgroups();
			}
            NPExitForm();
            handled = 1;
            break;
        }
	
		default:
    }
    return handled;
} /* HandlePoll */



void NPInvokePoll(int mode, UInt pos)
{
	pollmode = mode;
	pollart = pos;
	NPGotoForm(frmPoll);
} /* NPInvokePoll */

