/**************************************************************************

	Yanoff - the free UseNet news reader for the Palm Computing Platform
    Copyright (C) 1999 Matthias Jordan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Reach the author via 
	email: mjordan@sensenet.wirepool.free.de

**************************************************************************/
//
//
// ngroup.c
//
// newsgroup article management routines for use with yanoff
// Matthias Jordan, 4-1999
//
//


#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include <System/DataMgr.h>


#include "general.h"
#include "artdb.h"
#include "subscrdb.h"
#include "ngroup.h"
#include "prefs.h"
#include "msgiddb.h"


DmOpenRef ngdb = NULL;
char *active_newsgroup;
VoidHand anh;
int purge_what;




DmOpenRef NGOpenNewsgroup(char *ng)
{
	 char tempname[20];
	
	SubNGDatabaseName(ng, tempname);	
    ngdb = NPOpenDB(tempname);
    anh = MemHandleNew(StrLen(ng) + 1);
    if (anh == 0)
    {
    	return NULL;
    }
    active_newsgroup = MemHandleLock(anh);
    StrCopy(active_newsgroup, ng);
    return ngdb;
} /* NGOpenNewsgroup */



void NGCloseNewsgroup(void)
{
    if (ngdb != NULL)
    {
		DmCloseDatabase(ngdb);
		ngdb = NULL;
		MemHandleFree(anh);
		active_newsgroup = NULL;
    }
} /* NGCloseNewsgroup */



char *NGActiveNewsgroup(void)
{
	return active_newsgroup;
} /* NGActiveNewsgroup */



//
//
// Insert unique ID of an article in the global article db to
// the currently open newsgroup
//

int NGInsertArticle(ULong id)
{
    UInt pos = -1;
    VoidHand h;
    ULong *targetid;

    h = DmNewRecord(ngdb, &pos, sizeof(ULong));
    if (h == 0)
    {
        return 0;
    }
    targetid = (ULong *)MemHandleLock(h);
    DmWrite(targetid, 0, &id, sizeof(id));
    MemHandleUnlock(h);
    DmReleaseRecord(ngdb, pos, 1);
    return 1;
} /* NGInsertArticle */






//
//
// Install article with number pos to the corresponding newsgroups
// 
// pos will normally be -1 to install the last article
//


int NGWeaveInArticle(UInt pos)
{
    char *newsgroups,
         *ngdum,
         *ngend;
    ULong uid,
          last;
    VoidHand oldngh = 0,
             ngh;
    char dbid[6];
    struct articledb_item art;
	char *oldng = 0;
	UInt xpostings = 0;

	uid = ArtGetArticleUID(pos);
	if (active_newsgroup != NULL)
	{
		// There is a newsgroup db currently open.
		// So memorize current newsgroup and close newsgroup database
		oldngh = MemHandleNew(StrLen(active_newsgroup) + 1);
		if (oldngh == 0)
		{
			return 0;
		}
		oldng = (char *) MemHandleLock(oldngh);
		StrCopy(oldng, active_newsgroup);
		NGCloseNewsgroup();
	}
	
	ArtReadArticle(pos, &art);
	ngdum = art.newsgroups;
    ngh = MemHandleNew(StrLen(ngdum) + 1);
    if (ngh == 0)
    {
        return 0;
    }
    newsgroups = MemHandleLock(ngh);
    StrCopy(newsgroups, ngdum);
    ngend = newsgroups + StrLen(newsgroups);
    while (newsgroups < ngend)
    {
        ngdum = newsgroups;
        while (*ngdum && (*ngdum != ','))
        {
            ngdum++;
        }
        if (*ngdum == ',')
        {
            *ngdum = 0;
        }
        // newsgroups contains the name of one newsgroup to which the
        // article has to be posted

        if (!SubReadSubscription(newsgroups, dbid, &last))
        {
        	NGOpenNewsgroup(newsgroups);
            NGInsertArticle(uid);
            NGCloseNewsgroup();
            xpostings++;
        }
        newsgroups = ngdum + 1;
    }
    MemHandleUnlock(ngh);
    MemHandleFree(ngh);

	if (oldngh != 0)
	{
	    NGOpenNewsgroup(oldng);
	    MemHandleUnlock(oldngh);
	    MemHandleFree(oldngh);
	}

    ArtChangeArticleXP(uid, xpostings);
    return 1;
} /* NGWeaveInArticle */








//
//
// Count articles
//
//

int NGArticlesCount(void)
{
	return DmNumRecords(ngdb);
} /* NGArticlesCount */




//
//
// Find the n-th article in the currently open newsgroup
// and return its unique ID of its article database entry
//

ULong NGGetArticleUID(UInt pos)
{
	VoidHand h;
	ULong *uid, u;
	
	h = DmQueryRecord(ngdb, pos);
	if (h == 0)
	{
		return 0;
	}
	uid = MemHandleLock(h);
	u = *uid;
	MemHandleUnlock(h);
	return u; 
} /* NGGetArticleUID */




//
//
// Find the n-th article in the currently open newsgroup
//

int NGReadArticle(UInt ngpos, struct articledb_item *art)
{
	ULong uid;
	UInt apos;
	
	uid = NGGetArticleUID(ngpos);
	if (ArtSeekArticle(uid, &apos))
	{
		art->from = 0;
		art->subject = 0;
		return 1;
	}
	else
	{
		return ArtReadArticle(apos, art); 
	}
} /* NGReadArticle */



//
//
// Delete Article from NewsGroup database
//

int NGDeleteArticle(int pos)
{
	ULong uid;

	uid = NGGetArticleUID(pos);
	ArtDeleteArticle(uid);
	DmRemoveRecord(ngdb, pos);
    return 0;
} /* NGDeleteArticle */




int NGSetArticleFlag(UInt pos, int flag, int value)
{
	ULong uid;
	
	uid = NGGetArticleUID(pos);
	ArtSetArticleFlag(uid, flag, value);
	return 0; 
} /* NGSetArticleFlag */



void NGSetArticleFlagAll(int flag, int mode)
{
	UInt count, act;
	
	count = NGArticlesCount();
	for (act=0; (act < count); act++)
	{
		NGSetArticleFlag(act, flag, mode);
	}
} /* NGSetArticleFlagAll */



void NGMarkArticles(int mode)
{
	NGSetArticleFlagAll(yfArticleFlagMarked, mode);
} /* NGMarkArticles */




void NGDeleteSelectedRead(int mode)
{
	UInt count, act;
	struct articledb_item art;
	
	count = NGArticlesCount();
	for (act=0; (act < count);)
	{
		NGReadArticle(act, &art);
		// if article is not to keep:
		// either if art is marked and mode is 0
		// or if art is read an mode is 1:
		// delete the article
		if (!(art.flags & yfArticleFlagKeep) && 
			((!mode && (art.flags & yfArticleFlagMarked))
			|| (mode && (art.flags & yfArticleFlagRead))))
		{
			// Maybe this article is a crossposting. So unset the
			// marked flag to be sure the display is correct.
			NGSetArticleFlag(act, yfArticleFlagMarked, 0); 
			NGDeleteArticle(act);
			count--; // one article less in newsgroup
		}
		else
		{
			if (!mode)
			{
				NGSetArticleFlag(act, yfArticleFlagMarked, 0);
			}
			act++;
		}
	}
} /* NGDeleteSelectedRead */




void NGKeepSelected(int mode)
{
	UInt count, act;
	struct articledb_item art;
	
	count = NGArticlesCount();
	for (act=0; (act < count); act++)
	{
		NGReadArticle(act, &art);
		if (art.flags & yfArticleFlagMarked)
		{
			NGSetArticleFlag(act, yfArticleFlagKeep, mode); 
			NGSetArticleFlag(act, yfArticleFlagMarked, 0);
		}
	}
} /* NGKeepSelected */






int NGUpdateLastField(ULong newlast)
{
	return SubUpdateSubscription(active_newsgroup, newlast);
} /* NGUpdateLastField */


ULong NGReadLastField(void)
{
	ULong last;
	char dbid[6];

	SubReadSubscription(active_newsgroup, dbid, &last);
	return last;
} /* NGReadLastField */




void NGPurgeOld(void)
{
	UInt ngcount, actng,
		artcount, actart;
	char *ng;
	struct articledb_item art;
    VoidHand oldngh = 0;
	char *oldng = 0;


	if (active_newsgroup != NULL)
	{
		// There is a newsgroup db currently open.
		// So memorize current newsgroup and close newsgroup database
		oldngh = MemHandleNew(StrLen(active_newsgroup) + 1);
		if (oldngh == 0)
		{
			return;
		}
		oldng = (char *) MemHandleLock(oldngh);
		StrCopy(oldng, active_newsgroup);
		NGCloseNewsgroup();
	}


	ArtPurgeArticles(yprefs.artpurgedays);	
	ngcount = SubSubscriptionsCount();
	for (actng=0; (actng < ngcount); actng++)
	{
		// Purge every single subscribed newsgroup
		if (!SubSeekSubscription2(actng, &ng))
		{
			NGOpenNewsgroup(ng);
			artcount = NGArticlesCount();
			for (actart=0; (actart < artcount);)
			{
				if (NGReadArticle(actart, &art))
				{
					// article not in global database - so delete record
					DmRemoveRecord(ngdb, actart);
					artcount--;
				}
				else
				{
					actart++;
				}
			}
			NGCloseNewsgroup();
		}
	}
	
	if (oldngh != 0)
	{
	    NGOpenNewsgroup(oldng);
	    MemHandleUnlock(oldngh);
	    MemHandleFree(oldngh);
	}
} /* NGPurgeOld */




//
//
// In case of a crash during download the new downloaded articles
// are not properly installed in the databases
//
// Strategy: Kill all articles in the newsgroups and call
// WeaveInArticle for all articles in artdb
//


//int RestoreDatabase(void)
//{
//	FrmCustomAlert(alertID_dum, "Function not implemented.", "Yet.", " ");
//} /* RestoreDatabase */





Boolean HandlePurge(EventPtr event)
{
    int handled = 0;

    switch (event->eType)
    {
        case frmOpenEvent:
        {
			if (purge_what == purgeArticles)
			{
	            NPSetFormTitle(FrmGetActiveForm(), "Purging Articles...", 0);
	            RefreshForm();
				NGPurgeOld();
			}
			else
			{
	            NPSetFormTitle(FrmGetActiveForm(), "Purging MsgIDs...", 0);
	            RefreshForm();
				MidPurgeMsgIDs(yprefs.idpurgedays * 86400);
			}
			FrmUpdateForm(frmArtList, 1);
			NPExitForm();
			handled = 1;
			break;
		}
		
        default:
		
    }
    return handled;
} /* HandlePurge */



void NGPurge(int what)
{
	purge_what = what;
	NPGotoForm(frmPurge);
} /* NGPurge */
