/**************************************************************************

	Yanoff - the free UseNet news reader for the Palm Computing Platform
    Copyright (C) 1999 Matthias Jordan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Reach the author via 
	email: mjordan@sensenet.wirepool.free.de

**************************************************************************/
//
//
// subscrdb.c
//
// newsgroup subscription management routines for use with yanoff
// Matthias Jordan, 4-1999
//
//


#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include <System/DataMgr.h>


#include "general.h"
#include "yanoff.h"
#include "artdb.h"
#include "subscrdb.h"




char subscrdb_name[20] = {"NewsSubscriptions"};
char groupdb_prefix[] = {"NewsGroup-"};

// internal database descriptor
static DmOpenRef subsdb;



//
//
// All routines
// return 0 in case of success and
// return err > 0 in case of error
//



//
//
// Compare function
// if comp_db_entries is 1 the function compares db entries
// if comp_db_entries is 0 the value in *a1 is a string
//
// returns values like StrCompare
//

Int comp_subs(void *a1, void *a2, int comp_db_entries,
    SortRecordInfoPtr p1, SortRecordInfoPtr p2, VoidHand h)
{
	if (comp_db_entries)
	{
	    a1 += StrLen((char *)a1) + 1;
    	a1 += sizeof(ULong);
    }

    a2 += StrLen((char *)a2) + 1;
    a2 += sizeof(ULong);

    return StrCompare((char *)a1, (char *)a2);
} /* comp_subs */











DmOpenRef SubOpenSubscriptions(void)
{
    subsdb = NPOpenDB(subscrdb_name);
    return subsdb;
} /* SubOpenSubscriptions */



void SubCloseSubscription(void)
{
    if (subsdb != 0)
    {
		DmCloseDatabase(subsdb);
		subsdb = 0;
    }
} /* SubCloseSubscription */







//
//
// Read subscription information from database
//

int SubReadSubscription(char *newsgroup, char *dbid, ULong *last)
{
    UInt pos;
    VoidHand h;
    Ptr p;
    int res;
    char d[4];


    if (subsdb == 0)
    {
        // db open error
        return 1;
    }
    pos = DmFindSortPositionV10(subsdb, newsgroup, (DmComparF *)comp_subs, 0);
    h = DmQueryRecord(subsdb, pos - 1);
    if (h == 0)
    {
        return 2;
    }
    p = MemHandleLock(h);

	if (dbid != NULL)
	{
	    StrCopy(dbid, p);
	}
    p += StrLen(p) + 1;
	// Fool memory alignment problem that took me 10 hours to find...
	// (or however this prevents the Pilot from crashing...)
    d[0] = p[0];
    d[1] = p[1];
    d[2] = p[2];
    d[3] = p[3];
    *last = *(ULong *) d;
//    *last = *(ULong *) p;
    res = StrCompare(p+sizeof(ULong), newsgroup); // correct newsgroup record found?
    MemHandleUnlock(h);
    return res;
} /* SubReadSubscription */





char *SubNGDatabaseName(char *ng, char *buf)
{
	char dbid[6];
	ULong last;

	if (SubReadSubscription(ng, dbid, &last))
	{
		return NULL;
	}
	StrCopy(buf, groupdb_prefix);
	StrCat(buf, dbid);
	return buf;
} /* SubNGDatabaseName */






//
//
// Add subscription to database
// Subscription is added to the db, the db recurds are sorted and the
// newsgroup db is created without any records
//
// returns 0 in case of success
// returns >0 in case of error
//

int SubAddSubscription(char *newsgroup)
{
    Err err;
    ULong size;
    UInt pos = -1;
    VoidHand h;
    struct subscriptiondb_item *p;
    char dum[6];
    char tempname[20];
    UInt numdum = 0,
         dbpos;
    ULong last;


    if (subsdb == 0)
    {
        // db open error
        return 1;
    }

    if (!SubReadSubscription(newsgroup, dum, &last))
    {
        return 2;
    }
    // First, create a new newsgroup database
    // This part has complexity O(n) but we dont create a new
    // newsgroup very often and with this algorithm we dont need to care
    // about deleted newsgroups and gaps in the database name order
    // i.e.: pre: databases Group-1, Group-2, Group-3 exist. We delete Group-2
    // The next created database gets name Group-2 because the name is free.
    StrCopy(tempname, groupdb_prefix);
    do
    {
        numdum++;
        StrIToA(dum, numdum);
        StrCopy(tempname+10, dum);
        err = DmCreateDatabase(0, tempname, CREATOR, TYPE, 0);
        if (err && (err != dmErrAlreadyExists))
        {
            return 1;
        }

    }
    while (err == dmErrAlreadyExists);

    // At this point a new database has been created with name tempname

    dbpos = StrLen(dum) + 1;
    size = dbpos + sizeof(long) + StrLen(newsgroup) + 1;

    h = DmNewRecord(subsdb, &pos, size);
    if (h == 0)
    {
        // error
        return 3;
    }
    p = (struct subscriptiondb_item *) MemHandleLock(h);

    DmStrCopy(p, 0, dum);
	last = 0;
    DmWrite(p, dbpos, &last, sizeof(ULong));
    dbpos += 4;
    DmStrCopy(p, dbpos, newsgroup);
    MemHandleUnlock(h);
    DmReleaseRecord(subsdb, pos, 1);
    DmQuickSort(subsdb, (DmComparF *)comp_subs, 1);
    return 0;
} /* SubAddSubscription */








//
//
// Update subscription information to database
//

int SubUpdateSubscription(char *newsgroup, ULong last)
{
    UInt pos;
    VoidHand h;
    Ptr p;


    if (subsdb == 0)
    {
        // db open error
        return 1;
    }

    pos = DmFindSortPositionV10(subsdb, newsgroup, (DmComparF *)comp_subs, 0);
    h = DmQueryRecord(subsdb, pos - 1);
    if (h == 0)
    {
        return 2;
    }
    p = MemHandleLock(h);
    if (StrCompare(p+StrLen(p)+1+sizeof(ULong), newsgroup))
    {
        // incorrect newsgroup record found
        MemHandleUnlock(h);
        return 3;
    }

    DmWrite(p, StrLen(p) + 1, &last, sizeof(ULong));
    MemHandleUnlock(h);
    return 0;
} /* SubUpdateSubscription */






int SubSeekSubscription2(UInt pos, char **ngname)
{
    VoidHand h;
    Ptr p;


    if (subsdb == 0)
    {
        // db open error
        return 1;
    }
    h = DmQueryRecord(subsdb, pos);
    if (h == 0)
    {
        return 2;
    }
    p = MemHandleLock(h);
    *ngname = p+StrLen(p)+1+sizeof(ULong);
    MemHandleUnlock(h);
	return 0;
} /* SubSeekSubscription2 */







//
//
// Delete subscribed newsgroup database and subscription entry
//

int SubDeleteSubscription(char *newsgroup)
{
    UInt pos;
//    VoidHand h;
    LocalID lid;
//    Ptr p;
    char *ngname,
    	 groupdbname[20];

    pos = DmFindSortPositionV10(subsdb, newsgroup, (DmComparF *)comp_subs, 0);
	SubSeekSubscription2(pos - 1, &ngname);
	if (StrCompare(ngname, newsgroup))
	{
		// incorrect newsgroup record found - but why the hell?!?
		return 3;
	}
	SubNGDatabaseName(newsgroup, groupdbname);
    if (DmRemoveRecord(subsdb, pos - 1) != 0)
    {
        return 4;
    }
    lid = DmFindDatabase(0, groupdbname);
    if (DmDeleteDatabase(0, lid) != 0)
    {
        return 4;
    }
    return 0;
} /* SubDeleteSubscription */









int SubSubscriptionsCount(void)
{
    if (subsdb == 0)
    {
        // db open error
        return 0;
    }
	return DmNumRecords(subsdb);    
} /* SubSubscriptionsCount */


